#==============================================================================
# package      DE_EPAGES::Amazon::UI::AmazonConfig
# state        private
#==============================================================================
package DE_EPAGES::Amazon::UI::AmazonConfig;
use base DE_EPAGES::ProductPortal::UI::PortalConfig;

use strict;
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use DE_EPAGES::Core::API::PerlTools qw ( unique );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning);
use DE_EPAGES::Core::API::Log qw ( LogError LogDebug );
use DE_EPAGES::Amazon::API::Constants qw ( %AMAZON_ITEM_CONDITIONS %AMAZON_ONLINESTATUS );
use DE_EPAGES::Amazon::API::Object::RemoteAmazonMerchant;
use DE_EPAGES::Amazon::API::Object::RemoteMerchantToMWSAdapter;
use DE_EPAGES::Amazon::API::RemoteAdapterFactory;
use DE_EPAGES::Presentation::API::Constants qw ( PASSWORD_DUMMY );
use DE_EPAGES::Search::API::Object::PagerSearch qw ( ReplaceWildcards ReplaceWildcardsWithoutPosOne );
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);
use XML::Simple;

#==============================================================================
# Event handlers
#==============================================================================

sub RemoveProducts {
    my $self         = shift;
    my ($Servlet)    = @_;

    my $AmazonConfig = $Servlet->object;
    my $Form         = $Servlet->form;

    my $Shop    = $AmazonConfig->getSite();
    my $hValues = $Form->form( $AmazonConfig, 'ListedObjects' );

    my @ListObjects = map { $Shop->loadObject($_->{'ListObjectID'}, 'AmazonOffer') }
                      @{ $hValues->{'ListObjectIDs'} };

    $Servlet->testPermission( $_, 'Save' ) foreach @ListObjects;

    # delete only if accepted or no confirmation is set
    if ( $Servlet->user->get('DeleteConfirmation')
        and not $Servlet->params('ACCEPT') )
    {
        $Servlet->vars( 'DELETE', 1 );
        $Servlet->vars( 'DeleteObjects', [ map { $_->tleHash } @ListObjects ] );
        return;
    }
    else {
        $Servlet->vars( 'DELETE', 0 );
    }

    $AmazonConfig->removeOffers( \@ListObjects );
    return;
}

sub ExportPortalConfig {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::ExportPortalConfig( $Servlet, 0 );
    return;
}

sub AddProductsFromTray {
    my $self = shift;
    my ($Servlet) = @_;

    my $aWarnings = BlockWarnings(
        sub {
            eval { $self->SUPER::AddProductsFromTray($Servlet); };
            if ( ExistsError() ) {
                my $Error = GetError();
                LogError($Error) if ( $Error->code =~ /Native/ );
            }
        }
    );

    LogDebug( 'AddProduct Warnings', $aWarnings ) if ( scalar @$aWarnings );

    foreach my $warning (@$aWarnings) {
        if ( $warning->code eq 'NativeWarn' ) {
            Warning($warning);
        }
        else {
            $Servlet->form->mapFormError( {
                    'Form' => 'NewProduct',
                    'Name' => 'Tray'
                },
                $warning
            );
        }
    }

    $Servlet->form->executeFormError;
}

sub AddProduct {
    my $self      = shift;
    my ($Servlet) = @_;
    my $Object    = $Servlet->object;

    my $aWarnings = BlockWarnings(
        sub {
            eval { $self->SUPER::AddProduct( $Servlet, $Object ); };
            if ( ExistsError() ) {
                my $Error = GetError();
                LogError($Error) if ( $Error->code =~ /Native/ );
            }
        }
    );

    LogDebug( 'AddProduct Warnings', $aWarnings ) if ( scalar @$aWarnings );

    foreach my $warning (@$aWarnings) {
        if ( $warning->code eq 'NativeWarn' ) {
            Warning($warning);
        }
        else {
            $Servlet->form->mapFormError( {
                    'Form' => 'NewProduct',
                    'Name' => 'NewProduct'
                },
                $warning
            );
        }
    }

    $Servlet->form->executeFormError;
}

sub EndOffers {
    my $self         = shift;
    my ($Servlet)    = @_;

    my $PortalConfig = $Servlet->object;
    my $Form         = $Servlet->form;
    my $hValues      = $Form->form( $PortalConfig, 'ListedObjects' );
    my $Shop         = $PortalConfig->getSite();

    # Test existance of objects.
    my @ListObjects  = map { $Shop->loadObject($_->{'ListObjectID'}, 'AmazonOffer') }
                       @{ $hValues->{'ListObjectIDs'} };

    $Servlet->testPermission( $_, 'End' ) foreach @ListObjects;

    $PortalConfig->hideOffers( \@ListObjects );
}

sub ViewPagedChildren {
    my $self         = shift;
    my ($Servlet)    = @_;
    my $PortalConfig = $Servlet->object;

    return $self->SearchOffers( $Servlet, $PortalConfig, 1 );
}

sub SearchOffers {
    my $self = shift;
    my ( $Servlet, $IsList ) = @_;
    my $PortalConfig = $Servlet->object;

    $self->CountByStatus( $Servlet, $PortalConfig );

    my $SearchForm = $Servlet->form->form( $PortalConfig, 'SearchProducts' );
    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;
    my $LocalizedAttributeName = LoadClassByAlias('Product')->attribute('Name');
    my $SearchDefinition = GetSearchDefinition( LoadClassByAlias('AmazonOffer'), 'MBO-AmazonConfig' );

    my %Options = (
        'Servlet'          => $Servlet,
        'Object'           => $PortalConfig,
        'SearchDefinition' => $SearchDefinition,
        'Parameters'       => {
            'PortalConfig.ID'  => $PortalConfig->id,
            'SearchLanguageID' => $Servlet->languageID,
            'NameAttributeID'  => $LocalizedAttributeName->id,
        },
        'UriParameters' => {},
    );

    # if we have a ViewPagedChildren, then filter subproducts.
    $Options{'Parameters'}{'IsNotSubProduct'} = 1 if ( defined $IsList );

    foreach my $Parameter ( keys %$SearchForm ) {
        if ( $Parameter eq 'Alias' ) {
            ## UNITY-4143
            #replace '*' with '%' and add % to the end for (Alias/Productnumber)
            $Options{'Parameters'}{$Parameter} = ReplaceWildcardsWithoutPosOne($SearchForm->{$Parameter});
        }
        elsif ( $Parameter eq 'WildSearchString' ) {    #remove '*' at the end by '%' in (Alias/Productnumber)
            my $SearchString = $SearchForm->{$Parameter};
            $SearchString = ReplaceWildcards($SearchString);
            $Options{'Parameters'}{'Alias'} = $SearchString;
        }
        elsif ( $Parameter eq 'OnlineStatus' ) {
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
        $Options{'UriParameters'}{$Parameter} = $SearchForm->{$Parameter};
    }

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new( \%Options );
    $Search->statistic;
    $PortalConfig->updateBestPrice( $Search->items( $Search->get('Page') ) ) if $Search->get('CountOfPages') > 0;

    return $Search->page;
}

sub CountByStatus {
    my $self         = shift;
    my ($Servlet)    = @_;
    my $PortalConfig = $Servlet->object;

    my %hStatusCount = (
        'OnlineStatusCount_0' => 0,
        'OnlineStatusCount_1' => 0,
        'OnlineStatusCount_2' => 0,
        'OnlineStatusCount_4' => 0,
        'OnlineStatusCount_8' => 0,
    );

    map { $hStatusCount{ 'OnlineStatusCount_' . $_->get('OnlineStatus') }++; }
      @{ LoadClassByAlias('AmazonOffer')->getObjects( 'ByPortalConfigID', [ $PortalConfig->id ] ) };

    foreach my $key ( keys %hStatusCount ) {
        $Servlet->vars( $key, $hStatusCount{$key} );
    }
}

sub Save {
    my $self = shift;
    my ($Servlet) = @_;

    # fallback for SOAP.
    if ( defined($Servlet->params('DialogArea')) && $Servlet->params('DialogArea') eq 'SOAPArea' ) {
        $self->SaveSOAP($Servlet);
    }
    else {
        # default.
        $self->_saveMWS( $Servlet );
    }
    return;
}

sub SaveSOAP {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form         = $Servlet->form;
    my $PortalConfig = $Servlet->object;

    my $hFormValues = $Form->form( $PortalConfig, 'LoginData' );
    my $PortalSite = $PortalConfig->get('PortalSite');
    my $Remote;

    $hFormValues->{'MerchantPassword'} = $PortalConfig->get('MerchantPassword')
      if ( defined $hFormValues->{'MerchantPassword'} && $hFormValues->{'MerchantPassword'} eq PASSWORD_DUMMY );

    # test login data.
    $Remote = DE_EPAGES::Amazon::API::Object::RemoteAmazonMerchant->new(
        'merchant'   => $hFormValues->{'PortalMerchantID'},
        'username'   => $hFormValues->{'MerchantLogin'},
        'password'   => $hFormValues->{'MerchantPassword'},
        'url'        => $PortalSite->get('SOAPURL'),
        'is_logging' => $PortalSite->get('Portal')->get('Logging'),
    );

    $self->_activateAccount( $Remote, $PortalConfig, $Form, $hFormValues );
    return;
}

sub _saveMWS {
    my $self         = shift;
    my ($Servlet)    = @_;
    my $Form         = $Servlet->form;
    my $PortalConfig = $Servlet->object;
    my $hFormValues  = $Form->form( $PortalConfig, 'MWSLoginData' );
    # mws does not require contacting amazon support, so we can set ordercheck=1 by default
    $hFormValues->{'OrderCheck'} = 1;
    my $PortalSite   = $PortalConfig->get('PortalSite');

    my $hInfo = $PortalSite->get( [qw( Portal MWSURL MWSKeyID MWSSecretKeyID MWSMarketplaceID )] );

    my $Remote = DE_EPAGES::Amazon::API::Object::RemoteMerchantToMWSAdapter->new(
        'AWSURL'         => $hInfo->{'MWSURL'},
        'AWSAccessKeyId' => $hInfo->{'MWSKeyID'},
        'AWSSecretKeyId' => $hInfo->{'MWSSecretKeyID'},
        'Logging'        => $hInfo->{'Portal'}->get('Logging'),
        'MerchantID'     => $hFormValues->{'MWSMerchantID'},
        'MWSAuthToken'   => $hFormValues->{'MWSAuthToken'},
        'MarketplaceID'  => $hInfo->{'MWSMarketplaceID'},
    );

    $self->_activateAccount( $Remote, $PortalConfig, $Form, $hFormValues );
    return;
}

sub _activateAccount {
    my $self = shift;
    my ( $Remote, $PortalConfig, $Form, $hFormValues ) = @_;
    if ( $self->_isLoginDataOk( $Remote, $Form ) ) {
        $PortalConfig->set($hFormValues);
        $self->_activateOrderScheduler( $PortalConfig, $Form );
    }
    return;
}

sub _isLoginDataOk {
    my $self = shift;
    my ( $Remote, $Form ) = @_;
    eval { $Remote->login(); };
    if ( ExistsError() ) {
        my $Error = GetError();
        if ( $Error->code eq 'InvalidMerchantToken' ) {
            $Form->executeFormError( {
                    'Reason' => 'InvalidMerchantToken',
                    'Form'   => 'LoginData',
                    'Name'   => 'PortalMerchantID',
                    'Value'  => 'LoginFailed',
                }
            );
        }
        else {
            $Form->executeFormError( {
                    'Reason' => 'LoginFailed',
                    'Form'   => 'LoginData',
                    'Name'   => 'LoginData',
                    'Value'  => 'LoginFailed',
                }
            );
        }
    }
    else {
        return 1;
    }
}

sub _activateOrderScheduler {
    my $self = shift;
    my ( $PortalConfig, $Form ) = @_;
    eval {
        $PortalConfig->activateOrderScheduler('_15_MINUTES_');
    };
    if ( ExistsError() ) {
        my $Error = GetError();

        #EPG-25403 Remove MWSMerchantID, so attribute IsSettingsComplete returns false
        $PortalConfig->set({'MWSMerchantID' => ''}) if ( $Error->message =~ /Invalid seller id/ );
        Error($Error) if ( $Error->code =~ 'Native' );

        if ( $Error->message =~ /AuthToken is not valid/ ) {
            $PortalConfig->set({'MWSAuthToken' => ''});
            $Form->executeFormError({
                'Reason' => 'LoginMWSFailed',
                'Form'   => 'LoginData',
                'Name'   => 'MWSAuthToken'
            });
        }


        $Form->executeFormError({
            'Reason' => 'LoginMWSFailed',
            'Form'   => 'LoginData',
            'Name'   => 'MWSMerchantID'

        });
    }
}

sub SaveAmazonDefaultSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form         = $Servlet->form;
    my $PortalConfig = $Servlet->object;

    $Form->form( $PortalConfig, 'DefaultValues' );

    $self->SUPER::Save($Servlet);
}

sub UploadToAmazon {
    my $self = shift;
    my ($Servlet) = @_;

    my $PortalConfig = $Servlet->object;
    my $Form         = $Servlet->form;
    my $hValues      = $Form->form( $PortalConfig, 'ListedObjects' );
    my $Shop         = $PortalConfig->getSite();

    # Fetch objects.
    my @ListObjects  = map { $Shop->loadObject($_->{'ListObjectID'}, 'AmazonOffer') }
                       @{ $hValues->{'ListObjectIDs'} };

    my $aUploadOffers;
    my $aEANList = [];
    # filter products without EAN
    foreach my $Offer (@ListObjects) {

        # test if we have a variation master and add all child offers to the list
        if($Offer->hasChildren) {
            my $aSubOffers = $Offer->children();
            push @ListObjects,$_ foreach @$aSubOffers;
            next;
        }

        my $hInfoOffer = $Offer->get(['Product', 'ASIN', 'OnlineStatus']);

        # skip offers that are already online
        next if $hInfoOffer->{'OnlineStatus'}
            ~~ [$AMAZON_ONLINESTATUS{'OFFER_ONLINE'}, $AMAZON_ONLINESTATUS{'OFFER_PROCESSING'}];

        # test if we have an ASIN
        if (defined $hInfoOffer->{'ASIN'}) {
            push @$aUploadOffers, {'Offer' => $Offer };
            next;
        }

        # test if we have an UPCEAN
        my $EAN = $hInfoOffer->{'Product'}->getWithParent('UPCEAN');
        $EAN =~ s/-//g;

        if ($EAN) {
            # add to list for fetching ASIN later
            push @$aUploadOffers, {'Offer' => $Offer, 'EAN' => $EAN };
            push @$aEANList,$EAN;
        } else {
            $Servlet->form->addFormError({
                'Reason'=> 'OFFER_PRODUCT_NO_UPCEAN',
                'Name'  => $hInfoOffer->{'Product'}->alias,
                'NameOrAlias' => $hInfoOffer->{'Product'}->get('NameOrAlias'),
                'Product' => $hInfoOffer->{'Product'}->tleHash,
                'ObjectID' => $Offer->id,
            });
        }
    }

    # remove duplicate entries from EAN list
    @$aEANList = unique @$aEANList;

    my $ahProductsForIDs = [];
    if(scalar(@$aEANList)){
        # call MWS for mapping to ASIN
        my $Remote = DE_EPAGES::Amazon::API::RemoteAdapterFactory::NewRemoteMWSProduct( $PortalConfig );
        $ahProductsForIDs = $Remote->GetMatchingProductForId( 'EANs' => $aEANList, 'IdType' => 'EAN' );
    }

    # filter ASINs
    my $ASINMatches;
    foreach my $ResponseElement (@$ahProductsForIDs){

            $ASINMatches->{$ResponseElement->{'Id'}} = [];
            next unless $ResponseElement->{'status'} eq 'Success';
            my $aProducts = $ResponseElement->{'Products'}->{'Product'};

            # we need an array also if we have only one element (JSON problem)
            $aProducts = ref $aProducts eq 'ARRAY' ? $aProducts : [$aProducts];

            foreach my $hProduct (@$aProducts){
                push @{$ASINMatches->{$ResponseElement->{'Id'}}},
                    $hProduct->{'Identifiers'}->{'MarketplaceASIN'}->{'ASIN'};
            }
    }

    foreach my $hOffer (@$aUploadOffers) {

        my $hInfoOffer = $hOffer->{'Offer'}->get(['Product', 'AmazonPrice', 'DeliveryPeriod', 'ASIN']);
        my $HasASIN = 0;

        if ($hInfoOffer->{'ASIN'}) {
            $HasASIN = 1;
        } else {
            # add ASIN if we had none so far
            # test for one or more ASINs
            # only if we have one ASIN we start the offer
            # otherwise we have to do the matching manually
            my $aASINs = $ASINMatches->{$hOffer->{'EAN'}};

            # add form error if we have not only exactly one ASIN match for this offer product
            if (scalar(@$aASINs) != 1) {
                $Servlet->form->addFormError({
                    'Reason'=> scalar(@$aASINs) > 1 ? 'OFFER_PRODUCT_TOO_MANY_ASINS' : 'OFFER_PRODUCT_NO_ASIN',
                    'Name'  => $hOffer->{'Offer'}->get('Product')->alias,
                    'NameOrAlias' => $hOffer->{'Offer'}->get('Product')->get('NameOrAlias'),
                    'Product' => $hInfoOffer->{'Product'}->tleHash,
                    'ObjectID' => $hOffer->{'Offer'}->id,
                });
            } else {
                # set ASIN
                $hOffer->{'Offer'}->set({'ASIN' => $aASINs->[0]});
                $HasASIN = 1;
            }
        }

        my $ExistsPrice          = $hInfoOffer->{'AmazonPrice'} ? 1:0;
        my $ExistsDeliveryPeriod = $hInfoOffer->{'DeliveryPeriod'} ? 1:0;
        my $ExistsCondition      = defined $hInfoOffer->{'Product'}->getWithParent('AmazonCondition') ? 1:0;

        $Servlet->form->addFormError({
            'Reason'=> 'OFFER_PRODUCT_NO_PRICE',
            'Name'  => $hInfoOffer->{'Product'}->alias,
            'NameOrAlias' => $hInfoOffer->{'Product'}->get('NameOrAlias'),
            'Product' => $hInfoOffer->{'Product'}->tleHash,
            'ObjectID' => $hOffer->{'Offer'}->id,
        }) unless $ExistsPrice;

        $Servlet->form->addFormError({
            'Reason'=> 'OFFER_PRODUCT_NO_DELIVERY_PERIOD',
            'Name'  => $hInfoOffer->{'Product'}->alias,
            'NameOrAlias' => $hInfoOffer->{'Product'}->get('NameOrAlias'),
            'Product' => $hInfoOffer->{'Product'}->tleHash,
            'ObjectID' => $hOffer->{'Offer'}->id,
        }) unless $ExistsDeliveryPeriod;

        $Servlet->form->addFormError({
            'Reason'=> 'OFFER_PRODUCT_NO_CONDITION',
            'Name'  => $hInfoOffer->{'Product'}->alias,
            'NameOrAlias' => $hInfoOffer->{'Product'}->get('NameOrAlias'),
            'Product' => $hInfoOffer->{'Product'}->tleHash,
            'ObjectID' => $hOffer->{'Offer'}->id,
        }) unless $ExistsCondition;

        if($HasASIN and $ExistsPrice and $ExistsDeliveryPeriod and $ExistsCondition) {
            # upload offer to Amazon
            $hOffer->{'Offer'}->startOffer();
        }
    }

    $Servlet->form->executeFormError();

    return;
}

1;
