#========================================================================================
# package      DE_EPAGES::Calendar::API::TLE::CalendarHandler
# state        public
#========================================================================================
package DE_EPAGES::Calendar::API::TLE::CalendarHandler;

use strict;
use DateTime;
use DE_EPAGES::Core::API::Log qw (GetLog);
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);

my $NOON = DateTime->new(
    year   => 2000,
    month  => 1,
    day    => 1,
    hour   => 12,
    minute => 0,
    second => 0,
    nanosecond => 0
);

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               <\ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'CalendarHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Calendar::API::TLE::CalendarHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Calendar::API::TLE::CalendarHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'CalendarHandler.register' );

    $Processor->registerHandler('FunctionHandler', $self, 'BusinessHourWeek');
    return;
}

#========================================================================================
# function     BusinessHourWeek
# state        public
#----------------------------------------------------------------------------------------
# syntax       #BusinessHourWeek(#Calendar, #IsOddWeek)
# example      #LOOP(#BusinessHourWeek(#Calendar.Object, #IsOddWeek))
#                 #Abbreviation:
#                 #IF(#OpenDuskTillDawn)
#                   Open 24 hours.
#                 #ELSE
#                   We're open from #OpenMorning[time] to #CloseMorning[time] in the morning
#                   anf from #OpenAfternoon[time] to #CloseAfternoon[time] in the afternoon.
#                 #ENDIF
#                 <br />
#               #ENDLOOP
#----------------------------------------------------------------------------------------
# description  Loops all 7 days of the week and provides information about opening
#               and closing hours of each day.
#               <ul>
#                   <li>Name - localized name of the day, e.g. 'Monday'</li>
#                   <li>Abbreviation - short version of localized name of the day, e.g. 'Mon'</li>
#                   <li>WeekDayID - number 0..6</li>
#                   <li>OpenDuskTillDawn - open 24 hours</li>
#                   <li>OpenMorning - opening time in the morning</li>
#                   <li>CloseMorning - closing time in the morning</li>
#                   <li>OpenAfternoon - opening time in the afternoon</li>
#                   <li>CloseAfternoon - closing time in the afternoon</li>
#               </ul>
#----------------------------------------------------------------------------------------
# input        #Calendar | calendar object | DE_EPAGES::Calendar::API::Object::Calendar
# input        #IsOddWeek | (optional, default=0) true if the week number is odd | boolean
#========================================================================================
sub BusinessHourWeek {
    my $self = shift;
    my ($Processor, $aParameter) = @_;
    my $Calendar = $aParameter->[0];
    my $IsOddWeek = $aParameter->[1];
    $IsOddWeek = 0 unless defined $IsOddWeek;

    my $BaseCalendar = $Calendar->get('BaseCalendar');
    $Calendar = $BaseCalendar if defined $BaseCalendar && !$Calendar->get('HasOwnBookingPeriod') ;

    my $BookingPeriodClass = LoadClassByAlias('BookingPeriod');
    my $dbData = $BookingPeriodClass->getObjects('ByCalendarIDIsOddWeek', [$Calendar->id, $IsOddWeek]);
    my $Shop = $Calendar->getSite();
    my $LocaleID = $Shop->get('LocaleID');
    my $cResult = $Calendar->getSite()->siteLocale($LocaleID)->get('Locale')->{'DayNames'};
    my $aDays = $cResult->();
    foreach my $Day (@$aDays) {
        my @dbData  = grep { $_->get('WeekDay') == $Day->{'WeekDayID'} } @$dbData;
        $Day->{'OpenMorning'}=undef;
        $Day->{'CloseMorning'}=undef;
        $Day->{'OpenAfternoon'}=undef;
        $Day->{'CloseAfternoon'}=undef;
        if (scalar @dbData == 1) {
            $Day->{$dbData[0]->get('OpeningTime') <= $NOON ? 'OpenMorning'  : 'OpenAfternoon'} =$dbData[0]->get('OpeningTime');
            $Day->{$dbData[0]->get('ClosingTime') <= $NOON ? 'CloseMorning' : 'CloseAfternoon'}=$dbData[0]->get('ClosingTime');
        }
        elsif (scalar @dbData == 2) {
            $Day->{'OpenMorning'}=$dbData[0]->get('OpeningTime');
            $Day->{'CloseMorning'}=$dbData[0]->get('ClosingTime');
            $Day->{'OpenAfternoon'}=$dbData[1]->get('OpeningTime');
            $Day->{'CloseAfternoon'}=$dbData[1]->get('ClosingTime');
        }

        if(defined $self->_checkOpenCloseHour($Day->{'OpenMorning'})
            && !$self->_checkOpenCloseHour($Day->{'CloseMorning'})
            && !$self->_checkOpenCloseHour($Day->{'OpenAfternoon'})
            && defined $self->_checkOpenCloseHour($Day->{'CloseAfternoon'})
            && $self->_isTwentyFourHours($Day->{'OpenMorning'}, $Day->{'CloseAfternoon'})
        ) {
            $Day->{'OpenDuskTillDawn'} = 1;
        }
        else {
            $Day->{'OpenDuskTillDawn'} = 0;
        }
    }
    return $aDays;

}

sub _checkOpenCloseHour{
    my $self = shift;
    my ($DaySection) = @_;
    if(defined $DaySection && defined $DaySection->{'local_c'} && defined $DaySection->{'local_c'}->{'hour'}){
        return $DaySection->{'local_c'}->{'hour'};
    }
    return undef;
}

sub _isTwentyFourHours {
    my $self = shift;
    my ($DaySectionOpen, $DaySectionClose) = @_;
    if(defined $DaySectionOpen  && defined $DaySectionOpen->{'local_c'}  && defined $DaySectionOpen->{'local_c'}->{'hour'} &&
       defined $DaySectionClose && defined $DaySectionClose->{'local_c'} && defined $DaySectionClose->{'local_c'}->{'hour'} &&
       $DaySectionOpen->{'local_c'}->{'hour'} == $DaySectionClose->{'local_c'}->{'hour'} ){
        return 1;
    }
    return 0;
}

1;
