#========================================================================================
# package      DE_EPAGES::Customer::Hooks::Address
# state        public
# description  Sets the referenced BillingAddress undef if the address object is removed.
#========================================================================================
package DE_EPAGES::Customer::Hooks::Address;

use strict;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject );
use DE_EPAGES::Customer::API::Shop qw( GetCustomerForUser );

#========================================================================================
# function     OnDeleteAddress
#----------------------------------------------------------------------------------------
# syntax       OnDeleteAddress( $hParams );
#----------------------------------------------------------------------------------------
# description  Set billing address of a customer undef if the referenced address is deleted.
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of object Address, key is
#               <ul>
#                   <li>AddressID
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteAddress {
    my ($hParams) = @_;
    $_->set({'BillingAddress'=>undef}) foreach @{LoadClassByAlias('Customer')->getObjects('ByBillingAddressID', [ $hParams->{'AddressID'} ])};
}

#========================================================================================
# function     OnAfterUpdateAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateAddress( $hParams );
#----------------------------------------------------------------------------------------
# description  OnAfterUpdateAddress is triggered, to set name of user.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                   <li>AddressID
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateAddress {
    my ($hParams) = @_;

    my $AddressID = $hParams->{'AddressID'};
    my $Address = LoadObject($AddressID);

    # do the following steps only for registerd users
    # and if this users BillingAddress is changed
    my $Parent = $Address->parent->parent;
    return unless defined $Parent;
    return unless $Parent->instanceOf('User');
    return if $Parent->get('IsAnonymous');
    my $BillingAddress = $Parent->get('BillingAddress');
    return if !defined $BillingAddress || $AddressID != $BillingAddress->id;

    # set Name for user if users BillingAddress Name has changed
    my $Name = $Address->get('DisplayName');
    unless ($Name) {
        $Name = $Address->get('FullName');
    }
    $Parent->set({ 'Name' => $Name }) if $Name && $Name ne $Parent->get('Name');

    # Only for registered users and if we have unity we update the BillingAddress on the Customer 
    # (we have only a BillingAddress at the Customer) because we have no address sync for unity. 
    # So we can be sure all addresses are snchronized.
    if( $Address->getSite()->get('IsUnityRegistered') and scalar(@{$Parent->get('Customers')}) )  {
        my $Customer = $Parent->get('Customers')->[0];
        my $AddressFolder = $Customer->folder('Addresses');
        my $NewAlias = $AddressFolder->nextAlias('BillingAddress %s');

        # clone the billing address and use this for the customers BillingAddress
        my $NewBillingAddress = $BillingAddress->clone({ 'Alias' => $NewAlias, 'Parent' => $AddressFolder });
        my $CustomerBillingAddress = $Customer->get('BillingAddress');
        $Customer->set({'BillingAddress' => $NewBillingAddress});
        # delete old address
        $CustomerBillingAddress->delete;

    }

}

#========================================================================================
# function     _UserFromAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       $User = _UserFromAddress( $Address );
#----------------------------------------------------------------------------------------
# description  Returns the user associated with a given address or undef if the
#               address does not belong to a user.
#               Note that addresses may be direct child object or may live in an
#               'Addresses' folder.
#----------------------------------------------------------------------------------------
# input        $Address | address | object
# input        $User | user or undef | object
#========================================================================================
sub _UserFromAddress {
    my ($Address) = @_;

    my $Parent = $Address->parent;
    return unless defined $Parent;
    return $Parent if $Parent->instanceOf('User');

    $Parent = $Parent->parent;
    return unless defined $Parent;
    return $Parent if $Parent->instanceOf('User');
    return;
}

#========================================================================================
# function     _CustomerFromAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Customer = _CustomerFromAddress( $Address );
#----------------------------------------------------------------------------------------
# description  Returns the customer associated with a given address or undef if the
#               address does not belong to a customer.
#               Note that addresses may be direct child object or may live in an
#               'Addresses' folder.
#----------------------------------------------------------------------------------------
# input        $Address | address | object
# input        $Customer | customer or undef | object
#========================================================================================
sub _CustomerFromAddress {
    my ($Address) = @_;

    my $Parent = $Address->parent;
    return unless defined $Parent;
    return $Parent if $Parent->instanceOf('Customer');

    $Parent = $Parent->parent;
    return unless defined $Parent;
    return $Parent if $Parent->instanceOf('Customer');
    return;
}

#========================================================================================
# function     OnBeforeUpdateAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateAddress( $hParams );
#----------------------------------------------------------------------------------------
# description  OnBeforeUpdateAddress is triggered, test if bank account data are changed.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                   <li>AddressID
#                   <li>Values
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateAddress {
    my ($hParams) = @_;

    my $Address = $hParams->{'Object'};
    my $hValues = $hParams->{'Values'};

    my $Customer = _CustomerFromAddress($Address);

    #set LastUpdate if any related attribute is touched
    _UpdateProfiles($Customer, $Address, $hValues) if defined $Customer;

    # test, if the customer/merchant has changed the bank account data
    # if so, set the attribute IsMandateIDValid to 0
    my $User = _UserFromAddress($Address);
    return unless defined $User;
    $Customer = GetCustomerForUser($User,$User->getSite());
    return unless defined $Customer;

    foreach my $key ( qw(BankAccountHolder BankCode BankName BankAccountNo) ){
        next unless exists $hValues->{$key};
        next unless $hValues->{$key};
        my $ChangeValue = $hValues->{$key} // '';
        my $AddressValue = $Address->get($key) // '';
        if ($ChangeValue ne $AddressValue){
            $Customer->set({'IsMandateIDValid' => 0});
            last;
        }
    }

    return;
}

#set LastUpdate if any related attribute is touched
sub _UpdateProfiles {
    my ($Customer, $Address, $hValues) = @_;

    my $AddressProfile = LoadClassByAlias('Customer')->lastUpdateProfile('Address');
    $Customer->setUpdated( $AddressProfile );
}

1;
