#========================================================================================
# package      DE_EPAGES::DHL::API::Cartridge
# base         DE_EPAGES::Shipping::API::ShippingInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::DHL::API::Cartridge;
use base DE_EPAGES::Shipping::API::ShippingInstaller;

use strict;
use DE_EPAGES::Object::API::Factory qw (
        LoadObject LoadObjectByPath ExistsObjectByPath LoadClassByAlias ExistsClassByAlias );
use DE_EPAGES::Core::API::PerlTools qw ( GetLoopFromDoubleArray );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Database::API::Connection qw ( Transaction );
use DE_EPAGES::DHL::API::XML::DHLProductHandler qw ( ParseProductFile );
use DE_EPAGES::Core::API::File qw ( ExistsFile DeleteFile );
use FileHandle;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::DHL::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::ShippingType.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/DHL',
        'Version'            => '7.17.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0'],
#       'HasDbSchema'        => 0,
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;
    $self->SUPER::install;

    $self->installDHLProductList();
    $self->finishInstall;
    return;
}

#========================================================================================
# function     uninstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

#========================================================================================
# function     installDHLProductList
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->installDHLProductList();
#----------------------------------------------------------------------------------------
# description  Imports DHL/Data/DHLProductList/DHLProductList.xml
#========================================================================================
sub installDHLProductList {
    my $self = shift;
    #Import DHL Productlist
    GetLog()->info('run installDHLProductList');
    my $fh = FileHandle->new($ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES/DHL/Data/DHLProductList/DHLProductList.xml', 'r');
    ParseProductFile($fh, 1, 'IMPORT');
    if (defined $fh) {
        $fh->close;
    }
    return;
}

#========================================================================================
# function     patch_6_16_0
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->patch_6_16_0;
#----------------------------------------------------------------------------------------
# description  EPG-29236 - Patch for EPG-16641: DHL: Create patch for EPG-16641
# return       $newVersion | 6.16.1 | string
#========================================================================================
sub patch_6_16_0 {
    my $self = shift;

    $self->logInfo('import non conflicting (no loss of data) DHL DB changes');

    #apply data changes from patch.xml file
    $self->importPatchDir('Patch_6_16_0', 'Patch');

    #no need to move PBO paramenters for DHL because the data structure was not moved to
    #ShippingFulfillmentTypeDHL (from System).

    #check that required classes and objects are in place
    return '6.16.1' unless ( _patch_6_16_0_IsReadyToMigrateData($self) );

    #get hash of object and id references that will be used later on
    my $hRefData = _patch_6_16_0_GetGeneralReferencesForDataMigration($self);

    $self->logInfo('convert data from ShippingMethodDHL to ShippingMethodWeightOfBasket/ShippingFulfillmentMethodDHL');

    my $dbi = $self->getCurrentDBHandle();

    my $aaResultSet = $dbi->execute(<<ENDSQL, [$hRefData->{'ShipTypeDhlObjectId'}], ['Integer', 'Integer']);
SELECT sm.shippingmethodid, sm.shopid FROM shippingmethod sm WHERE shippingtypeid = ?
ENDSQL
    my $ahResultSet = GetLoopFromDoubleArray($aaResultSet, [qw ( ShipMethodID ShopID )]);

    Transaction ( sub {
        foreach my $hResult (@$ahResultSet) {
            my $Shop = LoadObject($hResult->{'ShopID'});
            my $ShippingMethodDHL = LoadObject($hResult->{'ShipMethodID'});

            $self->logInfo('processing Shop: ' . $Shop->get('Path') . '. Shipping: ' . $ShippingMethodDHL->get('Path'));

            # generate ShippingFulfillmentMethodDHL instance
            my $SfmDHL =_patch_6_16_0_GenerateShippingFulfillmentMethod($hRefData, $Shop, $ShippingMethodDHL);
            $self->logInfo('created  ' . $SfmDHL->alias . ' ShippingFulfillmentMethodDHL object');

            # convert ShippingMethodDHL into ShippingMethodWeightOfBasket object instance
            $self->logInfo('convert ' . $ShippingMethodDHL->alias . ' into ShippingMethodWeightOfBasket object');
            _patch_6_16_0_ConvertShippingMethod($hRefData, $dbi, $ShippingMethodDHL, $SfmDHL);
        }

        # remove old structures not needed any more
        _patch_6_16_0_RemoveObsoletedStructures ($hRefData, $self);

        # remove pending file(s) from old structure
        _patch_6_16_0_RemovePendingFiles($self);
    });

    $self->logInfo('patch to 6.16.1 is finished');

    return '6.16.1';
}

#-----------------------------------------------
sub _patch_6_16_0_IsReadyToMigrateData {
    my ($Installer) = @_;

    unless (ExistsClassByAlias('ShippingMethodDHL')) {
        $Installer->logInfo('ShippingMethodDHL class doesn\'t exist. Exit');
        return 0;
    }
    unless (ExistsClassByAlias('ShippingFulfillmentMethodDHL')) {
        $Installer->logInfo('ShippingFulfillmentMethodDHL class doesn\'t exist. Exit');
        return 0;
    }
    unless (ExistsObjectByPath('/ShippingTypes/DHL')) {
        $Installer->logInfo('/ShippingTypes/DHL object doesn\'t exist. Exit');
        return 0;
    }
    unless (ExistsObjectByPath('/ShippingFulfillmentTypes/DHL')) {
        $Installer->logInfo('/ShippingFulfillmentTypes/DHL object doesn\'t exist. Exit');
        return 0;
    }
    return 1;
}

#-----------------------------------------------
sub _patch_6_16_0_GetGeneralReferencesForDataMigration {
    my ($Installer) = @_;

    my $ShipTypeDhlObject = LoadObjectByPath('/ShippingTypes/DHL');

    my $hReferences = {
        'ShipTypeDhlObject'           => $ShipTypeDhlObject,
        'ShipTypeDhlObjectId'         => $ShipTypeDhlObject->id,
        'ShipTypeWeightObjectId'      => LoadObjectByPath('/ShippingTypes/WeightOfBasket')->id,
        'ShipFulTypeDhlObject'        => LoadObjectByPath('/ShippingFulfillmentTypes/DHL'),
        'ShipMethodWeightClassId'     => $Installer->getClassID('ShippingMethodWeightOfBasket'),
        'ShipMethodDhlClass'          => LoadClassByAlias('ShippingMethodDHL'),
        'ShipFulMethodDhlClass'       => LoadClassByAlias('ShippingFulfillmentMethodDHL'),
        'ShipMethodWeightWUnitAttrId' => LoadObjectByPath('/Classes/ShippingMethodWeightOfBasket/Attributes/WeightUnit')->id,
        'ShipMethodDhlWUnitAttrId'    => LoadObjectByPath('/Classes/ShippingMethodDHL/Attributes/WeightUnit')->id,
    };

    return $hReferences;
}

#-----------------------------------------------
sub _patch_6_16_0_GenerateShippingFulfillmentMethod {
    my ($hRef, $Shop, $ShippingMethodDHL) = @_;

    my $sfmPath = $Shop->get('Path') . '/ShippingFulfillmentMethods';
    my $sfMethodsObj;
    if ( ExistsObjectByPath($sfmPath) ) {
        $sfMethodsObj = LoadObjectByPath($sfmPath);
    } else {
        $sfMethodsObj = $Shop;
    }
    my $Site = $Shop->getSite();

    my $hSfmDHLinfo = {
        'Alias' => 'DHL',
        'DHLPaymentMethod' => $ShippingMethodDHL->get('DHLPaymentMethod'),
        'AcceptTOS' => $ShippingMethodDHL->get('AcceptTOS'),
        'AcceptSendUserEmail' => $ShippingMethodDHL->get('AcceptSendUserEmail'),
        'AutoGenerateLabels' => 0,
        'Inherit' => 1,
        'IsInTestMode' => 0,
        'ObjectID' => undef,
        'Parent' => $sfMethodsObj,
        'Position' => '0',
        'ShippingFulfillmentType' => $hRef->{'ShipFulTypeDhlObject'},
        'Shop' => $Shop,
        'Site' => $Site,
    };

    return $hRef->{'ShipFulMethodDhlClass'}->insertObject($hSfmDHLinfo);
}

#-----------------------------------------------
sub _patch_6_16_0_ConvertShippingMethod {
    my ($hRef, $dbi, $ShippingMethodDHL, $SfmDHL) = @_;

    #move object class
    $ShippingMethodDHL->set({'SelectedShippingFulfillmentMethod' => $SfmDHL});
    $dbi->do(<<END_SQL, [$hRef->{'ShipMethodWeightClassId'}, $ShippingMethodDHL->id]);
UPDATE object
SET classid = ?
WHERE objectid = ?
END_SQL

    #move shippingtype for the shipping method instance
    $dbi->do(<<END_SQL, [$hRef->{'ShipTypeWeightObjectId'}, $ShippingMethodDHL->id]);
UPDATE shippingmethod
SET shippingtypeid = ?
WHERE shippingmethodid = ?
END_SQL

    #arrange attribute values (WeightUnit)
    $dbi->do(<<END_SQL, [$hRef->{'ShipMethodWeightWUnitAttrId'}, $hRef->{'ShipMethodDhlWUnitAttrId'}, $ShippingMethodDHL->id]);
UPDATE referenceattribute
SET attributeid = ?
WHERE attributeid = ? AND objectid = ?
END_SQL

    return;
}

#---------------------------------------------------
sub _patch_6_16_0_RemoveObsoletedStructures {
    my ($hRef, $Installer) = @_;

    #remove old DHL shipping type
    $Installer->logInfo('deleting ' . $hRef->{'ShipTypeDhlObject'}->alias . ' shipping type object ');
    $hRef->{'ShipTypeDhlObject'}->delete();

    #remove old DHL class
    $Installer->logInfo('deleting ' . $hRef->{'ShipMethodDhlClass'}->alias . ' class ');
    $hRef->{'ShipMethodDhlClass'}->delete();

    return;
}

sub _patch_6_16_0_RemovePendingFiles {
    my ($Installer) = @_;

    my $file = $ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES/DHL/API/Object/ShippingMethodDHL.pm';
    if ( ExistsFile($file) ) {
        $Installer->logInfo('deleting file ' . $file);
        DeleteFile($file);
    }
    return;
}
#----------------------------------------------------------

#========================================================================================
# function     patch_6_16_2
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Installer->patch_6_16_2;
#----------------------------------------------------------------------------------------
# description  Patch for EPG-30386
# return       $newVersion | 6.16.3 | string
#========================================================================================
sub patch_6_16_2 {
    my $self = shift;

    $self->importPatchDir('Patch_6_16_1', 'Patch');

    my $dbi = $self->getCurrentDBHandle();

    $self->logInfo( 'Get ShippingLabel and DHLShippingLabel classid' );
    my $shippingLabelClassId = $self->getClassID('ShippingLabel');
    my $DHLShippingLabelClassId = $self->getClassID('DHLShippingLabel');

    $self->logInfo( 'Change DHLShippingLabel base class to Shipping Label' );
    $dbi->do(<<END_SQL, [$shippingLabelClassId, $DHLShippingLabelClassId]);
UPDATE class
SET baseclassid = ?
WHERE classid = ?;
END_SQL

    return '6.16.3';
}

1;
