#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Receiver
# state        public
# description  Receiver element for the DHL Versenden shipping label request builder
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Receiver;

use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Communication;
use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::AddressReceiver;
use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Packstation;

use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::LeafNodeHelper;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Receiver = DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Receiver->new();
#----------------------------------------------------------------------------------------
# description  Object contructor.
#----------------------------------------------------------------------------------------
# return       $Receiver | DHL Versenden receiver element object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {}, $class;

    return $self;
}

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hReceiver = $self->assemble();
#----------------------------------------------------------------------------------------
# description  Create the hash with receiver data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | DHL Versenden shipping label object | object
# return       $hReceiver | DHL Versenden receiver element | ref.hash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $CustomerAddress = $ShippingLabel->get('Address');

    my $LeafNodeHelper = DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::LeafNodeHelper->new();
    $LeafNodeHelper->addLeaf('name1', $CustomerAddress->get('FullName'));

    my $hReceiver = { 
        'name1' => $LeafNodeHelper->getLeaf('name1')
    };

    my $hCommunication = $self->_assembleCommunication($CustomerAddress);
    $hReceiver = { %{$hReceiver}, %{$hCommunication->{'nodeData'}} };

    my @Warnings = @{$LeafNodeHelper->getWarnings()};
    push(@Warnings, @{$hCommunication->{'nodeWarnings'}});

    if ( defined($CustomerAddress->get('DHLPackstationId')) ) {
        my $hPackstation = $self->_assemblePackstation($CustomerAddress);

        $hReceiver = { %{$hReceiver}, %{$hPackstation->{'nodeData'}} };
        push(@Warnings, @{$hPackstation->{'nodeWarnings'}});
    } else {
        my $hAddress = $self->_assembleAddressReceiver($CustomerAddress);

        $hReceiver = { %{$hReceiver}, %{$hAddress->{'nodeData'}} };
        push(@Warnings, @{$hAddress->{'nodeWarnings'}});
    }

    return {
        'nodeData' => {
            'Receiver' => {
                %{$hReceiver},
                # Postfiliate not used
                # ParcelShop not used
            }
        },
        'nodeWarnings' => \@Warnings
    };
}

#========================================================================================
# Local methods
#========================================================================================
sub _assembleCommunication {
    my $self = shift;
    my ($Address) = @_;

    return DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Communication->assemble($Address);
}

sub _assembleAddressReceiver {
    my $self = shift;
    my ($Address) = @_;

    return DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::AddressReceiver->assemble($Address);
}

sub _assemblePackstation {
    my $self = shift;
    my ($Address) = @_;

    return DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Packstation->assemble($Address);
}

1;