#==============================================================================
# package      DE_EPAGES::ExternalShipping::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::ExternalShipping::UI::Shop;
use base qw (DE_EPAGES::Order::UI::Shop Exporter);

use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools qw( AddDuration );
use DE_EPAGES::ExternalShipping::API::Shop qw (GetShippingFulfillmentMethodsFolder GetCurrentShippingOrder );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObjectByPath JoinObjectPath LoadRootObject );
use DE_EPAGES::Search::API::Object::PagerSearch qw (ReplaceWildcards ReplaceWildcardsWithoutPosOne);
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);

our @EXPORT_OK = qw (
    NewShippingFulfillmentMethod
) ;

#========================================================================================
# function     NewShippingFulfillmentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->NewShippingFulfillmentMethod( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction to create a shipping fulfillment method
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $ShippingFulfillmentMethod | ShippingFulfillmentMethod | object
#========================================================================================
sub NewShippingFulfillmentMethod {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $ShippingFulfillmentMethod = undef ;

    SaveShippingFulfillmentMethods($self,$Servlet);

    if ( $Servlet->params('NewAlias') =~ /\S/ ) {

        my $Form = $Servlet->form->form($Shop, 'NewShippingFulfillmentMethod');
        my $ShippingFulfillmentType = LoadRootObject->loadObject($Form->{'NewShippingFulfillmentTypeID'}, 'ShippingFulfillmentType');
        delete $Form->{'NewShippingFulfillmentTypeID'};

        my %Form;
        $Form{$_}= $Form->{'New'.$_} foreach map { s/^New// && $_} grep { /^New/ } keys %{$Form};

        # test if Method with the given name exists
        my $Folder = GetShippingFulfillmentMethodsFolder($Shop);

        if ($Form{'Alias'} && $Folder->existsChild($Form{'Alias'})){
            $Servlet->vars('OBJECT_EXISTS',1);
            $Servlet->form->executeFormError({
                'Reason'=> 'OBJECT_EXISTS',
                'Name'  => 'NewAlias',
                'Value' => $Form{'Alias'},
            }) ;
        }

        eval {
            $ShippingFulfillmentMethod = $ShippingFulfillmentType->newShippingFulfillmentMethod($Shop, \%Form);

            return unless defined $ShippingFulfillmentMethod;
         };

        if (ExistsError()) {
            my $Error = GetError();
            if ($Error->code eq 'NoSecondInstanceAllowed') {
                $Servlet->vars('NoSecondInstanceAllowed',1);
            }
            else {
                Error($Error);
            }
        }

        $Folder->renumberChildren;
    }

    return $ShippingFulfillmentMethod ;
}

#========================================================================================
# function     DeleteShippingFulfillmentMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->DeleteShippingFulfillmentMethods( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction to delete shipping fulfillment methods
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub DeleteShippingFulfillmentMethods {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    $self->SUPER::DeleteListedObjects($Servlet);
    GetShippingFulfillmentMethodsFolder($Shop)->renumberChildren;

    return;

}

#========================================================================================
# function     SaveShippingFulfillmentMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SaveShippingFulfillmentMethods( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction to save positions from shipping fulfillment method list
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SaveShippingFulfillmentMethods {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $ahListItems = $self->listItems( $Servlet, $Shop, 'ShippingFulfillmentMethod' );

    # save position
    foreach my $hListItem (@$ahListItems) {
        my $Method = $hListItem->{'Object'};
        $Method->set({
            'Position'    => $hListItem->{'Position'},
        });
    }
    GetShippingFulfillmentMethodsFolder($Shop)->renumberChildren;
    return;
}

#========================================================================================
# function     _ExecuteTransactionSearch
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->_ExecuteTransactionSearch( $Servlet, $SearchForm);
#----------------------------------------------------------------------------------------
# description  Search and Pager for functions <function ViewTransactions> and <function SearchTransactions>.
#               Overwrites the implementation from DE_EPAGES::Order::UI::Shop
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $SearchForm | form values | ref.hash.string
# input        $SearchClass | class for search definition | object
# return       $Page | Pager->page | ref.hash
#========================================================================================
sub _ExecuteTransactionSearch { ## no critic (ProhibitUnusedPrivateSubroutines)
    my $self = shift;
    my ($Servlet, $SearchForm, $SearchClass) = @_;

    # needed servlet vars.
    my $Shop = $Servlet->object;
    my $ShippingFulfillmentType = $self->getShippingFulfillmentType();
    $Servlet->vars('ShippingFulfillmentType',$ShippingFulfillmentType->tleHash);
    $Servlet->vars('ShippingOrder',GetCurrentShippingOrder($Shop, $ShippingFulfillmentType)->tleHash);

    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %{$SearchForm};
    $SearchForm->{'Search'} = 'Transactions' unless exists $SearchForm->{'Search'};
    my $SearchDefinition = GetSearchDefinition(
        $SearchClass ? $SearchClass
                    : LoadClassByAlias('ShippingTransaction' . $self->TransactionPaymentName()
    ), $SearchForm->{'Search'});

    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Shop,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'LanguageID' => $Servlet->languageID,
        },
        'UriParameters'  => {
            'Search' => $SearchForm->{'Search'},
        },
    );

    my $hSearchOptions = $self->SearchOptions($SearchForm->{'Search'});
    $Options{'Parameters'}{$_} = $hSearchOptions->{$_} foreach keys %{$hSearchOptions};

    delete $SearchForm->{'Search'};
    foreach my $Parameter (keys %{$SearchForm}) {
        if ( $Parameter eq 'ProductAliasString' ) {

            ## UNITY-4143
            #replace '*' with '%' and add % to the end for (Alias/Productnumber)
            $Options{'Parameters'}{$Parameter} = ReplaceWildcardsWithoutPosOne($SearchForm->{$Parameter});
        } elsif ($Parameter =~ /String/) {
            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards($SearchForm->{$Parameter});

        } elsif ($Parameter =~ /(.*)On$/x) {
            # status = 0/1/2 egal/yes/no
            my $Status = $1;
            if ($SearchForm->{$Parameter} == 1) {
                $Options{'Parameters'}{'Is'.$Status} = 1;
            } elsif ($SearchForm->{$Parameter} == 2) {
                $Options{'Parameters'}{'Not'.$Status} = 1;
            }
        } elsif ($Parameter eq 'TransTimeTo'){
            #add 1 day to the searchdate
            $Options{'Parameters'}{$Parameter} = AddDuration($SearchForm->{$Parameter}->clone(), 'days', 1);

        }else{
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }

        # EPG-12969: use original input values in pager URL
        if( $Parameter =~ /^TransTime(From|To)$/x ) {
            $Options{'UriParameters'}{$Parameter} = $Servlet->form->value($Parameter);
        }
        else {
            $Options{'UriParameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
    }

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

#========================================================================================
# function     SaveShippingFulfillmentMethodDependencies
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SaveShippingFulfillmentMethodDependencies( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction for depdendency table ShippingFulfillmentMethod / ShippingMethod
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SaveShippingFulfillmentMethodDependencies {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    my $Shop = $Servlet->object;
    my $hForm = $Form->form($Shop, 'SaveShippingFulfillmentMethodDependencies');

    foreach my $hShippingEntry (@{$hForm->{'ShippingMethods'}}) {
        my $ShippingID       = $hShippingEntry->{'ShippingMethodID'};
        my $ShippingMethod   = $Shop->loadObject($ShippingID, 'ShippingMethod');
        my $OldFulfillment   = $ShippingMethod->get('SelectedShippingFulfillmentMethod');
        my $NewFulfillmentID = $hShippingEntry->{'ShippingFulfillmentMethodID'};
        my $NewFulfillment;
        $NewFulfillment = $Shop->loadObject($NewFulfillmentID, 'ShippingFulfillmentMethod') if ($NewFulfillmentID);
        # case 1: old:undef    -> new:object
        # case 2: old:object   -> new:undef
        # case 3: old:objectid != new:objectid
        if (    ($NewFulfillmentID && !$OldFulfillment)
            || (!$NewFulfillmentID && $OldFulfillment)
            || ( $NewFulfillmentID && $OldFulfillment
              && $NewFulfillmentID != $OldFulfillment->id)) {
            $ShippingMethod->set({'SelectedShippingFulfillmentMethod' => $NewFulfillment});
        }
    }
    return;
}

#========================================================================================
# function     ReceiveShippingOrderResult
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ReceiveShippingOrderResult( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction to receive tracking information for shipping order
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub ReceiveShippingOrderResult {
    my $self = shift;
    my ($Servlet) = @_;
    return;
}

#========================================================================================
# function     UnityViewShippingOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->UnityViewShippingOrder( $Servlet );
#----------------------------------------------------------------------------------------
# description  ViewAction that redirects to the shippingorder
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $Result | whatever the event handler returns | ?
#========================================================================================
sub UnityViewShippingOrder {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $FulfillmentType = $self->getShippingFulfillmentType();
    my $Transaction = GetCurrentShippingOrder($Shop, $FulfillmentType);

    $Servlet->vars('ObjectID',     $Transaction->id);
    $Servlet->vars('ViewAction',   'UnityMBO-ViewShippingOrder');
    return $Servlet->doAction('UnityMBO-ViewShippingOrder');
}

#========================================================================================
# function     ViewShippingOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewShippingOrder( $Servlet );
#----------------------------------------------------------------------------------------
# description  ViewAction that redirects to the shippingorder
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $Result | whatever the event handler returns | ?
#========================================================================================
sub ViewShippingOrder {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $FulfillmentType = $self->getShippingFulfillmentType();
    my $Transaction = GetCurrentShippingOrder($Shop, $FulfillmentType);

    $Servlet->vars('ObjectID',     $Transaction->id);
    $Servlet->vars('ViewAction',   'MBO-ViewShippingOrder');
    return $Servlet->doAction('MBO-ViewShippingOrder');
}

#========================================================================================
# function     getShippingFulfillmentType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->getShippingFulfillmentType( );
#----------------------------------------------------------------------------------------
# description  returns ShippingFulfillmentType as object.
#----------------------------------------------------------------------------------------
# return       $Type | ShippingFulfillmentType | object
#========================================================================================
sub getShippingFulfillmentType {
    my $self = shift;
    my $Type = LoadObjectByPath(JoinObjectPath(['','ShippingFulfillmentTypes',$self->TransactionPaymentName()]));
    GetLog()->debug('type: '.$Type->alias);
    return $Type;
}


1;
