#========================================================================================
# package      DE_EPAGES::MailType::API::NewPassword
# state        public
#----------------------------------------------------------------------------------------
# description  Resets the password of a user and sends the new password by e-mail.
#========================================================================================
package DE_EPAGES::MailType::API::NewPassword;
use base Exporter;
use strict;

our @EXPORT_OK = qw(
    SendNewPassword
    SendNewPasswordToken
    GetUserNewPasswordUrl
);

use DE_EPAGES::Core::API::Password qw ( CryptPassword GenerateNewPassword );
use DE_EPAGES::Core::API::PerlTools  qw( GetCurrentDateTime CreateSecureCode );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::TestParameter qw( TestObject );
use DE_EPAGES::Object::API::Factory qw( ExistsClassByAlias LoadClassByAlias );
use DE_EPAGES::Object::API::Table::Language qw( GetInfoLanguage );

#========================================================================================
# function     SendNewPassword
# state        public (obsolete! use SendNewPasswordToken() )
#----------------------------------------------------------------------------------------
# syntax       $Success = SendNewPassword( $User, $MailType );
# syntax       $Success = SendNewPassword( $User, $MailType, $hMailParams );
#----------------------------------------------------------------------------------------
# description  Obsolete: This function uses SendNewPasswordToken().
#               Creates a new password for the user and sends it to the users's e-mail
#               address.
#----------------------------------------------------------------------------------------
# input        $User | user object | object
# input        $MailType | e-mail event | object
# input        $hMailParams | (optional) e-mail parameters, see
#                   <function DE_EPAGES::Mail::API::Mail::new> | ref.hash
# return       $Success | true if successful | boolean
#========================================================================================
sub SendNewPassword {
    my ($User, $MailType, $hMailParams) = @_;
    TestObject( 'User', $User );
    TestObject( 'MailType', $MailType );
    $hMailParams = {} unless defined $hMailParams;

    # Generate password for user.
    my $Password = GenerateNewPassword();
    $User->set({'Password' => CryptPassword($Password)});

    return _sendMail( $User, {'Password'=>$Password}, $MailType, $hMailParams );
}

#========================================================================================
# function     SendNewPasswordToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Success = SendNewPasswordToken( $Shop, $User, $MailType );
# syntax       $Success = SendNewPasswordToken( $Shop, $User, $MailType, $hMailParams );
#----------------------------------------------------------------------------------------
# description  Creates a random token for the user and sends it as url to the users's e-mail
#               address.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $User | user object | object
# input        $MailType | e-mail event | object
# input        $hMailParams | (optional) e-mail parameters, see
#                   <function DE_EPAGES::Mail::API::Mail::new> | ref.hash
# return       $Success | true if successful | boolean
#========================================================================================
sub SendNewPasswordToken {
    my ($Shop, $User, $MailType, $hMailParams) = @_;
    TestObject( 'Shop', $Shop );
    TestObject( 'User', $User );
    TestObject( 'MailType', $MailType );
    $hMailParams = {} unless defined $hMailParams;

    # Create Mail.
    return _sendMail( $User, {'URL'=>GetUserNewPasswordUrl($Shop, $User)}, $MailType, $hMailParams );
}

sub _url {
    my ($URL, $ParameterString) = @_;
    if ($URL =~ /\/\?/) {
        $URL .= '&'.$ParameterString;
    } else {
        $URL .= '/?'.$ParameterString;
    }
}

#========================================================================================
# function     GetUserNewPasswordUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Url = GetUserNewPasswordUrl($Shop, $User);
#----------------------------------------------------------------------------------------
# description  Returns an url for password change and creates a random token for the user.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $User | user object | object
# return       $Url | url to set up new password with new verification token | string
#========================================================================================
sub GetUserNewPasswordUrl {
    my ($Shop, $User) = @_;
    TestObject( 'User', $User );
    TestObject( 'Shop', $Shop );

    my $Class = LoadClassByAlias('User');
    my $Parent = $User->parent->parent;
    my $ChangePWDUrl;
    # MBO
    if ($Class->existsAttribute('IsMerchant') && $User->get('IsMerchant')) {
        $ChangePWDUrl = _url($Shop->get('WebUrlAdminSSL'), 'ViewAction=ViewUserChangeLostPassword&LastViewAction=MBO-ViewGeneral');
    # PBO
    } elsif (ExistsClassByAlias('Provider') && $Parent->instanceOf('Provider')) {
        $ChangePWDUrl = _url($Shop->get('WebUrlSSL'), 'ViewAction=ViewUserChangeLostPassword&LastViewAction=PBO-ViewPBO');
    # TBO
    } elsif ($Parent->instanceOf('System')) {
        $ChangePWDUrl = _url($Shop->get('WebUrlSSL'), 'ViewAction=ViewUserChangeLostPassword&LastViewAction=TBO-ViewTBO');
    # SF
    } else {
        if ($Shop->get('IsUnityRegistered')) {
            $ChangePWDUrl = _url($Shop->get('WebServerScriptUrlSSL'), 'ViewAction=UnityViewChangeLostPassword');
        } else {
            $ChangePWDUrl = _url($Shop->get('WebUrlSSL'), 'ViewAction=ViewChangeLostPassword');
        }
    }

    # Generate token for user.
    my $Password = GenerateNewPassword();
    my $PwdToken = CreateSecureCode($Password);
    $User->set({ 'PWDToken' => $PwdToken, 'PWDTokenExpireDate' => GetCurrentDateTime()->add('hours'=>24) });
    return "$ChangePWDUrl&ObjectID=" . $User->id . "&Token=$PwdToken";
}

#========================================================================================
# function     _sendMail
# state        privat
#----------------------------------------------------------------------------------------
# syntax       $Success = _sendMail( $User, $Url, $MailType );
# syntax       $Success = _sendMail( $User, $hVars, $MailType, $hMailParams );
#----------------------------------------------------------------------------------------
# description  send email to user about new password or password token.
#----------------------------------------------------------------------------------------
# input        $User | user object | object
# input        $hVars | TLE variables incl. password or password token url | ref.hash
# input        $MailType | e-mail event | object
# input        $hMailParams | (optional) e-mail parameters, see
#                   <function DE_EPAGES::Mail::API::Mail::new> | ref.hash
# return       $Success | true if successful | boolean
#========================================================================================
sub _sendMail {
    my ( $User, $hVars, $MailType, $hMailParams ) = @_;

    $hMailParams->{'TO'} = $User->get('EMail');
    $hMailParams->{'HTMLAllowed'} = $User->get('IsHtmlEMailAllowed');

    my $hUserLanguage = GetInfoLanguage($User->get('LanguageID'));
    my %RegionalParams = (
        Locale      => $User->get('LocaleID'),
        Language    => $hUserLanguage->{'Code2'},
    );

    $hVars->{'User'} = $User->tleHash;

    # add single customer to tle hash
    if ($User->class->existsAttribute('Customers')) {
        my $aCustomers = $User->get('Customers');
        if (scalar @$aCustomers == 1) {
            $hVars->{'Customer'} = $aCustomers->[0]->tleHash;
        } else {
            $hVars->{'Customer'} = undef;
        }
    }

    eval {
        $MailType->send( $User->getSite(), $hMailParams, \%RegionalParams, $hVars );
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        return 0;
    }

    return 1;
}

1;
