#========================================================================================
# package      DE_EPAGES::Order::API::OrderDocumentService
# state        public
#----------------------------------------------------------------------------------------
# description  Collects order documents data
#========================================================================================
package DE_EPAGES::Order::API::OrderDocumentService;

use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::WebService::API::Util qw ( ConvertAddressToJava ConvertOrderDatesForEPJ GetAddressAttributeNames );
use strict;



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Order::API::OrderDocumentService
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

#========================================================================================
# function     getOrderDocumentData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hOrderDocumentData = $self->getOrderDocumentData( $OrderDocument, $LocaleID );
#----------------------------------------------------------------------------------------
# description  Service that provides epj formatted data of the order document.
#----------------------------------------------------------------------------------------
# input        $OrderDocument | order document object | object
# input        $LocaleID | locale identifier value | integer
# return       $hOrderDocumentData | formatted order document data | refhash
#========================================================================================
sub getOrderDocumentData {
    my $self = shift;
    my ($OrderDocument, $LocaleID) = @_;

    my $hOrderDocument = $self->_getOrderDocumentAttributeValues($OrderDocument);

    my $hOrderDocumentData = {};
    $hOrderDocumentData = ConvertOrderDatesForEPJ($hOrderDocument, $hOrderDocumentData);
    $hOrderDocumentData->{'orderId'} = $hOrderDocument->{'Order'}->guid();
    $hOrderDocumentData->{'documentType'} = $hOrderDocument->{'Class'}->alias();
    $hOrderDocumentData->{'documentNumber'} = $hOrderDocument->{'Alias'};
    $hOrderDocumentData->{'address'} = $self->_getAddressData($hOrderDocument->{'Address'});
    $hOrderDocumentData->{'lineItemContainer'} = $self->_getLineItemContainerData($hOrderDocument->{'LineItemContainer'}, $LocaleID);

    return $hOrderDocumentData;
}

#========================================================================================
# function     _getOrderDocumentAttributeValues
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_getOrderDocumentAttributeValues( $OrderDocument );
#----------------------------------------------------------------------------------------
# description  attribute values from order document object required to generate service data
#----------------------------------------------------------------------------------------
# input        $OrderDocument | OrderDocument object | object
# return       $hOrderDocument | hash with OrderDocument attributes | refhash
#========================================================================================
sub _getOrderDocumentAttributeValues {
    my $self = shift;
    my ($OrderDocument) = @_;

    my @Attributes = qw (
        Order
        Class
        Alias
        CreationDate
        Address
        LineItemContainer
    );

    return $OrderDocument->get( \@Attributes );
}

#========================================================================================
# function     _getAddressData
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_getAddressData( $AddressObject );
#----------------------------------------------------------------------------------------
# description  It provides address data ready to be returned by the service
#----------------------------------------------------------------------------------------
# input        $AddressObject | address object to get data from | object
# return       $hAddressData | collected address data | refhash
#========================================================================================
sub _getAddressData {
    my $self = shift;
    my ($AddressObject) = @_;

    my @Attributes = GetAddressAttributeNames();

    return isValid($AddressObject) ? ConvertAddressToJava($AddressObject->get( \@Attributes )) : undef;
}

#========================================================================================
# function     _getLineItemContainerData
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hLineItemContainerData = $self->_getLineItemContainerData( $LineItemContainer, $LocaleID );
#----------------------------------------------------------------------------------------
# description  it gets epj formatted data from selected line item container
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer object | object
# input        $LocaleID | locale identifier value | integer
# return       $hLineItemContainerData | hash with LineItemContainer data | refhash
#========================================================================================
sub _getLineItemContainerData {
    my $self = shift;
    my ($LineItemContainer, $LocaleID) = @_;

    my $Locale = $self->_loadLocaleById($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');
    my $IsTaxIncluded = 0;
    my $LineItemContainerData = DE_EPAGES::Order::API::LineItemContainerData->new($LineItemContainer);

    return $LineItemContainerData->getLineItemContainerData($LanguageID, $LocaleID, $IsTaxIncluded);
}

#=========================================================================================
# Wrappers
#=========================================================================================
sub _loadLocaleById {
    my $self = shift;
    my ($LocaleID) = @_;
    return DE_EPAGES::Object::API::Object::Locale->load($LocaleID);
}

1;