#========================================================================================
# package      DE_EPAGES::Order::API::OrderService
# state        public
#----------------------------------------------------------------------------------------
# description  Collects order data
#========================================================================================
package DE_EPAGES::Order::API::OrderService;

use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_NET TAX_MODEL_GROSS );
use DE_EPAGES::WebService::API::Util qw ( ConvertAddressToJava ConvertOrderDatesForEPJ GetAddressAttributeNames );
use DE_EPAGES::Order::API::LineItemContainerData;
use DE_EPAGES::Order::API::OrderPatch;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Order::API::Order qw ( Basket2Order );
use DE_EPAGES::Order::API::Resource::Cart;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use strict;



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Order::API::OrderService
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

sub createOrderAndCustomer {
    my $self = shift;
    my ($Basket, $LocaleID) = @_;

    my $order;
    if(!$Basket->get('IsMinOrderAmountReached')){
        Error('BadRequest', {'cart has not reached minimum order amount! cartId'=>$Basket->guid});
    }
    #TODO validate manually if every product in cart meets stock-level before creating order!
    eval{
        $order = Basket2Order($Basket);
    };

    if (ExistsError()){
        my $Error = GetError->code;
        if($Error eq 'BASKET_BLOCKED'){
            Error('BadRequest', {'cart is blocked! cartId' => $Basket->guid });
        }
        if($Error eq 'NoBillingAddress'){
            Error('BadRequest', {'billingAddress is missing! cartId' => $Basket->guid });
        }
        Error('BadRequest', {'check cart! cartId' => $Basket->guid});
    }

    return $self->getOrderDataWithLineItemContainer($order, $LocaleID);
}


sub getOrderData {
    my $self = shift;
    my ($Order, $LocaleID, $IncludeTax) = @_;

    my $Locale     = $self->_loadLocaleById($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');
    my $hOrderData = {};

    my $hOrder = $self->_getOrderAttributeValues($Order, $LanguageID);
    $hOrderData = ConvertOrderDatesForEPJ($hOrder, $hOrderData);

    my $hBillingAddr = $self->_getAddressAttributeValues($hOrder->{'BillingAddress'});
    my $hShippingAddr = $self->_getAddressAttributeValues($hOrder->{'ShippingAddress'});
    $hOrderData->{'billingAddress'} = $hBillingAddr;
    $hOrderData->{'shippingAddress'} = $hShippingAddr;

    $hOrderData->{'orderId'} = $hOrder->{'GUID'};
    $hOrderData->{'orderNumber'} = $hOrder->{'Alias'};
    $hOrderData->{'customerComment'} = $hOrder->{'CustomerComment'};
    $hOrderData->{'internalNote'} = $hOrder->{'Comment'};
    $hOrderData->{'customerId'} = $hOrder->{'Customer'}->guid();
    $hOrderData->{'customerNumber'} = $hOrder->{'Customer'}->alias();

    # add user locale if user exists (AD-2777, AD-3047)
    my $User = $hOrder->{'User'};
    if(defined $User) {
        $hOrderData->{'locale'} = $User->get('LocaleID');
    }

    my $LineItemContainer= $hOrder->{'LineItemContainer'};
    $hOrderData->{'currencyId'}  = $LineItemContainer->get('CurrencyID');

    if($LineItemContainer->get('TaxModel') == TAX_MODEL_GROSS){
        $hOrderData->{'taxModel'}  = 'GROSS';
    }elsif($LineItemContainer->get('TaxModel') == TAX_MODEL_NET ){
        $hOrderData->{'taxModel'}  = 'NET';
    }

    $hOrderData->{'grandTotal'}  = $LineItemContainer->get('GrandTotal');
    $hOrderData->{'totalBeforeTax'}  = $LineItemContainer->get('TotalBeforeTax');
    $hOrderData->{'totalTax'}  = $LineItemContainer->get('TotalTax');

    my $LineItemContainerData = $self->_createLineItemContainerData($Order);
    $hOrderData->{'shippingData'} = $LineItemContainerData->getShippingData($LanguageID, $LocaleID, $IncludeTax);
    $hOrderData->{'paymentData'} = $LineItemContainerData->getPaymentData($LanguageID, $LocaleID, $IncludeTax);

    return $hOrderData;
}



sub getOrderDataWithLineItemContainer {
    my $self = shift;
    my ($Order, $LocaleID, $IncludeTax) = @_;

    my $Locale     = $self->_loadLocaleById($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');

    my $hOrderData = $self->getOrderData($Order, $LocaleID, $IncludeTax);

    my $LineItemContainerData = $self->_createLineItemContainerData($Order);
    $hOrderData->{'lineItemContainer'} = $LineItemContainerData->getLineItemContainerData($LanguageID, $LocaleID, $IncludeTax);

    return $hOrderData;
}



sub updateOrderPartially {
    my $self = shift;
    my ($Order, $LocaleID, $ahRequestBody) = @_;

    my $Locale     = $self->_loadLocaleById($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');

    my $Patch = $self->_createOrderPatch($Order, $LanguageID);
    foreach my $hPatchOperation (@{$ahRequestBody}) {
        $Patch->patch($hPatchOperation);
    }

    return;
}




sub _loadLocaleById {
    my $self = shift;
    my ($LocaleID) = @_;
    return DE_EPAGES::Object::API::Object::Locale->load($LocaleID);
}



sub _getOrderAttributeValues {
    my $self = shift;
    my ($Order, $LanguageID) = @_;

    my @Attributes = qw (
        GUID
        Alias
        LineItemContainer
        CreationDate
        DispatchedOn
        BillingAddress
        ShippingAddress
        Customer
        ArchivedOn
        CancelledOn 
        ClosedOn
        PaidOn
        ReturnedOn 
        ViewedOn
        PendingOn
        ShippedOn
        InvoicedOn
        Comment
        CustomerComment
        User
        InProcessOn
        ReadyForShippingOn
        PartlyDispatchedOn
        PartlyInvoicedOn
        PartlyPaidOn
    );

    return $Order->get( \@Attributes, $LanguageID );
}



sub _getAddressAttributeValues {
    my $self = shift;
    my ($AddressObject) = @_;

    my @Attributes = GetAddressAttributeNames();

    return isValid($AddressObject) ? ConvertAddressToJava($AddressObject->get( \@Attributes )) : undef;
}



sub _createLineItemContainerData {
    my $self = shift;
    my ($Order) = @_;

    my $LineItemContainer = $Order->get('LineItemContainer');
    return DE_EPAGES::Order::API::LineItemContainerData->new($LineItemContainer);
}



sub _createOrderPatch {
    my $self = shift;
    my ($Order, $LanguageID) = @_;
    return DE_EPAGES::Order::API::OrderPatch->new($Order, $LanguageID);
}



1;