#========================================================================================
# package      DE_EPAGES::Order::API::Resource::OrderCollection
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectCollectionResource
# description  REST resource for get Orders
#========================================================================================
package DE_EPAGES::Order::API::Resource::OrderCollection;
use base DE_EPAGES::WebService::API::Resource::BaseObjectCollectionResource;

use DE_EPAGES::Core::API::Url qw (ParseQueryString);
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::Error qw (ExistsError);
use DE_EPAGES::Core::API::TestParameter qw (TestInteger);
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObjectByPath );
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::Order::API::WebService::OrderService;
use DE_EPAGES::Order::API::Object::CustomerOrder;
use DE_EPAGES::WebService::API::Util qw (  ConvertDateForEP6 );
use DE_EPAGES::Order::API::OrderService;
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);
use DE_EPAGES::Search::API::Object::PagerSearch qw ();
use DE_EPAGES::XML::API::DateTimeFormatXML;
use DateTime::Format::Strptime;
use feature 'switch';
use JSON;

use strict;



use constant DEFAULT_PAGE_SIZE => 10;
use constant DEFAULT_MAX_PAGE_SIZE => 100;



sub service {
    return LoadObjectByPath('/RestServices/OrderCollection');
}



sub class {
    return 'Shop';
}



sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        GET => [
            ['get', 'application/vnd.epages.v1+json'],
            ['get', 'application/json'],
        ],
    }->{$RequestMethod};
}



sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/orders/?$|;
    return 1;
}



sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;


    given( $ActionName ) {
        when( 'get' ) {
            my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
            my $ResultsPerPage = $hQuery->{'resultsPerPage'}->[0];
            my $LocaleID = $hQuery->{'locale'}->[0];
            my $Page = $hQuery->{'page'}->[0];
            return [ $Object, $LocaleID, $ResultsPerPage, $Page, $hQuery];
        }
        default {
            return [ $Object ];
        }
    }
}



sub get {
    my $self = shift;
    my ($Shop, $LocaleID, $ResultsPerPage, $Page, $hQuery) = @_;

    # Validate Paging Input
    $Page //= 1;
    $ResultsPerPage //=  DEFAULT_PAGE_SIZE;
    eval {
        TestInteger('page', $Page, 1);
        TestInteger('resultsPerPage', $ResultsPerPage, 1);
    };
    if ( ExistsError() ) {
        return;
    }
    return unless $ResultsPerPage <= DEFAULT_MAX_PAGE_SIZE; ## no critic (ProhibitNegativeExpressionsInUnlessAndUntilConditions)
    return unless $ResultsPerPage > 0; ## no critic (ProhibitNegativeExpressionsInUnlessAndUntilConditions)
    return unless $Page > 0; ## no critic (ProhibitNegativeExpressionsInUnlessAndUntilConditions)

    my $Context = $self->_createShopRequestContext($Shop);
    $LocaleID = $Context->validLocale($LocaleID);

    my $SearchName = 'SimpleSearch';
    my $SearchForm;
    my $OrderBy = 'CreationDate';

    foreach my $Key (keys %$hQuery){
        # Check for (.*)On aka filters on status
        if ($Key =~ /(.*)On$/) {   ## no critic (ProhibitCascadingIfElse)
            #camel-case for epagesj
            my $NewKey = ucfirst($Key);
            my $Value = $hQuery->{$Key}->[0];
            if(isValid($Value)){
                $SearchForm->{$NewKey} = $hQuery->{$Key}->[0] eq 'true'?'1':'2';
            }
        } elsif ($Key =~ /(.*)Before$/) {
            my $NewKey = ucfirst($Key);
            my $Value = $hQuery->{$Key}->[0];
            if(isValid($Value)){
                # if value exists convert date for EP6
                $SearchForm->{$NewKey} = ConvertDateForEP6($Value);
            }
        } elsif ($Key =~ /(.*)After$/) {
            my $NewKey = ucfirst($Key);
            my $Value = $hQuery->{$Key}->[0];
            if(isValid($Value)){
                # if value exists convert date for EP6
                $SearchForm->{$NewKey} = ConvertDateForEP6($Value);
            }
        } elsif($Key eq 'customerId'){
            # Search orders for a customer guid
            my $Customer = $Shop->loadObjectByGuid( $hQuery->{$Key}->[0], 'Customer' );
            $SearchForm->{'CustomerID'} = $Customer->id();
        } elsif($Key eq 'productId'){
            # Search orders for a product guid
            my $Product = $Shop->loadObjectByGuid( $hQuery->{$Key}->[0], 'Product' );
            $SearchForm->{'ProductAliasString'} = $Product->alias();
        } elsif($Key eq 'currency'){
                $SearchForm->{'CurrencyID'} = $hQuery->{$Key}->[0];
        } elsif($Key eq 'updated'){
            if($hQuery->{$Key}->[0] eq 'true'){
                $OrderBy    = 'LastUpdated';
            }
        } elsif($Key eq 'updatedFrom'){
                $SearchForm->{'UpdatedDateFrom'} = ConvertDateForEP6($hQuery->{$Key}->[0]);
                $OrderBy    = 'LastUpdated';
        }
        # Set searchname based on LastUpdated.  We have saperate sql query for UpdateDate lookup since it requires additional join.
        if($OrderBy ne 'LastUpdated'){
            $SearchName = 'RESTSearch';
        }else{
            $SearchName = 'LastUpdatedSearch';
        }
    }

    $SearchForm->{'Search'} = $SearchName;
    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;
    my $SearchDefinition = $self->_GetSearchDefinition($self->_LoadClassByAlias('CustomerOrder'), $SearchName);

    my $hOptions = {
        'Object'  => $Shop,
        'ObjectID'=> $Shop->id,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'IsVisible'  => 1,
        },
        'PageSize'=>$ResultsPerPage,
        'OrderBy'    => $OrderBy,
        'OrderDesc'  => 1,
    };
    if($OrderBy eq 'LastUpdated'){
        $hOptions->{'UriParameters'}{'Search'} = 'LastUpdatedSearch';
    }
    foreach my $Parameter (keys %$SearchForm) {
        if ($Parameter =~ /String/) {
            # replace * with %
            $hOptions->{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        } elsif ($Parameter =~ /(.*)On$/) {
            # status = 0/1/2 egal/yes/no
            my $Status = $1;
            if ($SearchForm->{$Parameter} == 1) {
                $hOptions->{'Parameters'}{'Is'.$Status} = 1;
            } elsif ($SearchForm->{$Parameter} == 2) {
                $hOptions->{'Parameters'}{'Not'.$Status} = 1;
            }
        } else{
            $hOptions->{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }

    }

    if (defined $hOptions->{'Parameters'}{'ProductAliasString'}) {
        #ProductLineItemClassIDs
        my $LineItemProduct = $self->_LoadClassByAlias('LineItemProduct');
        my $Classes = $LineItemProduct->subClasses();
        unshift @$Classes, $LineItemProduct;
        $hOptions->{'Parameters'}{'ProductLineItemClassIDs'} = [ map { $_->id() } @$Classes ];
    }
    LogDebug('SearchForm', $SearchForm);
    LogDebug('Parameters', $hOptions->{'Parameters'});

    # Get all the order-ids meeting search options.
    my $Search = $self->_createPagerSearch($hOptions);
    # Get total order count meeting seach options.
    my $Count = $Search->{'ItemCount'};
    # Filter order-ids which belong to the asked page
    my $aOrders = $Search->items($Page);

    # Get Order-details for selected order-ids
    my @aResultOrders = ();
    my $OrderService = $self->_createOrderService();
    foreach my $Order (@{$aOrders}){
        my $hResponse = $OrderService->getOrderData($Order, $LocaleID);
        push(@aResultOrders, $hResponse);
    }

    return {
        results         => $Count,
        page            => $Page,
        resultsPerPage  => $ResultsPerPage,
        items           => \@aResultOrders,
    };
}


sub _createShopRequestContext {
    my $self = shift;
    my ($Shop) = @_;
    return DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
}



sub _createPagerSearch {
    my $self = shift;
    my ($hOptions) = @_;
    return DE_EPAGES::Search::API::Object::PagerSearch->new($hOptions);
}



sub _createOrderService {
    my $self = shift;
    return DE_EPAGES::Order::API::OrderService->new();
}



sub _LoadClassByAlias {
    my $self = shift;
    my ($ClassName) = @_;
    return LoadClassByAlias($ClassName);
}



sub _GetSearchDefinition {
    my $self = shift;
    my ($ResultClass, $SearchName) = @_;
    return GetSearchDefinition($ResultClass, $SearchName);
}

1;