#========================================================================================
# package      DE_EPAGES::Order::API::Resource::OrderResource
# state        public
# base         DE_EPAGES::Order::API::Resource::BaseOrderResource
# description  implements the REST resource for orders of a shop
#========================================================================================
package DE_EPAGES::Order::API::Resource::OrderResource;
use base DE_EPAGES::Order::API::Resource::BaseOrderResource;

use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::Core::API::Url qw (ParseQueryString);
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadClassByAlias );
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::XML::API::DateTimeFormatXML;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;
use DE_EPAGES::WebService::API::Util qw ( ConvertAddressToPerl ConvertOrderDatesForEP6 );
use DE_EPAGES::Order::API::WebService::OrderService;
use DE_EPAGES::Order::API::OrderService;
use DE_EPAGES::Order::API::Order qw( GetEditableOrder );
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
use feature 'switch';
use JSON::XS;

use strict;



sub service {
    return LoadObjectByPath('/RestServices/Order');
}



sub match {
    my $self = shift;
    my ($Request) = @_;
    return unless (
            ($Request->uri->path =~ m|^/api/orders/(.+)/?$|)
        && !($Request->uri->path =~ m|^/api/orders/(.+)/documents/?$|)
    );
    return $1;
}



sub params {
    my $self = shift;
    my ( $ActionName, $Object, $Request ) = @_;

    given ($ActionName) {
        when ('get') {
            my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
            my $LocaleID = $hQuery->{'locale'}->[0];
            my $IncludeTax = $hQuery->{'includeTax'}->[0];
            return [$Object, $LocaleID, $IncludeTax];
        }
        when ('put') {
            my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
            my $LocaleID = $hQuery->{'locale'}->[0];
            return [ $Object, $LocaleID, decode_json( $Request->decoded_content() ) ];
        }
        when( 'patch' ) {
            my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
            my $LocaleID = $hQuery->{'locale'}->[0];
            return [ $Object, $LocaleID ];
        }
    }
}



sub get {
    my $self = shift;
    my ( $Order, $LocaleID, $IncludeTax) = @_;

    my $Shop = $Order->getSite();
    my $Context = $self->_createShopRequestContext($Shop);
    $LocaleID = $Context->validLocale($LocaleID);

    my $OrderService = $self->_createOrderService();
    my $hOrderData = $OrderService->getOrderDataWithLineItemContainer($Order, $LocaleID, $IncludeTax);

    return $hOrderData;
}



sub put {
    my $self = shift;
    my ( $Order, $LocaleID, $hRequestBody ) = @_;

    my $Shop = $Order->getSite();
    my $Context = $self->_createShopRequestContext($Shop);
    $LocaleID = $Context->validLocale($LocaleID);

    $self->_checkForEditableOrder($Order);

    # Set Order Dates
    $Order = ConvertOrderDatesForEP6($Order, $hRequestBody);

    $self->_RunBatchProcess('UpdateOrders', sub {

        # Set Comment and internalNote
        if ( isValid $hRequestBody->{'internalNote'} ) {
            $Order->set( { 'Comment' => $hRequestBody->{'internalNote'} } );
        }
        if ( isValid $hRequestBody->{'customerComment'}){
            $Order->set( { 'CustomerComment' => $hRequestBody->{'customerComment'} } );
        }

        # Set Addresses
        my $BillingAddress = $hRequestBody->{'billingAddress'};
        my $ShippingAddress = $hRequestBody->{'shippingAddress'};

        my $OrderService = $self->_createSoapOrderService();
        my $SetChangeOrderForNonStatusChanges = 0;

        if ( isValid $ShippingAddress ) {
            my $hShippingAddressPerl = ConvertAddressToPerl($ShippingAddress);
            $OrderService->setAddress( $Order, $hShippingAddressPerl, 'ShippingAddress' );
            $SetChangeOrderForNonStatusChanges = 1;
        }
        if ( isValid $BillingAddress ) {
            my $hBillingAddressPerl = ConvertAddressToPerl($BillingAddress);
            $OrderService->setAddress( $Order, $hBillingAddressPerl, 'BillingAddress' );
            $SetChangeOrderForNonStatusChanges = 1 ;
        }

        if( $SetChangeOrderForNonStatusChanges ){
            $self->_setLastUpdate($Order);
        }

    });

    return $self->get($Order, $LocaleID);
}



sub patch {
    my $self = shift;
    my ($Order, $LocaleID, $ahRequestBody) = @_;

    my $Shop = $Order->getSite();
    my $Context = $self->_createShopRequestContext($Shop);
    $LocaleID = $Context->validLocale($LocaleID);
    $self->_checkForEditableOrder($Order);

    $self->_RunBatchProcess('UpdateOrders', sub {
        my $OrderService = $self->_createOrderService();
        $OrderService->updateOrderPartially($Order, $LocaleID, $ahRequestBody);
    });

    return $self->get($Order, $LocaleID);
}



sub _createShopRequestContext {
    my $self = shift;
    my ($Shop) = @_;
    return DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($Shop);
}



sub _createOrderService {
    return DE_EPAGES::Order::API::OrderService->new();
}



sub _createSoapOrderService {
    return DE_EPAGES::Order::API::WebService::OrderService->new();
}



sub _setLastUpdate {
    my $self = shift;
    my ($Order) = @_;
    my $Profile = LoadClassByAlias('CustomerOrder')->lastUpdateProfile('OrderChange');
    $Order->setUpdated($Profile);
    return;
}



sub _checkForEditableOrder {
    my $self = shift;
    my ($Order) = @_;
    my $EditableOrder = GetEditableOrder($Order);
    if(isValid $EditableOrder){
        Error('Conflict', {'Reason' => 'Currently order is in edit mode so it is not available for update!'});
    }
    return;
}



sub _RunBatchProcess {
    my $self = shift;
    my ($BatchTaskName, $cSub) = @_;
    RunBatchProcess($BatchTaskName, $cSub);
    return;
}

1;