package DE_EPAGES::Order::API::Test::HooksOrderProcess;
use base DE_EPAGES::Shop::API::Test;

use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias);
use DE_EPAGES::Order::Hooks::OrderProcess;
use DE_EPAGES::Test::Mock::EpagesObjectMockFactory;

use strict;
use Test::More;
use Test::Exception;

sub test {
    my $self = shift;

    $self->test_OnAfterConfirmation_ProductNotAvailable();
    $self->test_OnAfterSaveMultiCheckoutBasket_ProductNotAvailable();
    $self->test_OnAfterConfirmation_ProductIsChanged();
    $self->test_OnAfterSaveMultiCheckoutBasket_ProductIsChanged();

    $self->testServlet->form->resetFormErrors();
    $self->test_OnAfterConfirmation_NoException();
    $self->test_OnAfterSaveMultiCheckoutBasket_NoException();
    $self->test_OnAfterSaveMultiCheckoutBasket_MinOrderAmountNotReached();

    done_testing();
}

# ================================================================
# TEST METHODS
# ================================================================
sub test_OnAfterConfirmation_ProductIsChanged {

    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    $self->testServlet->form->resetFormErrors();
    $self->testBasket
         ->container
         ->childrenByClass('LineItemProduct')
         ->[0]->set({'IsChanged' => 1});

    throws_ok(
        sub {
            DE_EPAGES::Order::Hooks::OrderProcess::OnAfterConfirmation(
                {'Servlet'=>$self->testServlet});
        },
        qr/INVALID_FORM:.*ProductMarkedAsChanged/,
        "$SubTestName: Exception is thrown as expected.");

    return;
}


sub test_OnAfterConfirmation_ProductNotAvailable {

    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    $self->testProduct->set({'IsAvailable' => 0});

    # check that exception is thrown
    throws_ok(
        sub {
            DE_EPAGES::Order::Hooks::OrderProcess::OnAfterConfirmation(
                {'Servlet'=>$self->testServlet});
        },
        qr/INVALID_FORM:.*ProductNotAvailable/,
        "$SubTestName: Exception is thrown as expected.");

    return;
}

sub test_OnAfterSaveMultiCheckoutBasket_ProductIsChanged {

    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    $self->testServlet->form->resetFormErrors();
    $self->testBasket
         ->container
         ->childrenByClass('LineItemProduct')
         ->[0]->set({'IsChanged' => 1});

    throws_ok(
        sub {
            DE_EPAGES::Order::Hooks::OrderProcess::OnAfterConfirmation(
                {'Servlet'=>$self->testServlet});
        },
        qr/INVALID_FORM:.*ProductMarkedAsChanged/,
        "$SubTestName: Exception is thrown as expected.");

    return;
}

sub test_OnAfterSaveMultiCheckoutBasket_ProductNotAvailable {

    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    $self->testServlet->form->resetFormErrors();
    $self->testProduct->set({'IsAvailable' => 0});

    # check that exception is thrown
    throws_ok(
        sub {
            DE_EPAGES::Order::Hooks::OrderProcess::OnAfterSaveMultiCheckoutBasket(
                {'Servlet'=>$self->testServlet});
        },
        qr/INVALID_FORM:.*ProductNotAvailable/,
        "$SubTestName: Exception is thrown as expected.");

    return;
}

sub test_OnAfterSaveMultiCheckoutBasket_MinOrderAmountNotReached {

    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    $self->setIsMinOrderAmountReached(40);

    throws_ok(
        sub {
            DE_EPAGES::Order::Hooks::OrderProcess::OnAfterSaveMultiCheckoutBasket(
                {'Servlet'=>$self->testServlet});
        },
        qr/INVALID_FORM:.*MinOrderIsNotReached/,
        "$SubTestName: Exception is thrown as expected.");

    return;
}

sub test_OnAfterConfirmation_NoException {

    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    $self->testProduct->set({'IsAvailable' => 1});
    $self->testBasket
        ->container
        ->childrenByClass('LineItemProduct')
        ->[0]->set({'IsChanged' => 0});

    lives_ok(sub {
            DE_EPAGES::Order::Hooks::OrderProcess::OnAfterConfirmation(
                {'Servlet'=>$self->testServlet});
        }, "$SubTestName: succeeds.");

    return;
}

sub test_OnAfterSaveMultiCheckoutBasket_NoException {

    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    $self->testProduct->set({'IsAvailable' => 1});
    $self->testBasket
        ->container
        ->childrenByClass('LineItemProduct')
        ->[0]->set({'IsChanged' => 0});
    $self->setIsMinOrderAmountReached(0);

    lives_ok(sub {
            DE_EPAGES::Order::Hooks::OrderProcess::OnAfterSaveMultiCheckoutBasket(
                {'Servlet'=>$self->testServlet});
        }, "$SubTestName: succeeds.");

    return;
}
# ================================================================

sub init {
    my $self = shift;
    $self->SUPER::init;
    my $Shop = $self->shop;
    $DE_EPAGES::Object::API::Language::LANGUAGEID = 2;

    # create test product
    my $ProductFolder = $self->shop->get('ProductFolder');
    $Shop->featureMaxValue('Products', 1);
    $Shop->featureMaxValue('Basket', 1);

    my $ProductClass = LoadClassByAlias('Product');
    my $Product1     = $ProductClass->insertObject(
        {
            Parent => $ProductFolder,
            Alias  => 'Product_1',
            IsVisible => 1,
            StockLevel => 10
        }
    );
    $Product1->setListPrice( 'EUR', 1, 20 );
    $Product1->setListPrice( 'GBP', 1, 20 );

    # create basket
    my $AnonymesUser = $Shop->folder('AnonymousUsers')->insertChild('AnonymousUser', {
            'Alias' => 'TestUser',
            'Name' => 'TestUserName'
    });

    my $BillingAddress = $AnonymesUser->folder('Addresses')->insertChild('Address', {
            'Alias' => 'BillingAddress',
            'FirstName' => 'Max',
            'LastName' => 'Mustermann',
            'Street' => 'Blumenweg 42',
            'City' => 'Neustadt',
            'Zipcode' => '01234',
            'EMail' => 'm.mustermann@epages.de ',
            'CountryID' => 276,
    });

    $AnonymesUser->set({'BillingAddress' => $BillingAddress});

    my $ObjBaskets = $AnonymesUser->folder('Baskets');
    my $ObjectBasket = LoadClassByAlias('Basket')->insertObject({

        'Shop'   => $Shop,
        'Parent' => $ObjBaskets,
        'Alias' => 'TestBasket',
        'BillingAddress' => $BillingAddress
    });
    $ObjectBasket->insertLineItem(LoadClassByAlias('LineItemProduct'), {

            'Alias' => 'TestAliasProductLineItem',
            'Name'  => 'TestNameProductLineItem',
            'Product' => $Product1,
            'LineItemPrice' => '20.00'
    });

    # build servlet
    my $EpagesObjectMockFactory = DE_EPAGES::Test::Mock::EpagesObjectMockFactory->new;
    my $Servlet = $EpagesObjectMockFactory->buildServlet($ObjectBasket)->getMockObject();

    $self->testBasket($ObjectBasket);
    $self->testProduct($Product1);
    $self->testServlet($Servlet);
}

sub cleanup {    # optional
    my $self = shift;
    $self->SUPER::cleanup;    # deletes the shop
}

# ================================================================
# HELP METHODS
# ================================================================
sub testBasket {
    my $self = shift;
    my ($Basket) = @_;

    $self->{'Basket'} = $Basket if defined $Basket;
    return $self->{'Basket'};
}

sub testProduct {
    my $self = shift;
    my ($Product) = @_;

    $self->{'Product'} = $Product if defined $Product;
    return $self->{'Product'};
}

sub testServlet {
    my $self = shift;
    my ($Servlet) = @_;

    $self->{'Servlet'} = $Servlet if defined $Servlet;
    return $self->{'Servlet'};
}

#========================================================================================
# function     setIsMinOrderAmountReached
# state        private
#----------------------------------------------------------------------------------------
# syntax       setIsMinOrderAmountReached( 120 );
#----------------------------------------------------------------------------------------
# description  In order to set the value of boolean variable 'IsMinOrderAmountReached'
#                to false you need to set MinOrderValue > Price of the product.
#----------------------------------------------------------------------------------------
# input        MinOrderValue | double value e.g.: 0.0
#========================================================================================
sub setIsMinOrderAmountReached {
    my $self = shift;
    my ($MinOrderValue) = @_;

    my $LineItem = $self->testBasket->container->childrenByClass('LineItemProduct')->[0];
    my $OrderSettings = {
        'MinOrderAmounts' => [
            {
                'Value' => $MinOrderValue,
                'CurrencyID' => 'EUR'
            },
            {
                'Value' => $MinOrderValue,
                'CurrencyID' => 'GBP'
            }
        ]
    };

    my $Info = {};
    $Info->{'Quantity'} = $LineItem->get('Quantity');

    $self->testBasket->get('Shop')->set($OrderSettings);
    $LineItem->set($self->testProduct->lineItem($self->testBasket->container, $Info));
}
# ================================================================

1;

