#==============================================================================
# package      DE_EPAGES::Order::API::WebService::OrderService
# base         DE_EPAGES::Order::API::WebService::BaseOrderService
# state        public
# description  Implements the Order WebService.
#==============================================================================
package DE_EPAGES::Order::API::WebService::OrderService;
use base DE_EPAGES::Order::API::WebService::BaseOrderService;

use strict;
use DateTime;

use DE_EPAGES::Core::API::Log qw(LogDebug LogError);
use DE_EPAGES::Core::API::Error qw ( Error ResetError ExistsError GetError);
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw( Transaction LoadClassByAlias LoadRootObject );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Order::API::Constants qw( ORDER_STATUS_ATTRIBUTES );
use DE_EPAGES::Order::API::Order qw( CreateOrderWithoutContainer );
use DE_EPAGES::Order::API::Shop qw ( GetNextOrderNo );
use DE_EPAGES::Search::API::Object::PagerSearch qw (ReplaceWildcards);
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);
use DE_EPAGES::XML::API::SOAPDateTimeFormatXML;
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Service = DE_EPAGES::Order::API::WebService::OrderService->new( %Options );
#----------------------------------------------------------------------------------------
# description  Creates a new web service object as a blessed hash reference and adds
#               the values from the %Options hash.
#----------------------------------------------------------------------------------------
# input        %Options | user-defined values | hash
# return       $Service | web service object | object
#========================================================================================
sub new {
    my $self = shift;
    my @Options = @_;
    $self->SUPER::new((
        'ServiceUrn'      => 'urn://epages.de/WebService/OrderService/2005/01',
        'ServiceTypesUrn' => 'urn://epages.de/WebService/OrderTypes/2005/01',
        @Options,
    ));
}

#========================================================================================
# function     getInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getInfo( $aOrders, $aAttributes, $aLanguageCodes )
#----------------------------------------------------------------------------------------
# description  WebService-Method: getInfo
#               Returns information about a list of Orders (incl. line items, attributes).
#----------------------------------------------------------------------------------------
# input        $aOrders | list of Order paths | ref.array.string
# input        $aAttributes | list of AttributeIDs | ref.array.string
# input        $aLanguageCodes | list of LanguageCodes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            info of all given Orders | SOAP::Data
#========================================================================================
sub getInfo {
    my $self = shift;

    # expects a list of Order paths
    my ($aOrders, $aAttributes, $aLanguageCodes) = @_;
    LogDebug('aOrders=', $aOrders);
    LogDebug('aAttributes=', $aAttributes);
    LogDebug('aLanguageCodes=', $aLanguageCodes);

    my $aLanguages = $self->getLanguageIDs($aLanguageCodes);

    my @soapOrders = ();
    my $result;

    foreach my $orderPath (@$aOrders) {
        my $soapOrder;

        eval {
            # get object
            my $Order= $self->loadShopObject($orderPath, 'CustomerOrder');

            my $hOutOrder = {
                'Path'               => $self->soapPath( $Order ),
                'Alias'              => $self->soapString( $Order->alias ),
                'Class'              => $self->soapPath( $Order->class ),
                'Customer'           => $self->soapPath( $Order->get('Customer') ),
                'User'               => $self->soapPath( $Order->get('User') ),
                'BillingAddress'     => $self->soapAddress( $Order->get('BillingAddress') ),
                'ShippingAddress'    => $self->soapAddress( $Order->get('ShippingAddress') ),
                'CreationDate'       => $self->soapDateTime($Order->get('CreationDate'      )),
                'ViewedOn'           => $self->soapDateTime($Order->get('ViewedOn'          )),
                'CancelledOn'        => $self->soapDateTime($Order->get('CancelledOn'       )),
                'InProcessOn'        => $self->soapDateTime($Order->get('InProcessOn'       )),
                'PendingOn'          => $self->soapDateTime($Order->get('PendingOn'         )),
                'ReadyForShippingOn' => $self->soapDateTime($Order->get('ReadyForShippingOn')),
                'PartlyDispatchedOn' => $self->soapDateTime($Order->get('PartlyDispatchedOn')),
                'DispatchedOn'       => $self->soapDateTime($Order->get('DispatchedOn'      )),
                'PartlyInvoicedOn'   => $self->soapDateTime($Order->get('PartlyInvoicedOn'  )),
                'InvoicedOn'         => $self->soapDateTime($Order->get('InvoicedOn'        )),
                'ShippedOn'          => $self->soapDateTime($Order->get('ShippedOn'         )),
                'PartlyPaidOn'       => $self->soapDateTime($Order->get('PartlyPaidOn'      )),
                'PaidOn'             => $self->soapDateTime($Order->get('PaidOn'            )),
                'ClosedOn'           => $self->soapDateTime($Order->get('ClosedOn'          )),
                'ArchivedOn'         => $self->soapDateTime($Order->get('ArchivedOn'        )),
                'LineItemContainer'  => $self->buildSoapLineItemContainer($Order->get('LineItemContainer')),
                'Attributes'         =>
                $self->soapList( $self->soapAttributeValues($Order, $aAttributes, $aLanguages), 'TAttribute'),
            };

            # build return object
            $soapOrder = $self->soapData('Order', $hOutOrder, 'TGetInfo_Return');
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapOrder = $self->soapData('Order', {
                'Path'       => $self->soapString( $orderPath ),
                'Error'      => $self->soapError($Error)
            }, 'TGetInfo_Return');
            ResetError();
        }
        push @soapOrders, $soapOrder;
    }

    $result = $self->soapData('Orders', $self->soapList(\@soapOrders, 'TGetInfo_Return') );
    $self->mapType([qw (TGetInfo_Return)], $self->getServiceTypesUrn);
    $self->mapType([qw (TLocalizedValue)], $self->getEpagesTypesUrn);

    return $result;
}

#========================================================================================
# function     exists
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->exists( $aOrders )
#----------------------------------------------------------------------------------------
# description  WebService-Method: exists
#               Returns exists status of a list of Orders.
#----------------------------------------------------------------------------------------
# input        $aOrders | list of Order paths | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            exist status of all given Orders | SOAP::Data
#========================================================================================
sub exists {
    my $self = shift;

    # expects a list of Order paths
    my ($aOrders) = @_;
    LogDebug('aOrders=', $aOrders);

    my @soapOrders = ();
    my $result;

    foreach my $orderPath (@$aOrders) {
        my $soapOrder;

        my $hOutOrder = {
            'Path' => $self->soapString( $orderPath ),
            'exists' => $self->soapBool( $self->existsShopObject($orderPath, 'CustomerOrder') ),
        };

        # build return object
        $soapOrder = $self->soapData('Order', $hOutOrder, 'TExists_Return');
        push @soapOrders, $soapOrder;
    }

    $result = $self->soapData('Orders', $self->soapList(\@soapOrders, 'TExists_Return') );
    $self->mapType([qw (TExists_Return)], $self->getServiceTypesUrn);

    return $result;
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->delete( $aOrders )
#----------------------------------------------------------------------------------------
# description  WebService-Method: delete
#               Deletes a list of Orders.
#----------------------------------------------------------------------------------------
# input        $aOrders | list of Order paths | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            delete status of all given Orders | SOAP::Data
#========================================================================================
sub delete {
    my $self = shift;

    # expects a list of Order paths
    my ($aOrders) = @_;
    LogDebug('aOrders=', $aOrders);

    my @soapOrders = ();
    my $result;

    RunBatchProcess('DeleteOrders', sub {

        foreach my $orderPath (@$aOrders) {
            my $soapOrder;

            eval {
                # get object
                my $Order= $self->loadShopObject($orderPath, 'CustomerOrder');

                $Order->delete;

                my $hOutOrder = {
                    'Path' => $self->soapString( $orderPath ),
                    'deleted' => $self->soapBool( not $Order->exists ),
                };

                # build return object
                $soapOrder = $self->soapData('Order', $hOutOrder, 'TDelete_Return');
            };
            if (ExistsError()) {
                my $Error = GetError();
                $soapOrder = $self->soapData('Order', {
                    'Path'       => $self->soapString( $orderPath ),
                    'Error'      => $self->soapError($Error)
                }, 'TDelete_Return');
                ResetError();
            }
            push @soapOrders, $soapOrder;
        }

        $result = $self->soapData('Orders', $self->soapList(\@soapOrders, 'TDelete_Return') );
        $self->mapType([qw (TDelete_Return)], $self->getServiceTypesUrn);

    });

    return $result;
}

#========================================================================================
# function     update
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->update( $aOrders )
#----------------------------------------------------------------------------------------
# description  WebService-Method: update
#               Updates a list of Orders.
#----------------------------------------------------------------------------------------
# input        $aOrders | list of Order info (path and
#                              data to update) | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            update status of all given Orders | SOAP::Data
#========================================================================================
sub update {
    my $self = shift;

    # expects a list of Orders
    my ($aOrders) = @_;
    LogDebug('aOrders=', $aOrders);

    my @soapOrders = ();
    my $result;

    RunBatchProcess(['UpdateOrders', 'UpdateProducts'], sub {

        foreach my $hInOrder (@$aOrders) {
            my $soapOrder;
            my $orderPath = $hInOrder->{'Path'};
            eval {
                $self->checkMandatoryFields($hInOrder, ['Path']);

                # get object
                my $Order= $self->loadShopObject($orderPath, 'CustomerOrder');

                # get profile for lastupdate
                my $OrderChangeProfile = LoadClassByAlias('CustomerOrder')->lastUpdateProfile('OrderChange');

                Transaction( sub {
                    # set Order parameters
                    $self->setParameters($Order, $hInOrder);

                    # set Address
                    $self->setAddress($Order, $hInOrder->{'BillingAddress'}, 'BillingAddress');
                    $self->setAddress($Order, $hInOrder->{'ShippingAddress'}, 'ShippingAddress');

                    # set line time container
                    $self->setLineItemContainer($Order, $hInOrder->{'LineItemContainer'});

                    # set Attributes
                    $self->setAttributeValues($Order, $hInOrder->{'Attributes'});

                    # set lastUpdate for order
                    $Order->setUpdated($OrderChangeProfile);
                });

                my $hOutOrder = {
                    'Path' => $self->soapString( $orderPath ),
                    'updated' => $self->soapBool( 1 ),
                };

                # build return object
                $soapOrder = $self->soapData('Order', $hOutOrder, 'TUpdate_Return');
            };
            if (ExistsError()) {
                my $Error = GetError();
                $soapOrder = $self->soapData('Order', {
                    'Path'       => $self->soapString( $orderPath ),
                    'Error'      => $self->soapError($Error)
                }, 'TUpdate_Return');
                ResetError();
            }
            push @soapOrders, $soapOrder;
        }

        $result = $self->soapData('Orders', $self->soapList(\@soapOrders, 'TUpdate_Return') );
        $self->mapType([qw (TUpdate_Return)], $self->getServiceTypesUrn);

    });

    return $result;
}

#========================================================================================
# function     create
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->create( $aOrders )
#----------------------------------------------------------------------------------------
# description  WebService-Method: create
#               Creates a list of Orders.
#----------------------------------------------------------------------------------------
# input        $aOrders | list of Order info (data to create) | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            create status of all given Orders | SOAP::Data
#========================================================================================
sub create {
    my $self = shift;

    # expects a list of Orders
    my ($aOrders) = @_;
    LogDebug('aOrders=', $aOrders);

    my $Shop = $self->site;

    my @soapOrders = ();
    my $result;

    RunBatchProcess(['CreateOrders', 'UpdateProducts'], sub {

        foreach my $hInOrder (@$aOrders) {
            my $soapOrder;
            my $alias = $hInOrder->{'Alias'} || GetNextOrderNo($Shop, $self->loadShopObject($hInOrder->{'Customer'}));

            my $Order;
            eval {
                $self->checkMandatoryFields($hInOrder, ['Customer', 'BillingAddress', 'LineItemContainer']);

                # get order by search for alias
                my $aFoundOrders = $self->doSearch($Shop, {'Alias' => $alias});

                # check if exists
                Error('ORDER_ALREADY_EXISTS', {'Alias' => $alias}) if @$aFoundOrders;

                my $Customer = $self->loadShopObject($hInOrder->{'Customer'}, 'Customer');
                my $User = defined $hInOrder->{'User'}
                    ? $self->loadShopObject($hInOrder->{'User'}, 'User')
                    : shift @{$Customer->get('Users')};

                Transaction( sub {
                    # create new order with empty lineitem container
                    $Order = CreateOrderWithoutContainer( $Customer, {
                        'Alias' => $alias,
                        'User' => $User,
                    });

                    # set line item container
                    $self->setLineItemContainer($Order, $hInOrder->{'LineItemContainer'});

                    # set Order parameters
                    $self->setParameters($Order, $hInOrder);

                    # set Address
                    $self->setAddress($Order, $hInOrder->{'BillingAddress'}, 'BillingAddress');
                    if ( defined $hInOrder->{'ShippingAddress'} ) {
                        $self->setAddress($Order, $hInOrder->{'ShippingAddress'}, 'ShippingAddress');
                    }

                    # set Attributes
                    $self->setAttributeValues($Order, $hInOrder->{'Attributes'});
                });

                my $hOutOrder = {
                    'Alias'   => $self->soapString( $alias ),
                    'Path'    => $self->soapPath( $Order ),
                    'created' => $self->soapBool( 1 ),
                };

                # build return object
                $soapOrder = $self->soapData('Order', $hOutOrder, 'TCreate_Return');
            };
            if (ExistsError()) {
                my $Error = GetError();
                LogError($Error);
                $soapOrder = $self->soapData('Order', {
                    'Alias' => $self->soapString( $alias ),
                    'Error' => $self->soapError($Error)
                }, 'TCreate_Return');
                ResetError();
            }
            push @soapOrders, $soapOrder;

            $hInOrder->{'callerComponent'} = 'WS';
            TriggerHook('API_OrderCreated', {
                'Order'   => $Order,
                'Options' => $hInOrder
            });

        }

        $result = $self->soapData('Orders', $self->soapList(\@soapOrders, 'TCreate_Return') );
        $self->mapType([qw (TCreate_Return)], $self->getServiceTypesUrn);

    });

    return $result;
}

#========================================================================================
# function     find
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->find( $InSearchParameters )
#----------------------------------------------------------------------------------------
# description  WebService-Method: find
#               Find Orders by some search parameters.
#----------------------------------------------------------------------------------------
# input        $InSearchParameters | search arguments (all optional):
#                 <ul>
#                   <li> Alias         - Order nr               - string
#                   <li> Product       - product alias is part
#                                        of Order               - string
#                   <li> DateFrom      - time frame for order   - datetime string
#                   <li> DateTo        - time frame for order   - datetime string
#
#                   <li> IsViewed            - is status 'Viewed'              - boolean
#                   <li> IsCancelled         - is status 'Cancelled'           - boolean
#                   <li> IsInProcess         - is status 'InProcess'           - boolean
#                   <li> IsPending           - is status 'Pending'             - boolean
#                   <li> IsReadyForShipping  - is status 'ReadyForShipping'    - boolean
#                   <li> IsPartlyDispatched  - is status 'PartlyDispatched'    - boolean
#                   <li> IsDispatched        - is status 'Dispatched'          - boolean
#                   <li> IsShipped           - is status 'Shipped'             - boolean
#                   <li> IsPartlyPaid        - is status 'PartlyPaid'          - boolean
#                   <li> IsPaid              - is status 'Paid'                - boolean
#                   <li> IsClosed            - is status 'Closed'              - boolean
#                   <li> IsArchived          - is status 'Archived'            - boolean
#                   <li> IsPartlyInvoiced    - is status 'PartlyInvoiced'      - boolean
#                   <li> IsInvoiced          - is status 'Invoiced'            - boolean
#
#                   <li> NotViewed           - is status not 'Viewed'          - boolean
#                   <li> NotCancelled        - is status not 'Cancelled'       - boolean
#                   <li> NotInProcess        - is status not 'InProcess'       - boolean
#                   <li> NotPending          - is status not 'Pending'         - boolean
#                   <li> NotReadyForShipping - is status not 'ReadyForShipping'- boolean
#                   <li> NotPartlyDispatched - is status not 'PartlyDispatched'- boolean
#                   <li> NotDispatched       - is status not 'Dispatched'      - boolean
#                   <li> NotShipped          - is status not 'Shipped'         - boolean
#                   <li> NotPartlyPaid       - is status not 'PartlyPaid'      - boolean
#                   <li> NotPaid             - is status not 'Paid'            - boolean
#                   <li> NotClosed           - is status not 'Closed'          - boolean
#                   <li> NotArchived         - is status not 'Archived'        - boolean
#                   <li> NotPartlyInvoiced   - is status not 'PartlyInvoiced'  - boolean
#                   <li> NotInvoiced         - is status not 'Invoiced'        - boolean
#
#                   <li> Customer            - ObjectPath
#                 </ul> | ref.hash
# return       $SoapData  | a SOAP::Data structure with a list
#                            of Order paths | SOAP::Data
#========================================================================================
sub find { ## no critic qw( ProhibitLongLines )
    my $self = shift;
    my ($InSearchParameters) = @_;
    LogDebug('hInSearchParameters=', $InSearchParameters);

    my $Shop = $self->site;

    my $dateparser = DE_EPAGES::XML::API::SOAPDateTimeFormatXML->new();
    my $SystemTimeZone = LoadRootObject()->get('TimeZone');

    my $SearchParameters = {};

    ## no critic qw(ProhibitLongLines)
    $SearchParameters->{'OrderAliasString'      } = $InSearchParameters->{'Alias'}      if defined $InSearchParameters->{'Alias'};
    $SearchParameters->{'ProductAliasString'    } = $InSearchParameters->{'Product'}    if defined $InSearchParameters->{'Product'};

    $SearchParameters->{'CreationDateFrom'      } = $dateparser->parse_datetime($InSearchParameters->{'DateFrom'}, $SystemTimeZone) if defined $InSearchParameters->{'DateFrom'};
    $SearchParameters->{'CreationDateTo'        } = $dateparser->parse_datetime($InSearchParameters->{'DateTo'}, $SystemTimeZone)   if defined $InSearchParameters->{'DateTo'};

    $SearchParameters->{'IsViewed'              } = $InSearchParameters->{'IsViewed'           }   if $InSearchParameters->{'IsViewed'          };
    $SearchParameters->{'IsCancelled'           } = $InSearchParameters->{'IsCancelled'        }   if $InSearchParameters->{'IsCancelled'       };
    $SearchParameters->{'IsInProcess'           } = $InSearchParameters->{'IsInProcess'        }   if $InSearchParameters->{'IsInProcess'       };
    $SearchParameters->{'IsPending'             } = $InSearchParameters->{'IsPending'          }   if $InSearchParameters->{'IsPending'         };
    $SearchParameters->{'IsReadyForShipping'    } = $InSearchParameters->{'IsReadyForShipping' }   if $InSearchParameters->{'IsReadyForShipping'};
    $SearchParameters->{'IsPartlyDispatched'    } = $InSearchParameters->{'IsPartlyDispatched' }   if $InSearchParameters->{'IsPartlyDispatched'};
    $SearchParameters->{'IsDispatched'          } = $InSearchParameters->{'IsDispatched'       }   if $InSearchParameters->{'IsDispatched'      };
    $SearchParameters->{'IsShipped'             } = $InSearchParameters->{'IsShipped'          }   if $InSearchParameters->{'IsShipped'         };
    $SearchParameters->{'IsPartlyPaid'          } = $InSearchParameters->{'IsPartlyPaid'       }   if $InSearchParameters->{'IsPartlyPaid'      };
    $SearchParameters->{'IsPaid'                } = $InSearchParameters->{'IsPaid'             }   if $InSearchParameters->{'IsPaid'            };
    $SearchParameters->{'IsClosed'              } = $InSearchParameters->{'IsClosed'           }   if $InSearchParameters->{'IsClosed'          };
    $SearchParameters->{'IsArchived'            } = $InSearchParameters->{'IsArchived'         }   if $InSearchParameters->{'IsArchived'        };
    $SearchParameters->{'IsPartlyInvoiced'      } = $InSearchParameters->{'IsPartlyInvoiced'   }   if $InSearchParameters->{'IsPartlyInvoiced'  };
    $SearchParameters->{'IsInvoiced'            } = $InSearchParameters->{'IsInvoiced'         }   if $InSearchParameters->{'IsInvoiced'        };

    $SearchParameters->{'NotViewed'             } = $InSearchParameters->{'NotViewed'          }   if $InSearchParameters->{'NotViewed'          };
    $SearchParameters->{'NotCancelled'          } = $InSearchParameters->{'NotCancelled'       }   if $InSearchParameters->{'NotCancelled'       };
    $SearchParameters->{'NotInProcess'          } = $InSearchParameters->{'NotInProcess'       }   if $InSearchParameters->{'NotInProcess'       };
    $SearchParameters->{'NotPending'            } = $InSearchParameters->{'NotPending'         }   if $InSearchParameters->{'NotPending'         };
    $SearchParameters->{'NotReadyForShipping'   } = $InSearchParameters->{'NotReadyForShipping'}   if $InSearchParameters->{'NotReadyForShipping'};
    $SearchParameters->{'NotPartlyDispatched'   } = $InSearchParameters->{'NotPartlyDispatched'}   if $InSearchParameters->{'NotPartlyDispatched'};
    $SearchParameters->{'NotDispatched'         } = $InSearchParameters->{'NotDispatched'      }   if $InSearchParameters->{'NotDispatched'      };
    $SearchParameters->{'NotShipped'            } = $InSearchParameters->{'NotShipped'         }   if $InSearchParameters->{'NotShipped'         };
    $SearchParameters->{'NotPartlyPaid'         } = $InSearchParameters->{'NotPartlyPaid'      }   if $InSearchParameters->{'NotPartlyPaid'      };
    $SearchParameters->{'NotPaid'               } = $InSearchParameters->{'NotPaid'            }   if $InSearchParameters->{'NotPaid'            };
    $SearchParameters->{'NotClosed'             } = $InSearchParameters->{'NotClosed'          }   if $InSearchParameters->{'NotClosed'          };
    $SearchParameters->{'NotArchived'           } = $InSearchParameters->{'NotArchived'        }   if $InSearchParameters->{'NotArchived'        };
    $SearchParameters->{'NotPartlyInvoiced'     } = $InSearchParameters->{'NotPartlyInvoiced'  }   if $InSearchParameters->{'NotPartlyInvoiced'  };
    $SearchParameters->{'NotInvoiced'           } = $InSearchParameters->{'NotInvoiced'        }   if $InSearchParameters->{'NotInvoiced'        };

    $SearchParameters->{'Customer'} = $InSearchParameters->{'Customer'} if defined $InSearchParameters->{'Customer'};
    ## use critic

    my @OrderPaths = map {$self->soapPath($_)->name('Path')} @{$self->doSearch($Shop, $SearchParameters)};
    return $self->soapData('Orders', $self->soapList(\@OrderPaths, 'string' ));
}

#========================================================================================
# function     setParameters
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->setParameters( $Order, $hOrder )
#----------------------------------------------------------------------------------------
# description  No WebService-Method!
#               Sets parameters of a Order object (for create and update Orders).
#----------------------------------------------------------------------------------------
# input        $Order | Order to update | Order object
# input        $hOrder | data to assign to the Order
#                             <ul>
#                               <li>Customer            - ObjectPath
#                               <li>User                - ObjectPath
#                               <li>CreationDate        - date sting
#                               <li>ViewedOn            - date sting
#                               <li>CancelledOn         - date sting
#                               <li>InProcessOn         - date sting
#                               <li>PendingOn           - date sting
#                               <li>ReadyForShippingOn  - date sting
#                               <li>DispatchedOn        - date sting
#                               <li>ShippedOn           - date sting
#                               <li>PaidOn              - date sting
#                               <li>ClosedOn            - date sting
#                               <li>ArchivedOn          - date sting
#                             </ul> | ref.hash
#========================================================================================
sub setParameters {
    my $self = shift;
    my ($Order, $hInOrder) = @_;

    my $dateparser = DE_EPAGES::XML::API::SOAPDateTimeFormatXML->new();
    my $SystemTimeZone = LoadRootObject()->get('TimeZone');

    # set order attribute values
    my %Values;
    foreach my $attribute ( 'CreationDate', ORDER_STATUS_ATTRIBUTES ) {
        if( defined $hInOrder->{$attribute} ) {
            $Values{$attribute} = $dateparser->parse_datetime($hInOrder->{$attribute}, $SystemTimeZone);
        }

        # reset dateflag if an undef value was passed
        if (exists $hInOrder->{$attribute} and not defined $hInOrder->{$attribute}) {
            $Values{$attribute} = undef unless $attribute eq 'CreationDate';
        }
    }

    $Values{'Customer'} = $self->loadShopObject($hInOrder->{'Customer'}, 'Customer')
        if defined $hInOrder->{'Customer'};

    $Values{'User'} = $self->loadShopObject($hInOrder->{'User'}, 'User')
        if defined $hInOrder->{'User'};

    $Order->set( \%Values );

}

#========================================================================================
# function     doSearch
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->doSearch( $Shop, $SearchParameters )
#----------------------------------------------------------------------------------------
# description  No WebService-Method!
#               Searches for Orders by search parameters.
#----------------------------------------------------------------------------------------
# input        $Shop | shop | Shop object
# input        $SearchParameters | search arguments | ref.hash
# return       $itemPaths | list of Order paths | ref.array
#========================================================================================
sub doSearch {
    my $self = shift;

    my ($Shop, $SearchParameters) = @_;
    LogDebug('doSearch SearchParameters', $SearchParameters);

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('CustomerOrder'), 'DetailedSearch');
    my $ClassLineItemShipping     = LoadClassByAlias('LineItemShipping');
    my @LineItemShippingClassIDs = ( $ClassLineItemShipping->id, map { $_->id } @{$ClassLineItemShipping->subClasses} );
    my $AttributeShippingMethodID = $ClassLineItemShipping->attribute('ShippingMethod')->id;
    my $AttrPaymentMethodNameID   = LoadClassByAlias('PaymentMethod')->attribute('Name')->id;
    my $AttrShippingMethodNameID  = LoadClassByAlias('ShippingMethod')->attribute('Name')->id;
    my %Options = (
        'Object'  => $Shop,
        'ObjectID' => $Shop->id,
        'OrderBy' => 'Alias',
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'LanguageID' => $Shop->get('LanguageID'),
            'LineItemShippingClassIDs' => \@LineItemShippingClassIDs,
            'AttributeShippingMethodID' => $AttributeShippingMethodID,
            'AttrShippingMethodNameID' => $AttrShippingMethodNameID,
            'AttrPaymentMethodNameID' => $AttrPaymentMethodNameID,
        },
        'UriParameters'  => {
            'Search' => 'DetailedSearch',
        },
    );

   foreach my $Parameter (keys %$SearchParameters) {
        if ($Parameter =~ /String/) {
            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards($SearchParameters->{$Parameter});

        } elsif ($Parameter =~ /(.*)On$/) {
            # status = 0/1/2 egal/yes/no
            my $Status = $1;
            if ($SearchParameters->{$Parameter} == 1) {
                $Options{'Parameters'}{'Is'.$Status} = 1;
            } elsif ($SearchParameters->{$Parameter} == 2) {
                $Options{'Parameters'}{'Not'.$Status} = 1;
            }
        } else {
            $Options{'Parameters'}{$Parameter} = $SearchParameters->{$Parameter};
        }
        $Options{'UriParameters'}{$Parameter} = $SearchParameters->{$Parameter};
    }
    # add order dates
    if (not exists $Options{'Parameters'}{'CreationDateFrom'}) {
        $Options{'Parameters'}{'CreationDateFrom'} = DateTime->new('year'=>1900, 'month'=>1,'day'=>1);
    }
    if (not exists $Options{'Parameters'}{'CreationDateTo'}) {
        $Options{'Parameters'}{'CreationDateTo'} = GetCurrentDBHandle()->currentDateTime;
    }
    if (defined $Options{'Parameters'}{'ProductAliasString'}) {
        #ProductLineItemClassIDs
        my $LineItemProduct = LoadClassByAlias('LineItemProduct');
        my $Classes = $LineItemProduct->subClasses;
        unshift @$Classes, $LineItemProduct;
        $Options{'Parameters'}{'ProductLineItemClassIDs'} = join (',', map { $_->id } @$Classes);
    }

    LogDebug('SearchForm', $SearchParameters);
    LogDebug('Parameters', $Options{'Parameters'});

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    $Search->statistic() unless $Search->get('StatisticFinished');

    LogDebug('Result', $Search);

    my ($items) = [];
    for my $num (1 .. $Search->get('CountOfPages')) {
        push @$items, @{$Search->items($num)};
    }

    return $items;
}

1;
