use Test::More;
use strict;

use DE_EPAGES::Core::API::PerlTools qw (ElementOf GetCurrentDateTime );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);

my $ROOTPATH = '/Test';
my $SHOPPATH = '/Test/TestShop/';
local $ENV{'EPAGES_TESTSITE'} = 'Store';

sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    _testNewOrdersNotification();

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
    done_testing();
}

sub _testNewOrdersNotification {

    my $Shop = LoadObjectByPath($SHOPPATH);
    my $NotificationFolder = $Shop->folder('ShopNotifications');
    # exists
    ok( $NotificationFolder->existsChild('NewOrders'), 'ShopNotification NewOrders exists' );
    my $NewOrdersNotification = $NotificationFolder->child('NewOrders');

    my $Order1 = $Shop->path('TestCustomer/Orders/1234-5678');
    my $aObjects = $NewOrdersNotification->get('ShopNotificationObjects');
    is( $NewOrdersNotification->get('IsActive'), 0, 'ShopNotification NewOrders inactive');
    is( scalar(@$aObjects), 0, 'No orders unseen' );

    $Order1->set({'ViewedOn' => undef});
    note 'Order 1: ViewedOn => undef';
    $aObjects = $NewOrdersNotification->get('ShopNotificationObjects');
    is( $NewOrdersNotification->get('IsActive'), 1, 'ShopNotification NewOrders active');
    is( scalar(@$aObjects), 1, '1 order unseen' );
    ok( ElementOf($Order1, @$aObjects), 'Order 1 unseen' );

    my $Order2 = $Order1->clone({'Alias' => '08-15'}, 1);
    TriggerHook('API_OrderCreated', { Order => $Order2 });
    note 'clone Order 1';
    $aObjects = $NewOrdersNotification->get('ShopNotificationObjects');
    is( $NewOrdersNotification->get('IsActive'), 1, 'ShopNotification NewOrders active');
    is( scalar(@$aObjects), 2, '2 orders unseen' );
    ok( ElementOf($Order1, @$aObjects), 'Order 1 unseen' );
    ok( ElementOf($Order2, @$aObjects), 'Order 2 unseen' );

    $Order1->set({'ViewedOn' => GetCurrentDateTime()});
    note 'Order 1: ViewedOn => <current date time>';
    $aObjects = $NewOrdersNotification->get('ShopNotificationObjects');
    is( $NewOrdersNotification->get('IsActive'), 1, 'ShopNotification NewOrders active');
    is( scalar(@$aObjects), 1, '1 order unseen' );
    ok( ElementOf($Order2, @$aObjects), 'Order 2 unseen' );

    $Order2->set({'ViewedOn' => GetCurrentDateTime()});
    note 'Order 2: ViewedOn => <current date time>';
    $aObjects = $NewOrdersNotification->get('ShopNotificationObjects');
    is( $NewOrdersNotification->get('IsActive'), 0, 'ShopNotification NewOrders inactive');
    is( scalar(@$aObjects), 0, 'No orders unseen' );
}

sub Cleanup {
    if (ExistsObjectByPath($SHOPPATH)) {
        my $Shop = LoadObjectByPath($SHOPPATH);
        $Shop->set({ 'IsUnityRegistered' => 0 });
    }
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    my $Shop = LoadObjectByPath('/Test/TestShop/');
    $Shop->set({ 'IsUnityRegistered' => 1 });
    $Shop->folder('ShopNotifications')->insertChild( 'ShopNotification', { Alias => 'NewOrders', 'Position' => 20 })
        unless $Shop->folder('ShopNotifications')->existsChild('NewOrders');
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);

