#========================================================================================
# package      DE_EPAGES::PayPalPlus::API::MerchantOnboardingURLBuilder
# state        public
#----------------------------------------------------------------------------------------
# description  Builds URL for Merchant onboarding
#========================================================================================
package DE_EPAGES::PayPalPlus::API::MerchantOnboardingURLBuilder;

use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use URI;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::PayPalPlus::API::MerchantOnboardingURLBuilder
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}

#========================================================================================
# function     buildOnboardingURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       buildOnboardingURL( $PaymentMethodPayPalPlus );
#----------------------------------------------------------------------------------------
# description  Builds URL for Merchant onboarding
#----------------------------------------------------------------------------------------
# input        Payment Method to build onboarding url for |
#               DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus
# input        Url the user is forwards to after a successful onboarding | string
# return       1 if successful, 0 else | boolean
#========================================================================================
sub buildOnboardingURL {
    my $self = shift;
    my ($PaymentMethodPayPalPlus, $ReturlURL) = @_;

    eval {
        my $BaseUrl = $PaymentMethodPayPalPlus->getOnboardingUrl();
        my $hQueryParameters = {
            'channelId' => 'partner',
            'partnerId' => $PaymentMethodPayPalPlus->getPartnerAccountId(),
            'productIntentId' => 'ppplus',
            'displayMode' => 'regular',
            'integrationType' => 'TO',
            'features' => 'PAYMENT',
            'returnToPartnerUrl' => $ReturlURL,
            'tracking_id' => $PaymentMethodPayPalPlus->getTrackingId(),
            'partnerClientId' => $PaymentMethodPayPalPlus->getPartnerClientId(),
            'showPermissions' => 'TRUE',
        };

        my $URL = URI->new($BaseUrl);
        $URL->query_form($hQueryParameters);
        $self->_setOnboardingURL($URL->as_string());
    };
    if(ExistsError()) {
        my $Error = GetError();
        if($Error->code() eq 'PAYPALPLUS_PBO_SETUP_INCOMPLETE') {
            $self->_setError('ProviderPayPalAccountError');
        } else {
            $self->_setError($Error->code());
        }

        return 0;
    }

    return 1;
}

#========================================================================================
# function     _setOnboardingURL
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setOnboardingURL( $OnboardingURL );
#----------------------------------------------------------------------------------------
# description  sets URL for Merchant onboarding
#----------------------------------------------------------------------------------------
# input       $OnboardingURL | onBoarding URL | string
#========================================================================================
sub _setOnboardingURL {
    my $self = shift;
    my ($OnboardingURL) = @_;
    $self->{'OnboardingURL'} = $OnboardingURL;
}

#========================================================================================
# function     getOnboardingURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       getOnboardingURL( );
#----------------------------------------------------------------------------------------
# description  gets URL for Merchant onboarding
#----------------------------------------------------------------------------------------
# return       onBoarding URL | string
#========================================================================================
sub getOnboardingURL {
    my $self = shift;
    return $self->{'OnboardingURL'};
}

#========================================================================================
# function     getError
# state        public
#----------------------------------------------------------------------------------------
# syntax       getError( );
#----------------------------------------------------------------------------------------
# description  Get the error
#----------------------------------------------------------------------------------------
# return      Error | string
#========================================================================================
sub getError {
    my $self = shift;
    return $self->{'Error'};
}

#========================================================================================
# function     _setError
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setError( $Error);
#----------------------------------------------------------------------------------------
# description  Set error 
#----------------------------------------------------------------------------------------
# input        $Error| Error | string
#========================================================================================
sub _setError {
    my $self = shift;
    my ($Error) = @_;
    $self->{'Error'} = $Error;
    return;
}

1;