#========================================================================================
# package      DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service
# state        public
#----------------------------------------------------------------------------------------
# description  class to handle the GetMerchantStatus for PayPal plus
#========================================================================================
package DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service;
use base DE_EPAGES::PayPalPlus::API::Webservice::Base::Service;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Repository;
use DE_EPAGES::PayPalPlus::API::Constants_ErrorTypes qw (ErrorTypes_TestString ErrorTypes_TestObject ErrorTypes_TestInteger);
use DE_EPAGES::PayPalPlus::API::Logger;
use strict;

use constant BASE_ROUTE => 'payments/paypalplus/merchantstatus';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}
#========================================================================================
# function     doRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       doRequest( $PaymentMethodPayPalPlus );
#----------------------------------------------------------------------------------------
# description  Create request to epagesj and handle the answer
#----------------------------------------------------------------------------------------
# input        $PaymentMethodPayPalPlus | payment method paypal plus | DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus
# return       boolean flag if the request was successful or not | boolean
#========================================================================================
sub doRequest {
    my $self = shift;
    my ($PaymentMethodPayPalPlus) = @_;

    my $WasSuccessful;

    eval {
        my $Repository = $self->_prepareRequestRepository($PaymentMethodPayPalPlus);

        $self->{'Repository'} = $Repository;
        my $CredentialDTO = $Repository->getCredentialDTO();
        my $RequestDTO = $Repository->getRequestDTO();

        my $Url = $self->_buildUrlToEpagesJ();
        my $hHeaders = $CredentialDTO->toHash();
        my $hRequest = $RequestDTO->toHash();

        my $RestService = $self->_createRestService($PaymentMethodPayPalPlus); ## no critic (ProhibitPrivateSubroutineUse) protected (in base class)
        $RestService->setLogErrorsInComunicationlog(1);
        my $IsOnboardingCall = 1;
        $WasSuccessful = $RestService->post($Url, $hHeaders, $hRequest, $IsOnboardingCall);

        if( $WasSuccessful ) {
            $WasSuccessful = $self->_persistResponse( $RestService->getResponse(), $PaymentMethodPayPalPlus );
            my $Logger = $self->_createLogger();
            $Logger->logOnboarding({ 'Add MerchantClientId to PaymentMethod (Alias):' => $PaymentMethodPayPalPlus->alias() });
        } else {
            ## no critic (ProhibitPrivateSubroutineUse) protected (in base class)
            $self->_setErrorCode($RestService->getErrorCode());
            $self->_setErrorMessage($RestService->getErrorMessage());
            ## use critic (ProhibitPrivateSubroutineUse)
        }
    };
    if(ExistsError()) {
        return 0;
    };

    return $WasSuccessful;
}
#========================================================================================
# function     _createLogger
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_createLogger( );
#----------------------------------------------------------------------------------------
# description  Create logger class
#----------------------------------------------------------------------------------------
# return       $Object | Logger object | DE_EPAGES::PayPalPlus::API::Logger
#========================================================================================
sub _createLogger {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Logger->new();
}
#========================================================================================
# function     _prepareRequestRepository
# state        private
#----------------------------------------------------------------------------------------
# syntax       _prepareRequestRepository( $PaymentMethodPayPalPlus );
#----------------------------------------------------------------------------------------
# description  Builds RequestDTO from Order
#----------------------------------------------------------------------------------------
# input        $PaymentMethodPayPalPlus | PaymentMethod of PayPalPlus | DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus
# return      $RequestRepository | DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Repository
#========================================================================================
sub _prepareRequestRepository {
    my $self = shift;
    my ($PaymentMethodPayPalPlus) = @_;

    my $Repository = $self->_createRepository();
    eval {
        $Repository->buildRequest($PaymentMethodPayPalPlus);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        my $ErrorCode = $Error->code();
        if ( $self->_isHandleErrorFromBuildRequest( $ErrorCode )) { #TODO: AD-3097 beautify name isErrorFromBuildRequest
            $self->_handleError($Error, 'RequestError', 'It was not possible to get the data for the Request from the database');
        } else {
            Error($Error); #forward all other Errors
        }
        return;
    }

    return $Repository;
}
#========================================================================================
# function     _buildUrlToEpagesJ
# state        private
#----------------------------------------------------------------------------------------
# syntax       _buildUrlToEpagesJ( $CredentialDTO );
#----------------------------------------------------------------------------------------
# description  build the Url for the post
#----------------------------------------------------------------------------------------
# return       Url for request | string
#========================================================================================
sub _buildUrlToEpagesJ {
    my $self = shift;

    my $RestURLService = $self->_createRestURLService(); ## no critic (ProhibitPrivateSubroutineUse) protected defiend in base class

    return $RestURLService->buildEpagesJRestUrl(BASE_ROUTE, {});
}
#========================================================================================
# function     _persistResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       _persistResponse( $hResponse, $PaymentMethodPayPalPlus );
#----------------------------------------------------------------------------------------
# description  persists response in BussinessObject
#----------------------------------------------------------------------------------------
# input        $hResponse | response | refhash
# input        $PaymentMethodPayPalPlus | payment method paypal plus | DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus
# return       boolean if it was ok | boolean
#========================================================================================
sub _persistResponse {
    my $self = shift;
    my ($hResponse, $PaymentMethodPayPalPlus) = @_;

    my $Repository = $self->{'Repository'};
    eval {
        $Repository->writeResponse($hResponse,$PaymentMethodPayPalPlus);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        my $ErrorCode = $Error->code();
        if ( $self->_isHandleErrorFromPersistResponse( $ErrorCode )) { #TODO: AD-3097 beautify name
            return $self->_handleError($Error, 'PersistanceError', 'It was not possible to store the request in the database');
        } else {
            Error($Error); #forward all other Errors
        }
    }

    return 1;
}
#========================================================================================
# function     _handleError
# state        private
#----------------------------------------------------------------------------------------
# syntax       _handleError( $Error, $ErrorMessage, $ErrorCode );
#----------------------------------------------------------------------------------------
# description  Logs Error and sets Code and Message
#----------------------------------------------------------------------------------------
# input        $Error | Error | DE_EPAGES::Core::API::Error
# input        $ErrorCode | Error Code | string
# input        $ErrorMessage | Error Message | string
# return       0 | boolean
#========================================================================================
sub _handleError {
    my $self = shift;
    my ($Error, $ErrorCode, $ErrorMessage) = @_;
    ## no critic (ProhibitPrivateSubroutineUse) protected (in base class)
    $self->_LogError($Error);
    $self->_setErrorCode($ErrorCode);
    $self->_setErrorMessage($ErrorMessage);
    ## use critic (ProhibitPrivateSubroutineUse)
    return 0;
}
#========================================================================================
# function     _isHandleErrorFromPersistResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isHandleErrorFromPersistResponse( );
#----------------------------------------------------------------------------------------
# description  check if is an expected error code
#----------------------------------------------------------------------------------------
# input        $ErrorCode | Error code |string
# return       Flag to check if the error is one of the expected errors | boolean
#========================================================================================
sub _isHandleErrorFromBuildRequest {
    my $self = shift;
    my ($ErrorCode) = @_;
    if ( $ErrorCode ~~ [ ErrorTypes_TestString, ErrorTypes_TestObject, ErrorTypes_TestInteger ]){
        return 1;
    }else{
        return 0;
    }
}

#========================================================================================
# function     _isHandleErrorFromPersistResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isHandleErrorFromPersistResponse( );
#----------------------------------------------------------------------------------------
# description  check if is an expected error code
#----------------------------------------------------------------------------------------
# input        $ErrorCode | Error code |string
# return       Flag to check if the error is one of the expected errors | boolean
#========================================================================================
sub _isHandleErrorFromPersistResponse {
    my $self = shift;
    my ($ErrorCode) = @_;
    if ( $ErrorCode ~~ [ ErrorTypes_TestString, ErrorTypes_TestObject, 'MissingOAuthCredentials' ]){
         return 1;
     }else{
         return 0;
     }
}

#========================================================================================
# function     _createRepository
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createRepository( );
#----------------------------------------------------------------------------------------
# description  Create Repository
#----------------------------------------------------------------------------------------
# return       Repository object | DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Repository
#========================================================================================
sub _createRepository {
    my $self = shift;
    my $Repository = DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Repository->new();
    return $Repository;
}
1;