#========================================================================================
# package      DE_EPAGES::PayPalPlus::API::Webservice::RestService
# state        public
#----------------------------------------------------------------------------------------
# description  do the request to epagesj
#----------------------------------------------------------------------------------------
# example      my $RestService = DE_EPAGES::PayPalPlus::API::Webservice::RestService->new($Shop);
#               if($RestService->post->($URL,$hRequest)){
#                   my $hResponse = $RestService->getResponse();
#               } else {
#                   my $ErrorCode = $RestService->getErrorCode();
#                   my $ErrorMessage = $RestService->getErrorMessage();
#               }
#========================================================================================
package DE_EPAGES::PayPalPlus::API::Webservice::RestService;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::EPagesJ::API::Object::RestClient;
use DE_EPAGES::PayPalPlus::API::Logger;

use URI;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $EpagesObject | any epages object | DE_EPAGES::Object::API::Object:Object
# return       $self | self | DE_EPAGES::PayPalPlus::API::Webservice::RestService
#========================================================================================
sub new {
    my $class = shift;
    my ($EpagesObject) = @_;
    my $self  = bless {
        'EpagesObject' => $EpagesObject,
    }, $class;
    return $self;
}

#========================================================================================
# function     setLogErrorsInComunicationlog
# state        public
#----------------------------------------------------------------------------------------
# syntax       setLogErrorsInComunicationlog( );
#----------------------------------------------------------------------------------------
# description  sets a flag so that errors that happen during the request are
#               logged into the communication log instead of the error log.
#----------------------------------------------------------------------------------------
# input        $LogErrorsInComunicationlog | boolean
#========================================================================================
sub setLogErrorsInComunicationlog {
    my $self = shift;
    my ($LogErrorsInComunicationlog) = @_;
    $self->{'LogErrorsInComunicationlog'} = $LogErrorsInComunicationlog;
}
#========================================================================================
# function     post
# state        public
#----------------------------------------------------------------------------------------
# syntax       post( $Url,$hRequest );
#----------------------------------------------------------------------------------------
# description  does the request to epagesj
#               throws: WebserviceError and rethrow unknown dies
#----------------------------------------------------------------------------------------
# input        $Url | epagesj endpoint URL | string
# input        $hHeaders | header data | ref_hash
# input        $hRequest | request data | ref_hash
# return       1 or 0 | boolean
#========================================================================================
sub post {
    my $self = shift;
    my ($Url, $hHeaders, $hRequest, $IsOnboardingCall) = @_;
    #TODO: AD-3093 use DTO instead of hash

    my $RestClient = $self->_createRestClient($self->{'EpagesObject'}, {});
    eval {
        my $JSONRequest = $RestClient->encodeRequest($hRequest);
        #TODO: paypal might change the boarding proccess again, if nothing changes -> password is now stored in the paymentType, try to push this logic into the logPayment
        my $Password = $self->_getPasswordFromHeaders($hHeaders);

        my $Logger = $self->_createLogger();
        $Logger->logPayment('Request params', {'URL' => $Url, 'Headers' => $hHeaders, 'JSONRequest' => $JSONRequest}, $Password);
        
        my $Response = $RestClient->client()->post($Url, $hHeaders, $JSONRequest);
        $Logger->logPayment('Response params', {'URL' => $Url,'Response' => $Response}, $Password);
        $self->_setResponse($RestClient->decodeResponse($Response->content()));
        if($IsOnboardingCall){
            $Logger->logOnboarding({
            'epagesj url' => $Response->request()->uri()->as_string(),
            'epagesj request content' => $Response->request()->content(),
            'forwarded answer from paypal' => $Response->content(),
        });
        }
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        if($IsOnboardingCall){
            my $Logger = $self->_createLogger();
            $Logger->logOnboarding({"epagesj response" => $Error->vars()});
        }
        $self->_handleErrors($Error);
        return 0;
    }

    return 1;
}
#========================================================================================
# function     _getPasswordFromHeaders
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getPasswordFromHeaders( $hHeaders );
#----------------------------------------------------------------------------------------
# description  get the password from the Headers
#----------------------------------------------------------------------------------------
# input        $hHeaders | Headers with password | hashref
# return       the clientSecret of the URL | string
#========================================================================================
sub _getPasswordFromHeaders{
    my $self = shift;
    my ($hHeaders) = @_;
    return $hHeaders->{'providerClientSecret'};
}
#========================================================================================
# function     _handleErrors
# state        public
#----------------------------------------------------------------------------------------
# syntax       _handleErrors( $Error );
#----------------------------------------------------------------------------------------
# description  handle the errors wich can happen
#----------------------------------------------------------------------------------------
# input        $Error | epages error object | DE_EPAGES::Core::API::Error
#========================================================================================
sub _handleErrors {
    my $self = shift;
    my ($Error) = @_;

    if ( $Error->code() eq 'NativeDie' ) {
        my $Message = $Error->message();
        my $ErrorMessage_for_encodeRequest = 'hash- or arrayref expected';
        my $ErrorMessage_for_decodeResponse = 'malformed JSON string';
        if($Message =~ /$ErrorMessage_for_encodeRequest|$ErrorMessage_for_decodeResponse/g) {
            $self->_LogError($Error);
            $self->_setErrorCode('WebserviceError');
            $self->_setErrorMessage('JSON parser error');
        } else {
            Error($Error);#forward all other NativeDies
        }
    } elsif ($Error->code() eq 'REQUEST_FAILED') {
        $self->_LogError($Error);
        $self->_setErrorCode('WebserviceError');
        my $ErrorVars = $Error->vars();
        $self->_setErrorMessage($ErrorVars->{'StatusLine'});
    } else {
        Error($Error); #forward all other Errors
    }

    return;
}
#========================================================================================
# function     getResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       getResponse( );
#----------------------------------------------------------------------------------------
# description  will return the response as a hash/array
#----------------------------------------------------------------------------------------
# return       response | refhash refarray
#========================================================================================
sub getResponse {
    my $self = shift;
    return $self->{'HashResponse'};
}
#========================================================================================
# function     _setResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setResponse( $hResponse );
#----------------------------------------------------------------------------------------
# description  setter for response
#----------------------------------------------------------------------------------------
# input        $hResponse | response | refhash refarray
#========================================================================================
sub _setResponse {
    my $self = shift;
    my ($hResponse) = @_;
    $self->{'HashResponse'} = $hResponse;
    return;
}
#========================================================================================
# function     getErrorMessage
# state        publi
#----------------------------------------------------------------------------------------
# syntax       getErrorMessage( );
#----------------------------------------------------------------------------------------
# description  getter for error message
#----------------------------------------------------------------------------------------
# return       Error message | string
#========================================================================================
sub getErrorMessage {
    my $self = shift;
    return $self->{'ErrorMessage'};
}
#========================================================================================
# function     _setErrorMessage
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setErrorMessage( $ErrorMessage );
#----------------------------------------------------------------------------------------
# description  setter for error message
#----------------------------------------------------------------------------------------
# input        $ErrorMessage | error message | string
#========================================================================================
sub _setErrorMessage {
    my $self = shift;
    my ($ErrorMessage) = @_;
    $self->{'ErrorMessage'} = $ErrorMessage;
    return;
}
#========================================================================================
# function     getErrorMgetErrorCodeessage
# state        publi
#----------------------------------------------------------------------------------------
# syntax       getErrorCode( );
#----------------------------------------------------------------------------------------
# description  getter for error code
#----------------------------------------------------------------------------------------
# return       Error code | string
#========================================================================================
sub getErrorCode {
    my $self = shift;
    return $self->{'ErrorCode'} ;
}
#========================================================================================
# function     _setErrorCode
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setErrorCode( $Code );
#----------------------------------------------------------------------------------------
# description  setter for error code
#----------------------------------------------------------------------------------------
# input        $Message | error code | string
#========================================================================================
sub _setErrorCode {
    my $self = shift;
    my ($Code) = @_;
    $self->{'ErrorCode'} = $Code;
    return;
}
#========================================================================================
# function     _createRestClient
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_createRestClient( $Object, $hArgs );
#----------------------------------------------------------------------------------------
# description  Webservice client factory method
#----------------------------------------------------------------------------------------
# input        $Object | This context object will be used for initializing
#                         Resource parameters for the webservice client. | object
# input        $hArgs | Constructor arguments. See <function new> for list of valid keys | ref.hash.*
# return       $Object | new object | DE_EPAGES::EPagesJ::API::Object::RestClient
#========================================================================================
sub _createRestClient {
    my $self = shift;
    my ($Object, $hArgs) = @_;
    return DE_EPAGES::EPagesJ::API::Object::RestClient->create($Object, $hArgs);
}
#========================================================================================
# function     _createLogger
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_createLogger( );
#----------------------------------------------------------------------------------------
# description  Create logger class
#----------------------------------------------------------------------------------------
# return       $Object | Logger object | DE_EPAGES::PayPalPlus::API::Logger
#========================================================================================
sub _createLogger {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Logger->new();
}
#========================================================================================
# function     _LogError
# state        private
#----------------------------------------------------------------------------------------
# syntax       _LogError( $Error );
# example      eval { ... };
#               if( ExistsError() ) {
#                   LogError( GetError() );
#               }
#----------------------------------------------------------------------------------------
# description  Wrapper for unit test. Writes an error to the log file.
#----------------------------------------------------------------------------------------
# input        $Error | an error object | object
# return       1 | always 1 | boolean
#========================================================================================
sub _LogError {
    my $self = shift;
    my ($Error) = @_;

    if($self->{'LogErrorsInComunicationlog'}) {
        my $Logger = $self->_createLogger();
        $Logger->logPayment('Response params', {'Response' => $Error->longMessage()});
    } else {
        LogError($Error);
    }
}
1;