#========================================================================================
# package      DE_EPAGES::PayPalPlus::UI::PaymentMethodPayPalPlus
# base         DE_EPAGES::Payment::UI::PaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# description  Controller for the PaymentMethod PayPalPlus
#========================================================================================
package DE_EPAGES::PayPalPlus::UI::PaymentMethodPayPalPlus;
use base DE_EPAGES::Payment::UI::PaymentMethod;
use DE_EPAGES::PayPalPlus::API::Logger;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::PayPalPlus::API::ContextLogger;
use DE_EPAGES::PayPalPlus::API::MerchantOnboardingURLBuilder;
use DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Service;
use DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use MIME::Base64;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );

use strict;

#========================================================================================
# function     ViewSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  ViewAction to display Merchant onboarding information and URL
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub ViewSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $MethodName    = ( caller(0) )[3];
    my $Context       = $self->_createLogContext();
    my $PaymentMethod = $Servlet->object();

    my $System = $self->_LoadRootObject();
    my $HasSSLCertificate = $System->get('HasSSLCertificate');
    if(not $HasSSLCertificate) {
        $Servlet->vars( 'NoValidSSLCertificateError', 1 );
    }

    $Context->logContext($PaymentMethod,$MethodName, sub {
        my $IsBoardingURLSet = $self->_setOnBoardingURL($Servlet);
        
        return unless ($IsBoardingURLSet);

        my $WasMerchantClientIdSet = $self->_setMerchantClientId($Servlet); #TODO: AD-3099 find a name pointing to the fact that we do a webservice call here
        return unless ($WasMerchantClientIdSet);

        $self->_createExperienceProfileId($Servlet);
    });

    return;
}

#========================================================================================
# function     SaveSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       SaveSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction to save user inputs into the database
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $MethodName = ( caller(0) )[3];
    my $Context = $self->_createLogContext();
    my $PaymentMethod = $Servlet->object();
    $Context->logContext($PaymentMethod, $MethodName, sub {
        $self->Save($Servlet);
    });

    return;
}

#----------------------------------------------------------------------------------------
# private
#----------------------------------------------------------------------------------------
sub _getReturnToPartnerUrl {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethodPayPalPlus = $Servlet->object();
    my $hOptions = { 'Type' => 'admin', 'UseSSL' => 1, 'AddToken' => 1 };
    my $hParams = { 'ViewAction' => 'MBO-ViewSettings' };

    my $URL = $self->_BuildShopUrl( $PaymentMethodPayPalPlus, $hParams, $hOptions, $Servlet );
    my $EOL = '';

    return $self->_encode_base64( $URL, $EOL );
}

#----------------------------------------------------------------------------------------
sub _createMerchantOnboardingURLBuilder {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::MerchantOnboardingURLBuilder->new();
}

#----------------------------------------------------------------------------------------
sub _createExperienceProfileId {
    my $self          = shift;
    my ($Servlet)     = @_;
    my $PaymentMethod = $Servlet->object();

    return if defined $PaymentMethod->get('ExperienceProfileId');    #Stop doing webservice call if the experienceProfile already exists

    my $ExperienceProfileIdService = $self->_createExperienceProfileIdService($PaymentMethod);
    $Servlet->vars( 'ExperienceProfileIdError', 0 );

    #TODO: AD-3099 combine doRequest + persistResponse into one method in the ExperienceProfileIdService
    if ( $ExperienceProfileIdService->doRequest() ) {
        if ( not $ExperienceProfileIdService->persistResponse() ) {
            #TODO: AD-3099 check if this error messages are still used
            $Servlet->vars( 'ExperienceProfileIdError', 1 );
            $Servlet->vars( 'ExperienceProfileIdErrorCode',    $ExperienceProfileIdService->getErrorCode() );
            $Servlet->vars( 'ExperienceProfileIdErrorMessage', $ExperienceProfileIdService->getErrorMessage() );
        }
    }
    else {
        #TODO: AD-3099 check if this error messages are still used
        $Servlet->vars( 'ExperienceProfileIdError', 1 );
        $Servlet->vars( 'ExperienceProfileIdErrorCode',    $ExperienceProfileIdService->getErrorCode() );
        $Servlet->vars( 'ExperienceProfileIdErrorMessage', $ExperienceProfileIdService->getErrorMessage() );
    }
    return;
}

#----------------------------------------------------------------------------------------
sub _createExperienceProfileIdService {
    my $self = shift;
    my ($PaymentMethod) = @_;
    return DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Service->new($PaymentMethod);
}

#----------------------------------------------------------------------------------------
sub _BuildShopUrl {
    my $self = shift;
    my ( $Object, $hParams, $hOptions, $Servlet ) = @_;
    return BuildShopUrl( $Object, $hParams, $hOptions, $Servlet );
}

#----------------------------------------------------------------------------------------
sub _encode_base64 {
    my $self = shift;
    my ( $Data, $EOL ) = @_;
    return encode_base64( $Data, $EOL );
}

#----------------------------------------------------------------------------------------
sub _createLogContext {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::ContextLogger->new();
}
#----------------------------------------------------------------------------------------
sub _createLogger{
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Logger->new();
}
#----------------------------------------------------------------------------------------
sub _setOnBoardingURL {
    my $self = shift;
    my ($Servlet) = @_;

    my $Logger = $self->_createLogger();
    my $PaymentMethod = $Servlet->object();
    my $MerchantOnboardingURLBuilder = $self->_createMerchantOnboardingURLBuilder();
    my $ReturnURL = $self->_getReturnToPartnerUrl($Servlet);
    my $WasSuccessful = $MerchantOnboardingURLBuilder->buildOnboardingURL( $PaymentMethod, $ReturnURL );
    if ($WasSuccessful) {
        $Servlet->vars( 'OnboardingURL', $MerchantOnboardingURLBuilder->getOnboardingURL() );
        return 1;
    } else {
        $Logger->logOnboarding({
            'Status' => 'failed to build Onboarding Url',
            'Error' => $MerchantOnboardingURLBuilder->getError()
        });
        $Servlet->vars( 'OnboardingError', 1 );
        $Servlet->vars( 'OnboardingErrorCode', $MerchantOnboardingURLBuilder->getError() );
    }

    return 0;
}

#----------------------------------------------------------------------------------------
sub _setMerchantClientId {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    my $Logger = $self->_createLogger();
    if (isValid($PaymentMethod->get('MerchantClientId'))){
        $Logger->logOnboarding({
            'Status' => 'The MerchantClientId is set. (Boarding completed)'
        });
        return 1;
    }else{
        $Logger->logOnboarding({
            'Status' => 'The MerchantClientId is not set. (Boarding is incomplete)'
        });
    }

    my $GetMerchantStatusService = $self->_createGetMerchantStatusService();
    my $WasSucessfulRequest = $GetMerchantStatusService->doRequest($PaymentMethod);
    if (not $WasSucessfulRequest) {
        $Servlet->vars( 'GetMerchantStatusError', 1 );
        $Servlet->vars( 'GetMerchantStatusErrorCode', $GetMerchantStatusService->getErrorCode() );
        $Servlet->vars( 'GetMerchantStatusErrorMessage', $GetMerchantStatusService->getErrorMessage() );
    }

    return $WasSucessfulRequest;
}

#----------------------------------------------------------------------------------------
sub _createGetMerchantStatusService {
    my $self = shift;

    my $GetMerchantStatusService = DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service->new();

    return $GetMerchantStatusService;
}

#========================================================================================
# function     _LoadRootObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = _LoadRootObject();
#----------------------------------------------------------------------------------------
# description  Wrapper for unit test. Returns the root object.
#----------------------------------------------------------------------------------------
# return       root object | DE_EPAGES::Object::API::Object::Site
#========================================================================================
sub _LoadRootObject {
    my $self = shift;
    return LoadRootObject();
}

1;
