use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use Config;
use Getopt::Long;
use DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler qw( GetCartridges );
use DE_EPAGES::Core::API::File qw (
    GetFileContent DeleteFile ExistsFile
    WriteFile ExistsDirectory CorrectPathSlashes
    CopyFile fsplit
);
use Time::HiRes qw (time);
use DE_EPAGES::Presentation::Scripts::Sprites::SpriteComposer;
use DE_EPAGES::Presentation::Scripts::Sprites::SpriteUtil qw( ReadConfigFile ReplaceMarkers );
use Cwd qw(abs_path);


sub _getConfigParameters {
    my ($CurrentPath, undef, undef) = fsplit(abs_path($0));
    my $hConfigValues = ReadConfigFile($CurrentPath.'/SpriteComposer.conf');
    return $hConfigValues;
}

sub _runSpriteGenerator {
    my %hParams = @_;

    my $cartridge_time1 = time;
    my $Sprite = DE_EPAGES::Presentation::Scripts::Sprites::SpriteComposer->new(%hParams);
    my $aResult = $Sprite->generateSpriteImage;
    undef $Sprite;
    my $cartridge_time2 = time;
    printf "done, need %.3f seconds\n", $cartridge_time2-$cartridge_time1;

    return $aResult;
}

sub _generateSprite {
    my ($Package, $hConfigParams, $GenerateTestHTMLFile, $EpagesCartridges, $StoreTypeRoot, $TargetFileName) = @_;

    my $CartridgeName = $Package;
    $CartridgeName =~ s/DE_EPAGES|:://g;

    my @TouchedFiles = ();

    my $IconsPath = ReplaceMarkers($hConfigParams->{'imagetargetdirectoryrelative'}, { 'cartridge' => $CartridgeName });

    if(ExistsDirectory($IconsPath)){
        print " [ok] for cartridge: $CartridgeName ... ";
        my $aResult = _runSpriteGenerator(
            'configfilevalues'  => $hConfigParams,
            'imagedir'          => $IconsPath,
            'generatetest'      => $GenerateTestHTMLFile,
            'cartridge'         => $CartridgeName,
            'de_epages_dir'     => $EpagesCartridges.'/DE_EPAGES',
            'storetyperoot'     => $StoreTypeRoot,
            'targetfile'        => $TargetFileName
        );
        push(@TouchedFiles, @$aResult) if defined @$aResult;
    } else {
        print " [--] cartridge $CartridgeName has no directory $hConfigParams->{'imagetargetdirectoryrelative'}, skipping.\n";
    }

    my $TestAdditionalDir = lc($CartridgeName.'_imagetargetdirectoryrelative');
    if(defined $hConfigParams->{$TestAdditionalDir}) {
        my $UseImageDir = ReplaceMarkers($hConfigParams->{$TestAdditionalDir}, { 'cartridge' => $CartridgeName } );

        print " [ok] for cartridge: $CartridgeName ... ";
        my $aResult = _runSpriteGenerator(
            'configfilevalues'  => $hConfigParams,
            'imagedir'          => $UseImageDir,
            'generatetest'      => $GenerateTestHTMLFile,
            'cartridge'         => $CartridgeName,
            'de_epages_dir'     => $EpagesCartridges.'/DE_EPAGES',
            'storetyperoot'     => $StoreTypeRoot,
            'targetfile'        => $TargetFileName
        );
        push(@TouchedFiles, @$aResult) if defined @$aResult;
    }
    return @TouchedFiles;
}

sub Main {
    my ($Help, $ImageDirectory, $TargetFileName, $Cartridge, $Type, $GenerateTestHTMLFile, $OnlyCombineCss, $Version);

    GetOptions(
        'help'           => \$Help,
        'imagedir=s'     => \$ImageDirectory,
        'targetfile=s'   => \$TargetFileName,
        'cartridge=s'    => \$Cartridge,
        'generatetest'   => \$GenerateTestHTMLFile,
        'type=s'         => \$Type,
        'onlycombinecss' => \$OnlyCombineCss,
        'version=s'      => \$Version
    );
    $GenerateTestHTMLFile = 'true' if defined $GenerateTestHTMLFile;

    my $hConfigParams = _getConfigParameters();
    usage($hConfigParams) if $Help or not $Type or not $Version;

    my $StoreTypeRoot = join('/',$ENV{'EPAGES_WEBROOT'}, 'StoreTypes', $Version, $Type);

    my $time1 = time;

    # Cartridges/DE_EPAGES dir
    my $EpagesCartridges = CorrectPathSlashes($ENV{'EPAGES_CARTRIDGES'});

    my @TouchedFiles = ();
    my $OnlyOneCartridge = not defined $Cartridge ? 0 : 1;

    # get list of cartridges
    my $aCartridges = GetCartridges();

    if($OnlyCombineCss eq '')
    {
        # default action
        if(not $OnlyOneCartridge) {
            print "Applying SpriteComposer to all cartridges ...\n";

            # for all cartridges
            foreach my $Package ( @{$aCartridges->{$Type}} ){
                @TouchedFiles = (
                    @TouchedFiles, _generateSprite(
                        $Package,
                        $hConfigParams,
                        $GenerateTestHTMLFile,
                        $EpagesCartridges,
                        $StoreTypeRoot,
                        $TargetFileName
                    )
                );
            }
        } else {
            print "Applying SpriteComposer only to cartridge $Cartridge ...\n";
            @TouchedFiles = (
                @TouchedFiles, _generateSprite(
                    $Cartridge,
                    $hConfigParams,
                    $GenerateTestHTMLFile,
                    $EpagesCartridges,
                    $StoreTypeRoot,
                    $TargetFileName
                )
            );
        }
    }

    my $CombTargetCssFile = CorrectPathSlashes(ReplaceMarkers(
            $hConfigParams->{'defaultcombinespritecssfile'},
            { 'storetyperoot' => $StoreTypeRoot }
    ));
    print 'Target CSS=', $CombTargetCssFile, "\n"  if $hConfigParams->{'debug'} eq 'true';
    # If only one cartridge was changed and the target css file already exists. 
    if($OnlyOneCartridge and ExistsFile($CombTargetCssFile)) {
        # do not include dojo widget css in combined css file
        if(not $Cartridge =~ /Dojo/i) {
            # Replace old css stuff of $Cartridge Cartridge in $hConfigParams->{'defaultcombinespritecssfile'}
            print "Replace old css code of $Cartridge cartridge in $hConfigParams->{'defaultcombinespritecssfile'}\n"
            if $hConfigParams->{'debug'} eq 'true';
            my $TargetCSSContent = GetFileContent($CombTargetCssFile);
            # SourceCSSPath
            my $SourceCSSPath = ReplaceMarkers($hConfigParams->{'csstargetdirectoryrelative'}, {'cartridge' => $Cartridge});
            my $SourceCSSName = $hConfigParams->{'cssfileprefix'} . $Cartridge;
            my $SourceCSSFilePath = CorrectPathSlashes(join(CorrectPathSlashes('/'),$SourceCSSPath, $SourceCSSName.'.css'));

            my $SourceCSSContent = "/* $Cartridge */\n${GetFileContent($SourceCSSFilePath)}\n/*";
            $$TargetCSSContent =~ s/\/\*\s*$Cartridge.*?\/\*/$SourceCSSContent/sx;

            WriteFile(
                $CombTargetCssFile,
                $TargetCSSContent,
                {'append' => 0, 'binary' => 0}
            );
        }
    } else {
        # combine all css files to one in webroot dir
        print "Combining css files to one large ... \n" if $hConfigParams->{'debug'} eq 'true';

        DeleteFile($CombTargetCssFile) if ExistsFile($CombTargetCssFile);

        # for all cartridges
        foreach my $Package ( @{$aCartridges->{$Type}} ){
            my $CartridgeName = $Package;
            $CartridgeName =~ s/DE_EPAGES|::|\\|\///g;

            # do not include dojo widget css in combined css file
            next if $CartridgeName =~ /Dojo/i;

            $Package =~ s/::/\//g;

            # path
            my $SourceCSSPath = ReplaceMarkers($hConfigParams->{'csstargetdirectoryrelative'}, {'cartridge' => $CartridgeName});
            my $SourceCSSName = $hConfigParams->{'cssfileprefix'} . $CartridgeName;
            my $SourceCSSFilePath = CorrectPathSlashes(join(CorrectPathSlashes('/'),$SourceCSSPath, $SourceCSSName.'.css'));
            next if !ExistsFile($SourceCSSFilePath);

            my $Topic = "\n \/* ".$CartridgeName." *\/\n";
            WriteFile(
                $CombTargetCssFile,
                \$Topic,
                {'append' => 1, 'binary' => 0}
            );
            WriteFile(
                $CombTargetCssFile,
                GetFileContent($SourceCSSFilePath),
                {'append' => 1, 'binary' => 0}
            );
        }
    }
    push(@TouchedFiles, $CombTargetCssFile);
    print "Combining Done.\n" if $hConfigParams->{'debug'} eq 'true';

    # copy to webroot?
    if($hConfigParams->{'copycombinespritecsstowebroot'} eq 'true' and ExistsFile($CombTargetCssFile)) {

        my $WebrootTarget = ReplaceMarkers($hConfigParams->{'defaultwebrootcombinespritecssfile'},
            {'storetyperoot' => $StoreTypeRoot }
        );

        CopyFile($CombTargetCssFile, $WebrootTarget) if defined $WebrootTarget;
        push(@TouchedFiles, $WebrootTarget);
    }


    #summary
    print "\n\nSummary: these files were created / changed:\n\n";
    foreach my $TouchedFile (sort @TouchedFiles) {
        print '    ', $TouchedFile, "\n";
    }
    printf "\n\n";

    my $time2 = time;
    printf "need %.3f seconds\n", $time2-$time1;
}

sub usage {
    my ($hConfigParams) = @_;
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

Description:
    Combines all images in the directory [Cartridge]/Data/Public/BO/icons/ matching the pattern
    defined in the configuration file as slices into one image file given. Thus you will get
    one large image file with the sprites for each cartridge.
    You can specify to perform the action only for one cartridge using the -cartridge parameter.
    Furthermore you can perform the action only for a specific directory given with the -imagedir
    parameter. You still have to name the cartridge using -cartridge for the resulting file and
    class names.
    If no target file name (-targetfile) is given, the name is composed as:
    img_sprite_[Cartridge].png.

    In addition to the composed image, a CSS stylesheet file of the same filename is generated
    containing the pixel coordinates to address the particular slices. An additional html file
    for testing is generated if the -generatetest parameter is used. For further settings see
    the config file.

    This script does only work with PNG images!


Options for performing ALL cartridges:
    -type               database type                                               (required)
    -version            store types version number                                  (required)


Options for performing ONE cartridge:
    -type               database type                                               (required)
    -version            store types version number                                  (required)
    -imagedir           compose sprite image only for the images in that directory  (optional)
    -cartridge          compose sprite image only for that cartridge                (optional)
    -targetfile         file name of the resulting sprite image                     (optional)


Options for just combining the current css files:
    -type               database type                                               (required)
    -version            store types version number                                  (required)
    -onlycombinecss     just combine the existing css sprite files                  (optional)


Flags:
    -help               show the command line options                               (optional)
    -generatetest       generate test html file for all images                      (optional)


Examples:
    perl $0 -type Store -version 6.10.1
    (for all cartridges)

    perl $0 -type Store -cartridge Presentation -version 6.10.1
    (only for Presentation cartridge)

    perl $0 -type Store -imagedir "D:\\Temp\\Beispiel\\" -targetfile \"beispiel.png\" -cartridge Presentation  -version 6.10.1
    (a specific directory with specific target file name and file/class naming Presentation)
END_USAGE
exit 1;
}

RunScript(
    'Sub' => \&Main
);