#========================================================================================
# SCRIPT WHICH SEARCHES FOR UNUSED HTML AND XML FILES
#========================================================================================

use strict;
use Getopt::Long;
use DE_EPAGES::Cartridge::API::Table::Cartridge     qw( GetInfoCartridge GetAllInfoCartridges);
# not really required, fixes warning for : $DictionaryFile =~ /^Error\./i
use DE_EPAGES::Core::API::Error                     qw( Error );##no critic(RequireCorrectImports)
use DE_EPAGES::Core::API::File                      qw( GetFileNames ExistsDirectory);
use DE_EPAGES::Core::API::Script                    qw( RunScript );
use DE_EPAGES::Database::API::Connection            qw( RunOnStore );
use DE_EPAGES::Object::API::Factory                 qw( LoadClassByAlias );
use Test::More;

sub Main {

    my $StoreName;
    my $Password;
    my $Help;

    GetOptions(
        'help' => \$Help,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,
    );

    usage() if $Help;
    unless($StoreName) {
        print STDERR "Missing parameter -storename\n\n";
        usage();
    }

    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub  { FindUnusedTemplates() },
    );
    return;
}

sub FindUnusedTemplates {

    my $hTEMPFiles = GetTemplateFiles();    #--- from database

    my $aFiles = GetCartridgeTemplateFiles();  #--- from epages directory
    my @TemplateFiles;
    my @LocalizationFiles;
    foreach (@$aFiles) {
        if( /\.\w\w\.xml$/ ) {
            push @LocalizationFiles, $_;
        }
        elsif( /\.(html|txt|js|json)$/ ) {
            push @TemplateFiles, $_;
        }
    }

    plan tests => scalar(@LocalizationFiles) + scalar(@TemplateFiles);

    foreach my $path (@TemplateFiles) {
        SKIP: {
            skip "exception: $path", 1 if
                   $path =~ m!/Presentation/Templates/Simple!
                || $path =~ m!/Presentation/Templates/BasePageType.Empty.html!
                || $path =~ m!/CSVExportImport/Templates/CSVImportMonitor\.Message\.html!
                || $path =~ m!/Ebay/Templates/Ebay/!
                || $path =~ m!/Cartridge/Templates/Example!
                || $path =~ m!/Design/Templates/SF/SF-Shop.Content.html!    # overload by DE_EPAGES::Content
                || $path =~ m!/Design/Templates/SF/SF-Login.Content.html!   # overload by DE_EPAGES::Customer
                || $path =~ m!/ShopMailType/Templates/UnitySF/UnitySF.Header.html!   # overload by DE_EPAGES::Design
                || $path =~ m!/ShopMailType/Templates/UnitySF/UnitySF.Footer.html!   # overload by DE_EPAGES::Design
                || $path =~ m!/Saferpay/Templates/SaferpayDescription.txt!
                || $path =~ m!/PayPointPayCash/Templates/AmountRangeMessage.txt!
                || $path =~ m!/ShopMailType/Templates/MBO/TestEmailSentContent.txt!
                || $path =~ m!/UPS/Templates/MBO/TranslatedLinks/!;

            ok( exists $hTEMPFiles->{$path}, "template $path is used" );
        };
    }

    my %FilePrefixes = map { s/\.[^.]+$//; $_ => 1; } @TemplateFiles;
    foreach my $XMLPath (@LocalizationFiles) {
        next if $XMLPath !~ /\/Templates\//;
        my $DictionaryFile = $'; #'
        SKIP: {
            skip "exception: $XMLPath", 1 if
                   $DictionaryFile =~ /^Dictionary\./i
                || $DictionaryFile =~ /^Error\./i;

            my $StartPath = $XMLPath;
            $StartPath =~ s/\.\w\w\.xml$//;

            ok( exists $FilePrefixes{$StartPath}, "xml file $XMLPath is used" );
        }
    }

    return;
}

#========================================================================================
# function     GetTemplateFiles
#----------------------------------------------------------------------------------------
# syntax       $hTEMPFiles = GetTemplateFiles();
#----------------------------------------------------------------------------------------
# description  Gets template file names which are stored in the database.
#----------------------------------------------------------------------------------------
# return       $hTEMPFiles   | contains the path names of the template files,
#               relative to the directory $ENV{EPAGES_CARTRIDGES} | reference to hash
#========================================================================================
sub GetTemplateFiles {
    my %TEMP;
    my $ahTemplateTypes = LoadClassByAlias('TemplateType')->infoObjects();
    foreach my $hTemplateType (@$ahTemplateTypes) {
        my $hCartridge = GetInfoCartridge($hTemplateType->{'CartridgeID'});
        my @PartOfTempFileName = split /::/, $hCartridge->{'Package'};
        my $RelativeTempPath = "$PartOfTempFileName[0]/$PartOfTempFileName[1]/Templates/".$hTemplateType->{'FileName'};
        $TEMP{$RelativeTempPath} = '1';
    }
    return \%TEMP;
}

#========================================================================================
# function     GetCartridgeTemplateFiles
#----------------------------------------------------------------------------------------
# syntax       $aFiles = GetCartridgeTemplateFiles();
#----------------------------------------------------------------------------------------
# description  Gets the names of all "Templates" files in the
#               $ENV{EPAGES_CARTRIDGES} directory.
#----------------------------------------------------------------------------------------
# return       $aFiles | list of names | ref.array.string
#========================================================================================
sub GetCartridgeTemplateFiles {
    my $ahCartridges = GetAllInfoCartridges();

    my @Files;
    foreach my $Package (
        grep {!/CodeGenerator|SourceDoc|Diagnostics/}
        map { $_->{'Package'} }
        grep { $_->{'InstallState'}}
        @$ahCartridges
    ) {
        my @PartOfTempFileName = split /::/, $Package;
        my $TemplatePath = $ENV{EPAGES_CARTRIDGES}."/$PartOfTempFileName[0]/$PartOfTempFileName[1]/Templates";
        push @Files, @{GetFileNames( $TemplatePath, {'recursive' => 1} )} if ExistsDirectory($TemplatePath);
    }
    return [ sort map { (split /Cartridges\//i)[1] } @Files ];
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -passwd     database user password              ()
    -storename  name of store                       ()

flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
    Debug => 1
);
