#==============================================================================
# package      DE_EPAGES::Product::UI::AppointmentFolder
# state        private
#==============================================================================
package DE_EPAGES::Product::UI::AppointmentFolder;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Content::API::ShortUrl qw (GetShortUrlByClass);
use DE_EPAGES::Core::API::Error qw (GetError ExistsError Error);
use DE_EPAGES::Core::API::Log qw (LogDebug);
use DE_EPAGES::Core::API::PerlTools qw (GetCurrentDateTime);
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Search::API::Object::PagerSearch qw (ReplaceWildcards ReplaceWildcardsWithoutPosOne);
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);
use DE_EPAGES::ShopMailType::API::ShopMailType qw( GetShopMailType );

sub SearchAppointments {
    my $self = shift;
    my $Servlet = shift;
    my $Folder = $Servlet->object;
    my $SearchForm = $Servlet->form->form($Folder, 'SearchAppointments');
    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;

    my $SearchName = $SearchForm->{'Search'} || 'MBO-Appointments';
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('Appointment'), $SearchName);
    my $LocalizedAttributeName = LoadClassByAlias('Product')->attribute('Name');
    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Folder,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'   => $Folder->getSite()->id,
            'SearchLanguageID' => $Servlet->languageID,
            'NameAttributeID'  => $LocalizedAttributeName->id,
        },
        'UriParameters'  => {
            'Search' => $SearchName,
        },
    );
    foreach my $Parameter (keys %$SearchForm) {
        if ( $Parameter eq 'ProductAliasString' ) {

            ## UNITY-4143
            #replace '*' with '%' and add % to the end for (Alias/Productnumber)
            $Options{'Parameters'}{$Parameter} = ReplaceWildcardsWithoutPosOne($SearchForm->{$Parameter});
        } elsif ($Parameter =~ /String/) {
            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards($SearchForm->{$Parameter});

        } elsif ($Parameter =~ /^Is(.*)$/) {
            # status = 0/1/2 egal/yes/no
            if ($SearchForm->{$Parameter} == 1) {
                $Options{'Parameters'}{$Parameter} = 1;
            } elsif ($SearchForm->{$Parameter} == 2) {
                $Options{'Parameters'}{$Parameter} = 0;
            }
        } else {
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
        $Options{'UriParameters'}{$Parameter} = $SearchForm->{$Parameter};
    }
    $Servlet->vars('OrderBy', 'StartDate') unless defined $Servlet->params('OrderBy');

    if (defined $SearchForm->{'IsCancelled'}) {
        $Options{'UriParameters'}{'IsCancelled'} = $SearchForm->{'IsCancelled'};
        if ($SearchForm->{'IsCancelled'} == 2) {
            delete $Options{'Parameters'}{'IsCancelled'};
            $Options{'Parameters'}{'NotCancelled'} = 1;
        } else {
            $Options{'Parameters'}{'IsCancelled'} = 1;
        }
    }

    delete $SearchForm->{'Search'};
    LogDebug('Options', \%Options);

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

sub AcceptAppointments {
    my $self = shift;
    my $Servlet = shift;
    my $aWarnings = BlockWarnings( sub {
        $self->SaveOnList($Servlet, "AcceptedOn", GetCurrentDateTime());
    }, ['MAIL_SENT']);

    $Servlet->vars('MAIL_SENT', scalar @$aWarnings ? 1 : 0);
    return;
}

sub CancelAppointments {
    my $self = shift;
    my $Servlet = shift;
    # nothing to do
    return if ! $Servlet->params('ListObjectID');


    # cancel only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        my $MasterObject = $Servlet->object;
        my $Form = $Servlet->form;
        my $hValues = $Form->form($MasterObject, 'ListedObjects');
        my $Shop = $MasterObject->getSite();

        my @ListObjects = map  { $Shop->loadObject($_->{'ListObjectID'}, 'Appointment') }
                          @{$hValues->{'ListObjectIDs'}};
        return unless scalar @ListObjects;
        $Servlet->vars('DeleteObjects', [ map { $_->tleHash } @ListObjects]);
        $Servlet->vars('CANCEL_APPOINTMENT',1);
        return;
    } else {
        # else reset delete
        $Servlet->vars('CANCEL_APPOINTMENT',0);
    }

    my $aWarnings = BlockWarnings( sub {
        $self->SaveOnList($Servlet, "CancelledOn", GetCurrentDateTime());
    }, ['MAIL_SENT']);

    $Servlet->vars('MAIL_SENT', scalar @$aWarnings ? 1 : 0);
    return;
}

sub Save {
    my $self = shift;
    my $Servlet = shift;

    my $ProductFolder = $Servlet->object;
    my $hForm = $Servlet->form->form($ProductFolder, 'NewProduct');
    my %Values = map { s/^New//; $_ => $hForm->{'New'.$_} } grep { /^New/ && defined $hForm->{$_} } keys %$hForm;

    my $Name = $Values{'Name'};
    $Servlet->form->executeFormError({
        'Reason' => 'NoName',
        'Name'   => 'NewName',
        'Form'   => 'NewProduct',
    }) unless defined $Name;

    my $ListPrice = $Values{'ListPrice'};
    delete $Values{'ListPrice'};

    my $Shop = $ProductFolder->getSite();
    $Values{'Parent'} = $ProductFolder;
    $Values{'Alias'} = $ProductFolder->uniqueChildAlias($Name);
    $Values{'URI'} = GetShortUrlByClass($ProductFolder, $Shop->get('ProductClass'), $Name) if defined $Name and $Name ne '';

    my $Class = $Shop->get('ProductClass');
    eval {
        my $Product = $Class->insertObject(\%Values);
        $Product->setListPrice( $Shop->get('CurrencyID'), $Shop->get('InputTaxModel'), $ListPrice ) if defined $ListPrice;
    };
    if (ExistsError()) {
        my $Error = GetError();
        $Servlet->form->executeFormError({
            'Reason'=> 'FeatureMaxValueReached',
            'Form'  => 'NewProduct',
            'Name'  => 'NewName',
        }) if $Error->code eq 'FeatureLimitExceeded';
        Error($Error);
    }

    # activate mailtypes (EPG-11524)
    my $CountAppointments = scalar @{$ProductFolder->children};
    if ($CountAppointments == 1) {
        foreach my $MailTypeAlias (qw (AppointmentCreated AppointmentAccepted AppointmentCancelled) ) {
            my $MailType = GetShopMailType( $Shop, $MailTypeAlias );
            next unless defined $MailType;
            unless ($MailTypeAlias eq 'AppointmentAccepted') {
                my $BCC = $MailType->get('BCC');
                $MailType->set({ 'BCC' =>  $Shop->get('MailFrom') }) unless defined $BCC;
            }
            $MailType->set({'IsActive' => 1});
        }
    }
    return;
}

1;
