﻿#==============================================================================
# §package      DE_EPAGES::Product::UI::ProductFolder
# §state        private
#==============================================================================
package DE_EPAGES::Product::UI::ProductFolder;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use utf8;
use DE_EPAGES::Content::API::ShortUrl qw ( GetShortUrlByClass );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias Transaction );
use DE_EPAGES::Object::API::Table::Currency qw ( GetInfoCurrency );
use DE_EPAGES::Product::API::Constants qw ( MBOUI_PRODUCT_DETAIL_PAGE );
use DE_EPAGES::Search::API::Object::PagerSearch qw ( ReplaceWildcards ReplaceWildcardsWithoutPosOne );
use DE_EPAGES::Search::API::SearchDefinition qw ( GetSearchDefinition );
use DE_EPAGES::Search::API::Table::SearchSort qw ( ExistsSearchSortBySearchDefinitionID );
use DE_EPAGES::Shop::API::Shop qw ( GetNextNo );
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use Readonly;
use feature 'switch';

sub ViewNewProduct {
    my $self = shift;
    my ($Servlet) = @_;

    my $ProductKind = $Servlet->params('Kind') // 'NewProduct';
    my $ContainerID = $Servlet->params('Container');

    if(defined $ContainerID) {
        my $hParams = {
            'Object' => $Servlet->object,
            'ContainerID' => $ContainerID,
            'ContainerNameOrAlias' => undef,
        };
        TriggerHook('UI_BeforeViewNewProduct', $hParams);
        if(not defined $hParams->{'ContainerNameOrAlias'}) {
            $Servlet->vars('Container', undef);
        }
        else {
            $Servlet->vars('Container', {'ID' => $hParams->{'ContainerID'}, 'NameOrAlias' => $hParams->{'ContainerNameOrAlias'}});
        }
    }

    given($ProductKind) {
        when(/Bundle/) {
            $Servlet->vars('NEW_BUNDLEPRODUCT', 1);
        }
        when(/Download/) {
            $Servlet->vars('NEW_DOWNLOADPRODUCT', 1);
        }
        when(/Variation/) {
            $Servlet->vars('NEW_VARIATIONPRODUCT', 1);
        }
        when(/Customizable/) {
            $Servlet->vars('NEW_CUSTOMIZABLEPRODUCT', 1);
        }
        default {
            $Servlet->vars('NEW_PRODUCT', 1);
        }
    }

    $Servlet->params('MBOUI_ProductDetailPage', MBOUI_PRODUCT_DETAIL_PAGE);
    $Servlet->vars('Class', LoadClassByAlias('Product'));

    return $self->ViewNewObject($Servlet);
}

sub ViewUnityNewProduct {
    my $self = shift;
    my ($Servlet) = @_;
    my %Params = ();

    my $ProductFolder = $Servlet->object;
    my $Shop = $ProductFolder->getSite;

    # load new product default attributes
    $Servlet->vars('Class', LoadClassByAlias('Product'));
    my %DefaultParams = %{$self->ViewNewObject($Servlet)};

    # can the merchant add a new product?
    $Params{'ShopProductsMaxedOut'} = $Shop->get('FeatureCurrentValue')->{'Products'} >= $Shop->get('FeatureMaxValue')->{'Products'} ? 1 : 0;

    # new variation product?
    $Params{'IsVariationProduct'} = (defined $Servlet->params('Kind') and $Servlet->params('Kind') =~ m/Variation/) ? 1 : 0;

    # set product visible by default
    $Params{'IsVisible'} = 1;

    # container handling
    my $ContainerID = $Servlet->params('Container');

    if (defined $ContainerID) {
        my $hHookParams = {
            'Object' => $Servlet->object,
            'ContainerID' => $ContainerID,
            'ContainerNameOrAlias' => undef,
        };

        TriggerHook('UI_BeforeViewNewProduct', $hHookParams);

        if (defined $hHookParams->{'ContainerNameOrAlias'}) {
            $Params{'Container'} = {'ID' => $hHookParams->{'ContainerID'}, 'NameOrAlias' => $hHookParams->{'ContainerNameOrAlias'}};
            $Params{'ChangeObjectID'} = $hHookParams->{'ContainerID'};
        }
    }

    unless (defined $Params{'ChangeObjectID'}) {
        $Params{'Container'} = undef;
        $Params{'ChangeObjectID'} = $ProductFolder->id;
    }

    # currency
    my $CurrencyID = $Shop->get('CurrencyID');

    $Params{'ShopCurrencyID'} = $CurrencyID;
    $Params{'ShopCurrencySymbol'} = GetInfoCurrency($CurrencyID)->{'Symbol'};

    # last order unit as new default
    if (defined $Shop->get('LastOrderUnitForProductCreation')) {
        $Params{'OrderUnitID'} = $Shop->get('LastOrderUnitForProductCreation')->id;
    }
    else {
        $Params{'OrderUnitID'} = $DefaultParams{'OrderUnit'}->{'ObjectID'};
    }

    $Servlet->params('IsNewProduct', 1);

    # use an increasing number as alias
    $Params{'Alias'} = GetNextNo($Shop, 'Product', $ProductFolder, 0);

    # return a new product
    return { %DefaultParams, %Params };
}

sub ViewPagedChildren {
    my $self = shift;
    my ($Servlet) = @_;
    my $Folder = $Servlet->object;
    my $ProductClass = LoadClassByAlias('Product');
    my $SearchDefinition = GetSearchDefinition($ProductClass, 'MBO-AllStrings');
    my $LocalizedAttributeName = $ProductClass->attribute('Name');
    my $Shop = $Folder->getSite();
    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Folder,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'   => $Shop->id,
            'Folder.ID' => $Folder->id,
            'IsNotSubProduct' => 1,
            'SearchLanguageID' => $Servlet->languageID,
            'NameAttributeID' => $LocalizedAttributeName->id,
            'CurrencyID' => $Shop->get('CurrencyID'),
        },
    );
    my $OrderBy = $Servlet->params('OrderBy');
    $Servlet->vars('OrderBy', 'Alias') unless defined $OrderBy;

    LogDebug('Options', \%Options);

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);

    if (defined $Servlet->params('JumpToProductID')){
        my $Page = $Search->pageOfItemByObjectID($Servlet->params('JumpToProductID'));
        $Search->set({'Page' => $Page}) if defined $Page;
    }

    return $Search->page;
}

sub CloneProducts {
    my $self = shift;
    my ($Servlet) = @_;

    my $MasterObject = $Servlet->object;
    my $Form = $Servlet->form;
    my $hValues = $Form->form($MasterObject, 'ListedObjects');
    my $Shop = $MasterObject->getSite();

    my @ListObjects;
    foreach my $hListObjectID (@{$hValues->{'ListObjectIDs'}}){
        my $ListObject = $Shop->loadObject( $hListObjectID->{'ListObjectID'}, 'Product');
        $Servlet->testPermission($ListObject, 'Save');
        push @ListObjects,$ListObject;
    }

    my $Index = -1;
    RunBatchProcess('CloneProducts', sub {
        foreach my $Object (@ListObjects) {
            $Index++;

            if ($Object->get('SuperProduct')) {
                $Form->addFormError( {
                    Reason   => 'CantCloneSubProduct',
                    Index    => $Index,
                    LoopName => 'ListObjectIDs',
                    Form     => 'ListedObjects',
                    Name     => 'ListObjectID',
                    Product  => $Object->tleHash,
                } );
                next;
            }
            elsif (not $Object->canClone()) {
                $Form->addFormError( {
                    Reason   => 'CantCloneProduct',
                    Index    => $Index,
                    LoopName => 'ListObjectIDs',
                    Form     => 'ListedObjects',
                    Name     => 'ListObjectID',
                    Product  => $Object->tleHash,
                } );
                next;
            }
            my $NewAlias = $self->_generateNewAlias($Object);
            # clone the product and set cloned product and subproducts invisible
            $self->_cloneProduct($Servlet, $Object, $NewAlias);
        }
    });
    $Form->executeFormError;
    return;

}

sub _cloneProduct {
    my $self = shift;
    my ($Servlet, $Object, $NewAlias) = @_;
    eval {
        Transaction( sub {
            my $Clone = $Object->clone({'Alias'=> $NewAlias, 'Inherit' => 1,'IsVisible' => 0,'URI' => undef},1);

            if( $Clone->existsChild('SubProducts') ) {
                my $SubProducts = $Clone->child('SubProducts')->children;

                #set invisible and the alias to <alias>[x]
                foreach my $SubProduct (@$SubProducts){
                    my $SubAlias = $self->_generateNewAlias($SubProduct);
                    $SubProduct->set({'Alias' => $SubAlias, 'IsVisible' => 0});
                }
            }
            my %HookVars = ('Object' => $Object,'Clone' => $Clone);
            TriggerHook('AfterProductClone',\%HookVars);
        });
    };

    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code  eq 'FeatureLimitExceeded'){
            $Servlet->vars('FeatureMaxValueReached', $Error->vars);
            Warning($Error);
        } else {
            Error($Error);
        }
    }
}

sub _generateNewAlias {
    my $self = shift;
    my ($Object) = @_;
    my $NewAlias = $Object->alias;
    my $cnt = 1;

    # test if Alias has already an ext like [1] and replace ext if exists
    if ($NewAlias =~ /.*\[\d{1,}\]$/gx){
        while ($Object->parent->existsChild($NewAlias)){
            $cnt++;
            $NewAlias =~ s/(\[)\d{1,}(\])$/$1$cnt$2/gx;
        }
    }else{
        while ($Object->parent->existsChild($NewAlias."[$cnt]")){
            $cnt++;
        }
        $NewAlias .= "[$cnt]";
    }
    return $NewAlias;
}

sub SetProductsVisible {
    my $self = shift;
    my ($Servlet) = @_;
    my $Object  = $Servlet->object();
    my $Shop = $Object->getSite();
    my $IsUnityRegistered = $Shop->get('IsUnityRegistered');

    if($IsUnityRegistered){
        #add sub products to list
        my @newListIds = _addProductsAndSubProducts($Servlet, $Shop);

        #save new objects
        $self->_saveOnListFromHash($Servlet, $Shop, 'IsVisible', 1, @newListIds);
    } else {
        RunBatchProcess('UpdateProducts', sub {
            $self->SaveOnList($Servlet, 'IsVisible', 1);
        });
    }

    return;
}

sub SetProductsNotVisible {
    my $self = shift;
    my ($Servlet) = @_;
    my $Object  = $Servlet->object();
    my $Shop = $Object->getSite();
    my $IsUnityRegistered = $Shop->get('IsUnityRegistered');

    if($IsUnityRegistered){
        #add sub products to list
        my @newListIds = _addProductsAndSubProducts($Servlet, $Shop);

        #save new objects
        $self->_saveOnListFromHash($Servlet, $Shop, 'IsVisible', 0, @newListIds);
    } else {
        RunBatchProcess('UpdateProducts', sub {
            $self->SaveOnList($Servlet, 'IsVisible', 0);
        });
    }

    return;
}

sub _addProductsAndSubProducts {
    my ($Servlet, $Shop) = @_;
    my $Object  = $Servlet->object();

    my $hValues = $Servlet->form->form($Object, 'ListedObjects');
    my @ListObjectIds;

    # loop over object ids and add ids of subproducts if exist
    foreach my $hListObjectID (@{$hValues->{'ListObjectIDs'}}){
        my $ListObject = $Shop->loadObject( $hListObjectID->{'ListObjectID'}, 'Product');

        #if product has sub products add Subproducts IDs, otherwise add product ID
        if ($ListObject->get('IsMaster')) {
            my $SubProducts = $ListObject->child('SubProducts')->children;

            # add all subproduct ids
            foreach my $SubProduct (@$SubProducts){
                push @ListObjectIds, {'ListObjectID' => $SubProduct->id};
            }
        } else {
            push @ListObjectIds, {'ListObjectID' => $ListObject->id};
        }
    }

    return @ListObjectIds;
}

sub _saveOnListFromHash {
    my $self = shift;
    my ($Servlet, $Shop, $Name, $Value, @filteredListIds) = @_;

    my @ListObjects;
    foreach my $hListObjectID (@filteredListIds){
        #get Object
        my $ListObject = $Shop->loadObject( $hListObjectID->{'ListObjectID'}, 'Product');
        #check save permission
        $Servlet->testPermission($ListObject, 'Save');
        push @ListObjects,$ListObject;
    }

    #save objects and invalidate page cache
    if(scalar(@ListObjects) > 0) {
        $_->set( { $Name => $Value } ) foreach @ListObjects;
        # EPG-10681,EPG-11046:
        $Shop->invalidatePageCache;
    }
    return;
}

sub SetProductsNew {
    my $self = shift;
    my ($Servlet) = @_;
    RunBatchProcess('UpdateProducts', sub {
        $self->SaveOnList($Servlet, 'IsNew', 1);
    });
    return;
}

sub SetProductsNotNew {
    my $self = shift;
    my ($Servlet) = @_;
    RunBatchProcess('UpdateProducts', sub {
        $self->SaveOnList($Servlet, 'IsNew', 0);
    });
    return;
}

sub DeleteListedObjects {
    my $self = shift;
    my ($Servlet) = @_;
    RunBatchProcess('DeleteProducts', sub {
        $self->SUPER::DeleteListedObjects($Servlet);
    });
    return;
}

sub SearchProducts {
    my $self = shift;
    my ($Servlet) = @_;

    my $hOptions = $self->_performSearch($Servlet);

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new($hOptions);
    return $Search->page;
}

sub UnitySearchProducts {
    my $self = shift;
    my ($Servlet) = @_;

    my $hOptions = $self->_performSearch($Servlet);

    # set search language as output language for Unity
    $Servlet->vars('AltLanguageID', $hOptions->{'Parameters'}->{'SearchLanguageID'});

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new($hOptions);
    return $Search->page;
}

sub _performSearch {
    my $self = shift;
    my ($Servlet) = @_;
    my $Folder = $Servlet->object;

    my $SearchName = 'MBO-AllStrings';
    my $ProductClass = LoadClassByAlias('Product');
    my $SearchDefinition = GetSearchDefinition($ProductClass, $SearchName);

    # EPG-26277: Check if Orderby is not a valid SearchSort.
    if(defined $Servlet->params('OrderBy') && !ExistsSearchSortBySearchDefinitionID($SearchDefinition->id, $Servlet->params('OrderBy'))) {
        # Set by default "OrderBy Alias".
        $Servlet->params('OrderBy', 'Alias');
    }

    my $SearchForm = $Servlet->form->form($Folder, 'SearchProducts');
    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;
    $SearchForm->{'SearchString'} = $SearchForm->{'WildSearchString'} if defined $SearchForm->{'WildSearchString'};

    my $LocalizedAttributeName = $ProductClass->attribute('Name');
    my $Shop = $Folder->getSite();

    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Folder,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'   => $Shop->id,
            'SearchLanguageID' => $Servlet->languageID,
            'NameAttributeID' => $LocalizedAttributeName->id,
            'IsAppointment' => 0,
            'CurrencyID' => $Shop->get('CurrencyID')
        },
        'UriParameters'  => {
            'Search' => $SearchName,
        },
    );
    delete $SearchForm->{'StockWarn'} if exists $SearchForm->{'StockWarn'} and $SearchForm->{'StockWarn'} ==  0;

    foreach my $Parameter (keys %$SearchForm) {
        if ($Parameter =~ /String/) {
            # EPG-9002: limit the search string
            Readonly::Scalar my $MaxSearchStringLength => 80;
            my $SearchString = substr $SearchForm->{$Parameter}, 0, $MaxSearchStringLength;
            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards( $SearchString );
        }
        elsif($Parameter eq 'Alias'){
            ## UNITY-4143
            #replace '*' with '%' and add % to the end for (Alias/Productnumber)
            $Options{'Parameters'}{$Parameter} = ReplaceWildcardsWithoutPosOne($SearchForm->{$Parameter});
        }
        elsif ($Parameter =~ /^Is(.*)$/) {
            # status = 0/1/2 egal/yes/no
            if ($SearchForm->{$Parameter} == 1) {
                $Options{'Parameters'}{$Parameter} = 1;
            } elsif ($SearchForm->{$Parameter} == 2) {
                $Options{'Parameters'}{$Parameter} = 0;
            }
        } else {
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
        $Options{'UriParameters'}{$Parameter} = $SearchForm->{$Parameter};
    }
    if (defined $Servlet->params('OrderBy') && $Servlet->params('OrderBy') eq 'ListPrice') {
        $Options{'Parameters'}{'TaxModel'} = $Shop->get('TaxModel');
    }
    if (defined $SearchForm->{'Alias'}) {
        $Options{'Parameters'}{'Parent.ID'} = $Folder->id;
    }

    delete $SearchForm->{'Search'};
    LogDebug('Options', \%Options);

    return \%Options;
}

sub Save {
    my $self = shift;
    my ($Servlet) = @_;

    my $ProductFolder = $Servlet->object;
    my $hForm = $Servlet->form->form($ProductFolder, 'NewProduct');
    my %Values = map { s/^New//; $_ => $hForm->{'New'.$_} } grep { /^New/ && defined $hForm->{$_} } keys %$hForm;
    my $Site = $ProductFolder->getSite();

    if (not defined $Values{'Alias'}) {
        $Servlet->form->executeFormError({
            'Reason' => 'NoAlias',
            'Name'   => 'NewAlias',
            'Form'   => 'NewProduct',
        });
    }
    elsif ($ProductFolder->existsChild($Values{'Alias'})) {
        $Servlet->form->executeFormError({
            'Reason' => 'ObjectExists',
            'Name'   => 'NewAlias',
            'Form'   => 'NewProduct',
        });
    }
    elsif($Values{'Alias'} eq GetNextNo($Site, 'Product', $ProductFolder, 0) ){
        $Values{'Alias'} = GetNextNo($Site, 'Product');
    }

    my $Shop = $ProductFolder->getSite();
    $Values{'Shop'} = $Shop;
    $Values{'Parent'} = $ProductFolder;
    my $Name = $Values{'Name'};
    $Values{'URI'} = GetShortUrlByClass($ProductFolder, $Shop->get('ProductClass'), $Name) if defined $Name and $Name ne '';
    my $TaxModel = $Shop->get('InputTaxModel');
    my $CurrencyID = $Shop->get('CurrencyID');
    my $ListPrice = $Values{'ListPrice'};
    delete $Values{'ListPrice'};
    my $ManufacturerPrice = $Values{'ManufacturerPrice'};
    delete $Values{'ManufacturerPrice'};
    my $EcoParticipation = $Values{'EcoParticipation'};
    delete $Values{'EcoParticipation'};
    my $Deposit = $Values{'Deposit'};
    delete $Values{'Deposit'};
    my $Class = $Shop->get('ProductClass');

    # EPG-19649: Remember order unit for product creation
    my $LastOrderUnitForProductCreation = $Shop->get('LastOrderUnitForProductCreation');
    $Values{'OrderUnit'} = $LastOrderUnitForProductCreation if defined $LastOrderUnitForProductCreation;

    my $Product = undef;
    RunBatchProcess('UpdateProducts', sub {
        eval {
            $Product = $Class->insertObject(\%Values);
            $Product->setListPrice( $CurrencyID, $TaxModel, $ListPrice ) if defined $ListPrice;
            $Product->setManufacturerPrice( $CurrencyID, $TaxModel, $ManufacturerPrice ) if defined $ManufacturerPrice;
            $Product->setEcoParticipation( $CurrencyID, $TaxModel, $EcoParticipation ) if defined $EcoParticipation;
            $Product->setDeposit( $CurrencyID, $TaxModel, $Deposit ) if defined $Deposit;
        };
        if (ExistsError()) {
            my $Error = GetError();
            $Servlet->form->executeFormError({
                'Reason'=> 'FeatureMaxValueReached',
                'Form'  => 'NewProduct',
                'Name'  => 'NewAlias',
            }) if $Error->code eq 'FeatureLimitExceeded';
            Error($Error);
        }
    });

    $Servlet->vars('Search', undef);
    $Servlet->vars('JumpToProductID', $Product->id);

    return;
}

sub ExportProducts{
    my $self = shift;
    my ($Servlet) = @_;

    my $ProductFolder = $Servlet->object;
    my $hValues = $Servlet->form->form($ProductFolder, 'ListedObjects');
    my $Shop = $ProductFolder->getSite();
    my @ListObjects = map { $Shop->loadObject( $_->{'ListObjectID'}, 'Product' ) } @{$hValues->{'ListObjectIDs'}};
    return return unless scalar @ListObjects;

    $Servlet->vars('SelectedObjects', [map {$_->tleHash} @ListObjects]);
    $Servlet->vars('CSVExportImportType', 'Product');
    $Servlet->vars('EXPORT_SELECTED_OBJECTS', 1);
    $Servlet->vars('ViewActionAbort', $Servlet->params('ViewAction'));
    $Servlet->vars('ObjectIDAbort', $ProductFolder->id);
    $Servlet->vars('ViewAction', 'MBO-ViewCSVExportImportProduct');
    $Servlet->vars('ObjectID', $ProductFolder->getSite->id);
    return;
}

sub NewObject {
    my $self = shift;
    my ($Servlet, @Parameter) = @_;
    my $NewObject;

    my $ProductFolder = $Servlet->object;
    my $Shop = $ProductFolder->getSite();

    my $Name = $Servlet->form->value('Name');
    if (defined $Name && $Name =~ /\S/) {
        # check if we have a product name only with special chars
        $Servlet->form->executeFormError({
            'Form'   => 'Save',
            'Reason' => 'NAME_NOT_VALID',
            'Name'   => 'Name',
            'Value'  => $Name,
        }) unless $Name =~ /\w/;
    }

    # set default class if not defined ProductClass
    if (not $Servlet->params('ClassID')){
        my $Class = $Shop->get('ProductClass');
        if (defined $Class) {
            $Servlet->vars('ClassID', $Class->id);
        }
    }

    # set the tax class form value because we have a normal and a digital tax class
    _SetTaxClassFormValue($Servlet,$Shop );


    if (defined $Servlet->form->value('StockLevelAlert') && $Servlet->form->value('StockLevelAlert') ne '') {
        $Servlet->form->addFormError({
            'Reason'  => 'WrongValue',
            'Form'    => 'NewProduct',
            'Name'    => 'StockLevelAlert',
        }) if $Servlet->form->value('StockLevelAlert') le '0';
    }

    RunBatchProcess('CreateProducts', sub {
        eval {
            $NewObject = $self->SUPER::NewObject($Servlet,@Parameter);
        };
        if (ExistsError()) {
            my $Error = GetError();
            $Servlet->form->executeFormError({
                'Reason'=> $Error->code,
                'Form'  => 'NewProduct',
                'Name'  => 'URI',
            }) if $Error->code eq 'INVALID_CHARACTER' or $Error->code eq 'SHORTURL_ALREADY_EXISTS';
            Error($Error);
        }
    });

    return $NewObject;
}

sub _SetTaxClassFormValue {
    my ($Servlet,$Shop) = @_;

    my $FormTaxClass = $Servlet->form->value('FormTaxClass');
    my $hValues      = $Servlet->form->values();

    # we have a TaxMethod
    if ($Shop->get('TaxMethod')) {
        #set correct tax class on product
        $hValues->{'TaxClass'}          = $FormTaxClass;
        $hValues->{'TaxClassDigital'}   = undef;
    }
    else {

        my $ClassTaxClass = LoadClassByAlias( 'TaxClass');
        my $IsNormalTaxClass = $ClassTaxClass->existsObject($FormTaxClass);

        if (not $IsNormalTaxClass) {
            # use the default normal tax class, if we use the digital tax class for the product
            my $ShopDefaultTaxClass = $Shop->get('TaxMatrix')->get('DefaultTaxClass');
            $hValues->{'TaxClass'} = $ShopDefaultTaxClass->id;
            $hValues->{'TaxClassDigital'} = $FormTaxClass;
            }
        else {
            $hValues->{'TaxClassDigital'} = undef;
            $hValues->{'TaxClass'} = $FormTaxClass;
        }
    }

    $Servlet->form->setValues($hValues);
}

1;
