#========================================================================================
# package      DE_EPAGES::RemoteSearch::API::Batch
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------
# description  High level Batch API used by scripts from the RemoteSearch cartridge
#========================================================================================
package DE_EPAGES::RemoteSearch::API::Batch;
use base Exporter;

use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::RemoteSearch::API::DTO::UpdateItem;
use DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessFlagIndexHandler;
use DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessHandler;
use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw (
    MarkShopDirty
    RunIfRemoteSearchInitialized
);
use DE_EPAGES::Core::API::IniConfig;
use DE_EPAGES::Trigger::API::BatchProcessor qw (
    HandleItem
    HandleItemInBatchModeOnly
    IsCalledInBatchMode
    RegisterHandler
    RunBatchProcess
);

use strict;

use constant BATCH_UPDATE_HANDLER     => 'DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessHandler';
use constant BATCH_FLAG_INDEX_HANDLER => 'DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessFlagIndexHandler';

our @EXPORT_OK = qw (
  BATCH_CONTEXTS
  IsScheduleMode
  GetBatchUpdateHandlerPackage
  BatchUpdateProduct
  BatchDeleteProduct
  BatchUpdateProductType
  BatchUpdateCategoryProductMap
  RegisterDeleteHandler
  RegisterUpdateHandler
  RegisterProductTypeUpdateHandler
);


#========================================================================================
# global BATCH_CONTEXTS | batch contexts relevant for communicating product updates | array.string
#========================================================================================
sub BATCH_CONTEXTS { ## no critic qw(RequireCommentForExportedFunctions)
return qw (
    UpdateProducts ImportXML CSVCatalogImport
    CSVProductImport CSVProductTypeImport
    CloneProducts CreateObjects UpdateObjects DeleteProducts
    DeleteAllProducts DeleteObjects 
    );
}

#========================================================================================
# function     BatchUpdateProductType
# state        public
#----------------------------------------------------------------------------------------
# syntax       BatchUpdateProductType( $ProductType )
#----------------------------------------------------------------------------------------
# description  Batch update call for ProductType attributes.
#----------------------------------------------------------------------------------------
# input        $ProductType | ProductType | object
#========================================================================================
sub BatchUpdateProductType {
    my ($ProductType) = @_;
    return unless $ProductType->instanceOf('ProductType');
    if (IsScheduleMode()) {
        MarkShopDirty($ProductType->getSite);
    } else {
        RunBatchProcess(
            _FindBatchUpdateContext(),
            sub {
                my $Products = $ProductType->objectIterator( 'ByClassID', [ $ProductType->id ] );
                while ( my $Product = $Products->next ) {
                    BatchUpdateProduct($Product);
                }
            }
        );
    }
    return;
}

#========================================================================================
# function     GetBatchUpdateHandlerPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $HandlerPackage = GetBatchUpdateHandlerPackage();
#----------------------------------------------------------------------------------------
# description  Returns BatchUpdateHandler package name, usable in
#               <function DE_EPAGES::Trigger::API::BatchProcessor::HandleItem>.
#               The selected BatchProcessor depends on value returned by <function IsScheduleMode>
#----------------------------------------------------------------------------------------
# return       $BatchUpdateHandlerPackage | BatchUpdateHandler package name | string
#========================================================================================
sub GetBatchUpdateHandlerPackage {
    return IsScheduleMode()
      ? BATCH_FLAG_INDEX_HANDLER
      : BATCH_UPDATE_HANDLER;
}

#========================================================================================
# function     IsScheduleMode
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $RunInScheduleMode = IsScheduleMode();
#----------------------------------------------------------------------------------------
# description  Check if RemoteSearch configured to react on hook events.
#----------------------------------------------------------------------------------------
# return       $RunInScheduleMode | Value of System.RemoteSearchScheduleUpdates | boolean
#========================================================================================
sub IsScheduleMode() {
    my $System            = LoadRootObject();
    my $RunInScheduleMode = 0;

    # on installation of the cartridge this method is called but the attribute is not registered yet
    if ( $System->class->existsAttribute('RemoteSearchScheduleUpdates') ) {
        $RunInScheduleMode = $System->get('RemoteSearchScheduleUpdates');
    }

    return $RunInScheduleMode;
}

#========================================================================================
# function     BatchUpdateProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       BatchUpdateProduct($Product);
#----------------------------------------------------------------------------------------
# description  Batches a product update call
#----------------------------------------------------------------------------------------
# input        $Product | product | DE_EPAGES::Product::API::Object::Product
#========================================================================================
sub BatchUpdateProduct {
    my ($Product) = @_;
    TestObject( 'Product', $Product, 'DE_EPAGES::Product::API::Object::Product' );
    RunIfRemoteSearchInitialized(
        $Product->getSite,
        sub {
            HandleItem(
                _FindBatchUpdateContext(),
                GetBatchUpdateHandlerPackage(),
                DE_EPAGES::RemoteSearch::API::DTO::UpdateItem->new( product => _GetParentOrSelf($Product) )
            );
        }
    );
    return;
}

#========================================================================================
# function     BatchUpdateCategoryProductMap
# state        public
#----------------------------------------------------------------------------------------
# syntax       BatchUpdateCategoryProductMap($Product);
#----------------------------------------------------------------------------------------
# description  Batches a category product map update call
#----------------------------------------------------------------------------------------
# input        $Product | product | DE_EPAGES::Product::API::Object::Product
#========================================================================================
sub BatchUpdateCategoryProductMap {
    my ($Product) = @_;
    my $Shop = $Product->getSite();
    RunIfRemoteSearchInitialized(
        $Shop,
        sub {
            HandleItemInBatchModeOnly(
                'CSVCatalogImport',
                'DE_EPAGES::RemoteSearch::API::Object::ProductCategoryMappingBatchProcessHandler',
                $Shop
            );
        }
    );
    return;
}


#========================================================================================
# function     BatchDeleteProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       BatchDeleteProduct($Product);
#----------------------------------------------------------------------------------------
# description  Batches a product delete call
#----------------------------------------------------------------------------------------
# input        $Product | product | DE_EPAGES::Product::API::Object::Product
#========================================================================================
sub BatchDeleteProduct {
    my ($Product) = @_;
    TestObject( 'Product', $Product, 'DE_EPAGES::Product::API::Object::Product' );
    my $UpdateProduct = _GetParentOrSelf($Product);
    my $UpdateItem    = DE_EPAGES::RemoteSearch::API::DTO::UpdateItem->new(
        product    => $UpdateProduct,
        deleteItem => ( $UpdateProduct->id == $Product->id )
    );
    RunIfRemoteSearchInitialized(
        $Product->getSite,
        sub {
            HandleItem(
                _FindBatchUpdateContext(), GetBatchUpdateHandlerPackage(),
                $UpdateItem
            );
        }
    );
    return;
}

#========================================================================================
# function     RegisterUpdateHandler
# state        public
#----------------------------------------------------------------------------------------
# syntax       RegisterUpdateHandler($Context);
#----------------------------------------------------------------------------------------
# input        $Context | batch context key | string
# description  Register ProductBatchProcessHandler which used by every HandleItem call.<br>
#               see <package DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessHandler>
#========================================================================================
sub RegisterUpdateHandler {
    my ($Context) = @_;
    RegisterHandler( $Context, GetBatchUpdateHandlerPackage()->new( ) );
    return;
}

#---------------------------------------------------------------------------
#  private helpers
#---------------------------------------------------------------------------
sub _FindBatchUpdateContext {
    foreach my $Item (BATCH_CONTEXTS()) {
        return $Item if ( IsCalledInBatchMode($Item) );
    }
    return [BATCH_CONTEXTS()]->[0];
}

sub _GetParentOrSelf {
    my ($Product) = @_;
    my $SuperProduct = $Product->get('SuperProduct');
    return $SuperProduct if defined($SuperProduct);
    return $Product;
}

1;
