#========================================================================================
# package      DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessHandler
# base         DE_EPAGES::Trigger::API::Object::BatchProcessHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Handles batch-updates of products to pass them to the remote search.
#               Implements Interface <package DE_EPAGES::Trigger::API::Object::BatchProcessHandler><br>
#               Uses <package DE_EPAGES::RemoteSearch::API::Object::SearchIndex>
#========================================================================================
package DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessHandler;
use base DE_EPAGES::Trigger::API::Object::BatchProcessHandler;

use strict;
use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestInteger);
use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw ( RunIfRemoteSearchInitialized );
use DE_EPAGES::Trigger::API::BatchProcessor qw ( IsCalledInBatchMode );
use DE_EPAGES::Core::API::EpagesConfig qw ( EpagesConfig );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Handler = DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessHandler->new( {BatchSize => 50} );
# syntax       my $Handler = DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessHandler->new();
#----------------------------------------------------------------------------------------
# description  Pass optional parameter 'BatchSize' to define the
#               number of items queued before <function processItemList>() is called
#               automatically. Call processItemList() manualy before discarding the Handler
#               to ensure that all items are processed. <br>
#               <br>
#               see <package DE_EPAGES::Trigger::API::Object::BatchProcessHandler> for inherited methods
#----------------------------------------------------------------------------------------
# input        $hOptions |<ul>
#                            <li>BatchSize - this limit trigger automated processing while adding Products(optional) - integer &gt; 1 </li>
#                            <li>SearchIndex - Custom SearchIndex object for testing/debugging - DE_EPAGES::RemoteSearch::API::Object::SearchIndex</li>
#                          </ul> | ref.hash
# return       $Object | new batch process handler | DE_EPAGES::RemoteSearch::API::Object::ProductBatchProcessHandler
#========================================================================================
sub new {
    my $class = shift;
    my ($hOptions) = @_;
    my $self = $class->SUPER::new($hOptions);
    $self->{'Options'} = $self->_initProductBatchProcessHandler($hOptions);
    return $self;
}

#----------------------------------------------------------------------------------------
sub _initProductBatchProcessHandler {
    my $self = shift;
    my ($hOptions) = @_;
    

    if ( !defined $hOptions->{'ReindexThreshold'} ) { 
        my $DefaultValue = 3000;
        my $ReindexThreshold = EpagesConfig()->get( 'DE_EPAGES::RemoteSearch', 'ReindexThreshold', $DefaultValue );
        $hOptions->{'ReindexThreshold'} = $ReindexThreshold;
    }
    TestInteger( 'ReindexThreshold', $hOptions->{'ReindexThreshold'}, 1 );

    if ( !defined $hOptions->{'ProductBatchSize'} ) { 
        my $DefaultValue = 1000;
        my $ProductBatchSize = EpagesConfig()->get( 'DE_EPAGES::RemoteSearch', 'ProductBatchSize', $DefaultValue );
        $hOptions->{'ProductBatchSize'} = $ProductBatchSize;
    }
    TestInteger( 'ProductBatchSize', $hOptions->{'ProductBatchSize'}, 1 );

    return $hOptions;
}
#========================================================================================
# function     reindexThreshold
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ReindexThreshold = $Handler->ReindexThreshold();
#----------------------------------------------------------------------------------------
# description  Returns the treshold when the reindex should be used.
#               Instead of single product update.
#----------------------------------------------------------------------------------------
# return       $ReindexThreshold | treshold of reindex | integer
#========================================================================================
sub reindexThreshold {
    my $self = shift;

    return $self->{'Options'}->{'ReindexThreshold'};
}
#========================================================================================
# function     productBatchSize
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ProductBatchSize = $Handler->productBatchSize();
#----------------------------------------------------------------------------------------
# description  The amount of items in one chunk
#----------------------------------------------------------------------------------------
# return       $ProductBatchSize | number of items in one chunk | integer
#========================================================================================
sub productBatchSize {
    my $self = shift;
    return $self->{'Options'}->{'ProductBatchSize'};
}
#========================================================================================
# function     processItemList
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->processItemList();
#----------------------------------------------------------------------------------------
# description  Stored products are passed to SearchIndex object and flushed.
#               On first call of this method the search initialized with
#               parameters from first Object in item list.<br>
#               <br>
#               see <package DE_EPAGES::RemoteSearch::API::Object::SearchIndex>.
#========================================================================================
sub processItemList {
    my $self = shift;

    return unless scalar @{ $self->items() };

    RunIfRemoteSearchInitialized(
        $self->items()->[0]->product()->getSite(),
        sub {
            my ($Search) = @_;
            return unless $self->_isProductBatchAction();
            if ( IsCalledInBatchMode('DeleteAllProducts') ) {
                $Search->drop();
                $Search->initialize();
            } else {
                $self->_updateProducts($Search);
            }
        }
    );

    $self->cleanupItems();

    return;
}
#----------------------------------------------------------------------------------------
sub _isProductBatchAction {
    my $self = shift;
    
    # Check if we are in one of this RunBatchProcesses.
    my @BatchTaskNames = qw ( CreateCategories UpdateCategories DeleteCategories CSVCatalogImport CSVContentImport);
    # CSVCatalogImport == CSV-Import of categories
    # CSVContentImport == CSV-Import of product-category mapping
    foreach my $BatchTaskName (@BatchTaskNames) {
        if ( IsCalledInBatchMode($BatchTaskName) ) {
            #The surrounding RunBatchProcess will take care about update of the products.
            return 0;
        }
    }
    return 1;
}
#----------------------------------------------------------------------------------------
sub _updateProducts {
    my $self = shift;
    my ($Search) = @_;

    if ( $self->_isReindexThresholdReached() ) {
        $Search->initialize();
    } else {
        $self->_updateProductsInBatches( $self->items(), $Search );
    }

    return;
}
#----------------------------------------------------------------------------------------
sub _updateProductsInBatches {
    my $self = shift;
    my ( $aProducts, $Search ) = @_;

    my $BatchSize = $self->productBatchSize();

    # Split product list in chunks of one batch size each.
    while ( my @ProductBatch = splice( @{$aProducts}, 0, $BatchSize ) ) {
        $self->_updateProductBatch( \@ProductBatch, $Search );
    }

    return;
}

#----------------------------------------------------------------------------------------
sub _updateProductBatch {
    my $self = shift;
    my ( $aProducts, $Search ) = @_;

    my $aDeletedProducts = [ map { $_->product() } grep { $_->deleteItem() } @{$aProducts} ];
    if ( scalar @{$aDeletedProducts} ) {
        $Search->deleteItems($aDeletedProducts);
    }
    my $aUpdatedProducts = [ map { $_->product() } grep { not $_->deleteItem() } @{$aProducts} ];
    if ( scalar @{$aUpdatedProducts} ) {
        $Search->updateItems($aUpdatedProducts);
    }

    return;
}

#========================================================================================
# function     addItem
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->addItem( $NewItem );
#----------------------------------------------------------------------------------------
# description  Wrapper for <function DE_EPAGES::Trigger::API::Object::BatchProcessHandler::addItem>
#               Item (Product) is only added if $Item->id is not found in $Handler->items
#----------------------------------------------------------------------------------------
# input        $NewProduct | Request for processing a product. | DE_EPAGES::RemoteSearch::API::DTO::UpdateItem
#========================================================================================
sub addItem {
    my $self = shift;
    my ($NewProduct) = @_;
    TestObject( 'UpdateItem', $NewProduct, 'DE_EPAGES::RemoteSearch::API::DTO::UpdateItem' );

    my $aProductsInStack = $self->items();

    # send piled items if object belongs to a different shop
    if (   ( scalar @{$aProductsInStack} ) && ( $aProductsInStack->[0]->getSiteID() != $NewProduct->getSiteID() ) ){
        $self->processItemList();
    }

    foreach my $ProductInStack ( @{$aProductsInStack} ) {

        # find existing item.
        if ( $ProductInStack->id() == $NewProduct->id() ) {

            # set to delete if we find a delete
            $ProductInStack->deleteItem( $ProductInStack->deleteItem() | $NewProduct->deleteItem() );

            # do not add to stack.
            return;
        }
    }

    push(@{ $self->items() }, $NewProduct);
    $self->_checkIfReindexThresholdIsReached();

    return;
}

#----------------------------------------------------------------------------------------
sub _isReindexThresholdReached {
    my $self = shift;
    return $self->{'ReindexThresholdIsReached'};
}

#----------------------------------------------------------------------------------------
sub _checkIfReindexThresholdIsReached {
    my $self = shift;

    if ( ( $self->reindexThreshold() > 0 ) && $self->reindexThreshold() <= $self->countItems() ){
        $self->{'ReindexThresholdIsReached'} = 1;
    }

    return;
}

1;
