#========================================================================================
# package      DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient
# base         DE_EPAGES::EPagesJ::API::Object::RestClient
# state        public
# description  Webservice client for remote search index management operations.
#               Operations are usually appended by a replication token, used as a latch
#               for the remote server, to wait until the synchronization token appears
#               in the database. That way we can make sure all changes have been written to
#               database, even if the remote server is using a replicated slave database.
#----------------------------------------------------------------------------------------
# example      # Regular usage:
#               my $Client = $SearchIndex->remoteIndex();
#               $Client->deleteItems($aItems, IncrementAndGetReplicationToken());
#========================================================================================
package DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient;
use base DE_EPAGES::EPagesJ::API::Object::RestClient;

use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::EPagesJ::API::UriTemplateFactory qw ( CreateUriTemplates );
use DE_EPAGES::EPagesJ::API::Object::UriTemplate;
use DE_EPAGES::RemoteSearch::API::DTO::Task;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Shop::API::Resource::Formatter;
use Readonly;

use strict;

Readonly my $INDEX_SHOPS_PATH     => 'product/{dsName}';
Readonly my $INDEX_SHOP_PATH      => 'product/{dsName}/{shopId}';
Readonly my $INDEX_ITEMS_PATH     => 'product/{dsName}/{shopId}/update';
Readonly my $DELETE_ITEMS_PATH    => 'product/{dsName}/{shopId}/delete';
Readonly my $CREATE_CATEGORY_PATH => '{dsName}/{shopId}/category';

sub new {
    my $class = shift;
    my %args  = @_;

    foreach my $Mandatory (qw ( IndexShopUrl IndexShopsUrl IndexItemsUrl DeleteItemsUrl CreateCategoryUrl )) {
        TestObject($Mandatory, $args{$Mandatory});
    }

    return __PACKAGE__->SUPER::new(%args);
}

sub create {
    my $class = shift;
    my ($Object, $hArgs) = @_;

    my $hTemplates = CreateUriTemplates($Object,
        [$INDEX_SHOP_PATH, $INDEX_SHOPS_PATH, $INDEX_ITEMS_PATH, $DELETE_ITEMS_PATH, $CREATE_CATEGORY_PATH ],
        $hArgs->{BaseURL});

    $hArgs->{IndexShopUrl}   = $hTemplates->{$INDEX_SHOP_PATH}->uri;
    $hArgs->{IndexShopsUrl}  = $hTemplates->{$INDEX_SHOPS_PATH}->uri;
    $hArgs->{IndexItemsUrl}  = $hTemplates->{$INDEX_ITEMS_PATH}->uri;
    $hArgs->{DeleteItemsUrl} = $hTemplates->{$DELETE_ITEMS_PATH}->uri;
    $hArgs->{CreateCategoryUrl} = $hTemplates->{$CREATE_CATEGORY_PATH}->uri;

    return __PACKAGE__->SUPER::create($Object, $hArgs);
}

#========================================================================================
# function     deleteItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Client->deleteItems( $aItems, $ReplicationTokenID );
#----------------------------------------------------------------------------------------
# description  Removes items from remote search index via HTTP POST<br>
#               <br>
#               Throws error 'REQUEST FAILED' if remote service is not reachable or did not
#               anwser in the given time limit.
#               See <package DE_EPAGES::RemoteSearch::API::Object::HTTPClient>
#----------------------------------------------------------------------------------------
# input        $aItems | list of item IDs | ref.array.int
# input        $ReplicationTokenID | database token used for master-slave synchronization | int
# return       $Task | remote task | DE_EPAGES::RemoteSearch::API::DTO::Task
#========================================================================================
sub deleteItems {
    my $self = shift;
    my ( $aItems, $ReplicationTokenID ) = @_;
    return $self->_createTask(sub {
       return $self->client->post($self->deleteItemsUrl, undef, $self->_indexItemsOperationBody( $aItems, $ReplicationTokenID ) );
    });
}

#========================================================================================
# function     indexItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Client->indexItems( $aItems, $ReplicationTokenID );
#----------------------------------------------------------------------------------------
# description  Updates or inserts items to the remote search index via HTTP POST<br>
#               <br>
#               Throws error 'REQUEST FAILED' if remote service is not reachable or did not
#               anwser in given the given timelimit.
#               See <package DE_EPAGES::RemoteSearch::API::Object::HTTPClient>
#----------------------------------------------------------------------------------------
# input        $aItems | list of item IDs | ref.array.int
# input        $ReplicationTokenID | database token used for master-slave synchronization | int
# return       $Task | remote task | DE_EPAGES::RemoteSearch::API::DTO::Task
#========================================================================================
sub indexItems {
    my $self = shift;
    my ( $aItems, $ReplicationTokenID ) = @_;

    return $self->_createTask(sub {
        return $self->client->post( $self->indexItemsUrl, undef, $self->_indexItemsOperationBody( $aItems, $ReplicationTokenID ) );
    });
}

#========================================================================================
# function     indexShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Client->indexShop( $ReplicationTokenID );
#----------------------------------------------------------------------------------------
# description  Creates a remote search index via HTTP POST. If it already exists, the previous index
#               will be replaced with the new one.<br>
#               <br>
#               Throws error 'REQUEST FAILED' if remote service is not reachable or did not
#               anwser in given the given timelimit.
#               See <package DE_EPAGES::RemoteSearch::API::Object::HTTPClient>
#----------------------------------------------------------------------------------------
# input        $ReplicationTokenID | database token used for master-slave synchronization | int
# return       $Task | remote task | DE_EPAGES::RemoteSearch::API::DTO::Task
#========================================================================================
sub indexShop {
    my $self = shift;
    my ($ReplicationTokenID, $Shop) = @_;

    return $self->_createTask(sub {
        return $self->client->post( $self->indexShopUrl, undef, $self->_indexOperationBody($ReplicationTokenID, $Shop) );
    });
}

#========================================================================================
# function     initializeNoSolr
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Client->initializeNoSolr( $Shop );
#----------------------------------------------------------------------------------------
# description  updates a document in the shop collection only. It does not touch the index.<br>
#               <br>
#               Throws error 'REQUEST FAILED' if remote service is not reachable or did not
#               anwser in given the given timelimit.
#               See <package DE_EPAGES::RemoteSearch::API::Object::HTTPClient>
#----------------------------------------------------------------------------------------
# input        $Shop | shop to update | DE_EPAGES::Shop::API::Object::Shop
# return       $Task | remote task | DE_EPAGES::RemoteSearch::API::DTO::Task
#========================================================================================
sub initializeNosolr {
    my $self = shift;
    my ( $Shop ) = @_;
    $self->client->post( $self->indexShopUrl, undef, $self->_indexOperationBody(undef, $Shop, 1) );
}

#========================================================================================
# function     indexShops
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Client->indexShops( $aItems, $ReplicationTokenID );
#----------------------------------------------------------------------------------------
# description  Updates or inserts multiple shops into remote serach index via HTTP POST.
#               Throws error 'REQUEST FAILED' if remote service is not reachable or did not
#               anwser in the given time limit.
#               See <package DE_EPAGES::RemoteSearch::API::Object::HTTPClient>
#----------------------------------------------------------------------------------------
# input        $aItems | list of item IDs | ref.array.int
# input        $ReplicationTokenID | database token used for master-slave synchronization | int
# return       $Task | remote task | DE_EPAGES::RemoteSearch::API::DTO::Task
#========================================================================================
sub indexShops {
    my $self = shift;
    my ( $aItems, $ReplicationTokenID ) = @_;
    return $self->_createTask(sub {
        return $self->client->post($self->indexShopsUrl, undef, $self->_indexItemsOperationBody( $aItems, $ReplicationTokenID ) );
    });
}

#========================================================================================
# function     dropIndex
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Client->dropIndex( $ReplicationTokenID );
#----------------------------------------------------------------------------------------
# description  Removes a remote search index using HTTP DELETE.<br>
#               <br>
#               Throws error 'REQUEST_FAILED' if remote service is not reachable or did not
#               anwser in given the given timelimit.
#               See <package DE_EPAGES::RemoteSearch::API::Object::HTTPClient>
#----------------------------------------------------------------------------------------
# input        $ReplicationTokenID | database token used for master-slave synchronization | int
# return       $Task | remote task | DE_EPAGES::RemoteSearch::API::DTO::Task
#========================================================================================
sub dropIndex {
    my $self = shift;
    my ($ReplicationTokenID) = @_;

    return $self->_createTask(sub {
        return $self->client->delete( $self->indexShopUrl, undef );
    });
}

#========================================================================================
# function     createCategory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $EndpointRequest->createCategory($Category);
#----------------------------------------------------------------------------------------
# description  A resource that inserts a newly created category
#----------------------------------------------------------------------------------------
# input        $Category | Category Object | object
# return       $Response | epagesj endpoint url | object
#========================================================================================
sub createCategory {

    my $self = shift;
    my ( $Category ) = @_;

    my $hEncodedRequest = $self->_createCategoryBody($Category);

    my $hHeaders = undef;
    $self->client->put( $self->createCategoryUrl(), $hHeaders, $hEncodedRequest );

    return;
}

#========================================================================================
# function     _createTask
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $Task = $Client->_createTask( $cClientCall );
#----------------------------------------------------------------------------------------
# description  Executes remote call and transforms response into a Task object.
#----------------------------------------------------------------------------------------
# input        $cClientCall | code to eecute. Should return an HTTP::Response | ref.sub
# return       $Task | remote task | DE_EPAGES::RemoteSearch::API::DTO::Task
#========================================================================================
sub _createTask {
    my $self = shift;
    my ($cClientCall) = @_;

    my $Response = $cClientCall->();
    return $self->_taskFromResponse($Response);
}

#========================================================================================
# function     getTask
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Task = $Client->getTask( $TaskURL );
#----------------------------------------------------------------------------------------
# description  Retrieves a remote task via HTTP GET<br>
#               <br>
#               Throws error 'REQUEST_FAILED' if remote service is not reachable or did not
#               anwser in given the given timelimit.
#               See <package DE_EPAGES::RemoteSearch::API::Object::HTTPClient>
#----------------------------------------------------------------------------------------
# input        $TaskURL | task url | string
# return       $Task | task object | DE_EPAGES::RemoteSearch::API::DTO::Task
#========================================================================================
sub getTask {
    my $self = shift;
    my ($TaskURL) = @_;

    my $Response = $self->client->get( URI::URL->new($TaskURL) );
    return $self->_taskFromResponse($Response);
}

#========================================================================================
# function     _indexOperationBody
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Client->_indexOperationBody(  $ReplicationTokenID );
#----------------------------------------------------------------------------------------
# description  Build message body for index operations
#----------------------------------------------------------------------------------------
# input        $ReplicationTokenID | database token used for master-slave synchronization | int
# return       $json_string | json encoded string | string
#========================================================================================
sub _indexOperationBody {
    my $self = shift;
    my ($ReplicationTokenID, $Shop, $Nosolr ) = @_;

    my $Formatter = DE_EPAGES::Shop::API::Resource::Formatter->new($Shop, $Shop->get('LocaleID'));

    my $System = $self->_LoadRootObject();
    return $self->encodeRequest( {
        'replicationToken' => $ReplicationTokenID,
        'nosolr' => $Nosolr,
        'shop' => {
            'guid' => $Shop->guid(),
            'publicId' => $Shop->alias(),
            'objectId' => $Shop->id(),
            'storename' => $System->get('StoreName'),
            'domainName' => $Shop->get('DomainName'),
            'systemDomainName' => $System->get('DomainName'),
            'domainSslSecured' => $Formatter->bool($Shop->get('HasSSLCertificate')),
            'systemDomainSslSecured' => $Formatter->bool($System->get('HasSSLCertificate')),
        },
    } );
}

#========================================================================================
# function     _taskFromResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $Task = $Client->_taskFromResponse( $Response );
#----------------------------------------------------------------------------------------
# description  Creates a task object based on the http response.
#----------------------------------------------------------------------------------------
# input        $Response | HTTP response object | HTTP::Response
# return       $Task | remote task | DE_EPAGES::RemoteSearch::API::DTO::Task
#========================================================================================
sub _taskFromResponse {
    my $self = shift;
    my ($Response) = @_;
    return DE_EPAGES::RemoteSearch::API::DTO::Task->new( $self->decodeResponse( $Response->decoded_content ) );
}

#========================================================================================
# function     _indexItemsOperationBody
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Client->_indexItemsOperationBody($aItems, $ReplicationTokenID );
#----------------------------------------------------------------------------------------
# description  Build message body for index item operations
#----------------------------------------------------------------------------------------
# input        $aItemes | list of item IDs | ref.array.int
# input        $ReplicationTokenID | database token used for master-slave synchronization | int
# return       $json_string | json encoded string | string
#========================================================================================
sub _indexItemsOperationBody {
    my $self = shift;
    my ( $aItems, $ReplicationTokenID ) = @_;

    return $self->encodeRequest( {
            'replicationToken' => $ReplicationTokenID,
            'indexItems'       => { 'items' => $aItems, }
        }
    );
}

##========================================================================================
# function     _createCategoryBody
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_createCategoryBody($Category);
#----------------------------------------------------------------------------------------
# description  Build message body for Category insertion
#----------------------------------------------------------------------------------------
# input        $Category | Category Object | object
# return       $json_string | json encoded string | string
#========================================================================================
sub _createCategoryBody {
    my $self = shift;
    my ($Category) = @_;

    my $ParentCategory = $Category->get('Parent');
    my $Shop = $Category->getSite();

    my @Languages = @{$Shop->get('Languages')};
    my %LocalizedName =
            map { $_->{'Code2'} => $Category->get('NameOrAlias', $_->{'LanguageID'}) } @Languages;
    my %LocalizedDisplayName =
            map { $_->{'Code2'} => $Category->get('DisplayTitle', $_->{'LanguageID'}) } @Languages;

    my $hCategoryBody = {
        'guid' => $Category->guid(),
        'objectId' => $Category->id(),
        'visible' => $Category->get('IsVisible') ? 'true' : 'false',
        'visibleInNavigation' => $Category->get('VisibleInNavigation') ? 'true' : 'false',
        'position' => $Category->get('Position'),
        'isSpecialOffer' => $Category->get('IsSpecialOffer') ? 'true' : 'false',
        'parentId' => $ParentCategory->id(),
        'alias' => $Category->alias(),
        'siteId' => $Shop->id(),
        'name' => \%LocalizedName,
        'displayName' => \%LocalizedDisplayName
    };

    return $self->encodeRequest($hCategoryBody);
}

### getters
sub indexShopsUrl {
    return shift->{IndexShopsUrl};
}

sub indexShopUrl {
    return shift->{IndexShopUrl}
}

sub indexItemsUrl {
    return shift->{IndexItemsUrl};
}

sub deleteItemsUrl {
    return shift->{DeleteItemsUrl};
}

sub createCategoryUrl {
    return shift->{CreateCategoryUrl};
}

sub _LoadRootObject {
    my $self = shift;
    return LoadRootObject();
}

1;
