#========================================================================================
# package      DE_EPAGES::RemoteSearch::API::RemoteSearchImpl
# base         Exporter
# state        public
#----------------------------------------------------------------------------------------
# description  High level API used by scripts from the RemoteSearch cartridge
#========================================================================================
package DE_EPAGES::RemoteSearch::API::RemoteSearchImpl;
use base Exporter;

use strict;

our @EXPORT_OK = qw (
  DropAllSearchIndexes
  GetOrCreateSearchIndex
  IsRemoteSearchActive
  MarkShopDirty
  RunIfRemoteSearchInitialized
  SearchProducts
  SearchProductsSqlFallback
  UpdateSearchStatistics
  GetDestinctSelections
  UseSearchInCategoryContext
  GetPageSizeForSearch
);

use DE_EPAGES::Catalog::API::Shop qw ( GetPageSizeForObject );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog LogDebug );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadRootObject Transaction );
use DE_EPAGES::RemoteSearch::API::IndexStatus qw ( INDEXSTATUS_DIRTY IndexStatusID );
use DE_EPAGES::Search::API::SearchDefinition qw ( GetSearchDefinition );
use DE_EPAGES::Search::API::SearchTerm qw ( RegisterTerm );
use DE_EPAGES::Search::API::Object::PagerSearch qw ( ReplaceWildcards );

use LWP::UserAgent;
use JSON::XS qw (encode_json);
#========================================================================================
# function     IsRemoteSearchActive
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $IsUseable = IsRemoteSearchActive( $Shop );
#----------------------------------------------------------------------------------------
# description  Check if remote search can be used for the shop
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
# return       $IsActive | Returns true if System and $Shop flagged have set RemoteSearchEnabled | Boolean
#========================================================================================
sub IsRemoteSearchActive($) {
    my ($Shop) = @_;
    return LoadRootObject->get('RemoteSearchEnabled') && $Shop->get('RemoteSearchEnabled');
}

#========================================================================================
# function     GetOrCreateSearchIndex
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $SearchIndex = GetOrCreateSearchIndex( $Shop );
#----------------------------------------------------------------------------------------
# description  Creates (if necessary) and returns SearchIndex object.
#               Does not set the remote call to index the shop. Use
#               <package DE_EPAGES::RemoteSearch::API::Object::IndexManager> for that.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
# return       $SearchIndex | search index | DE_EPAGES::RemoteSearch::API::Object::SearchIndex
#========================================================================================
sub GetOrCreateSearchIndex($) {
    my ($Shop)    = @_;
    my $ChildAlias = 'Product';
    my $ClassType = 'SearchIndexes';
    my $Folder    = $Shop->folder($ClassType);
    if ( not $Folder->existsChild($ChildAlias) ) {
        Transaction(sub {
            $Folder->insertChild( 'SearchIndex', { Alias => $ChildAlias, } );
            $Shop->set( { 'RemoteSearchEnabled' => 1 } );
        });
    }
    return $Folder->child($ChildAlias);
}

#========================================================================================
# function     DropAllSearchIndexes
# state        public
#----------------------------------------------------------------------------------------
# syntax       DropAllSearchIndexes( $Shop );
# syntax       DropAllSearchIndexes( $Shop, $DropRemote );
#----------------------------------------------------------------------------------------
# description  Resets search info on shop and tries to delete the index from the remote
#               search, if needed.
#----------------------------------------------------------------------------------------
# input        $Shop | Shop | object
# input        $DropRemote | (optional, default: false; set to true, if you want to remove the remote index | boolean
#========================================================================================
sub DropAllSearchIndexes($;$) {
    my ($Shop, $DropRemote) = @_;
    return unless ( $Shop->existsChild('SearchIndexes') );

    if ($DropRemote) {
        foreach my $Index (@{$Shop->child('SearchIndexes')->children}){
            eval {
                $Index->drop;
            };
            if (ExistsError()) {
                my $Error = GetError();
                Warning($Error);
            }
        }
    }
    $Shop->child('SearchIndexes')->delete;
    return;
}

#========================================================================================
# function     RunIfRemoteSearchInitialized
# state        public
#----------------------------------------------------------------------------------------
# syntax       RunIfRemoteSearchInitialized( $Shop, $cSub );
#----------------------------------------------------------------------------------------
# description  Runs code if remote search is active and RemoteSearchIndex object has been
#               created. Passes RemoteProductSearchIndex object into the sub.
#               Non-fatal Errors get turned into Warnings.
#----------------------------------------------------------------------------------------
# example      RunIfRemoteSearchActive($Shop, sub {
#                   my ($Search) = @_;
#                   $Search->updateItems([ $Product ]);
#               });
#----------------------------------------------------------------------------------------
# input        $Shop | shop | DE_EPAGES::Shop::API::Object::Shop
# input        $cSub | code ref | ref.code
#========================================================================================
sub RunIfRemoteSearchInitialized($$) {
    my ($Shop, $cSub) = @_;
    return unless IsRemoteSearchActive($Shop);
    my $Search = $Shop->get('RemoteProductSearchIndex');
    return unless (defined $Search);
    eval {
        $cSub->($Search);
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) if ($Error->code =~ /^Native/);
        Warning($Error);
    }
}

#========================================================================================
# function     MarkShopDirty
# state        public
#----------------------------------------------------------------------------------------
# syntax       MarkShopDirty( $Shop );
#----------------------------------------------------------------------------------------
# description  Mark (existing) search index of $Shop as dirty.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object DE_EPAGES::Shop::API::Object::Shop
#========================================================================================
sub MarkShopDirty($) {
    my ($Shop) = @_;
    GetLog->debug( 'mark Shop: ' . $Shop->alias . 'as dirty' );

    if (my $Index = $Shop->get('RemoteProductSearchIndex')) {
        $Index->set( { 'IndexStatus' => IndexStatusID(INDEXSTATUS_DIRTY) } );
    }
}


#========================================================================================
# function     UpdateSearchStatistics
# state        public
#----------------------------------------------------------------------------------------
# syntax       UpdateSearchStatistics( $Shop, $Query, $Results, $Fallback);
#----------------------------------------------------------------------------------------
# description  Registers the user's search query at our search statistics.<br><br>
#               Search services automatically provides a fallback if a query
#               yields to no results.
#               If fallback search kicks in, the results are registered against
#               fallback term.
#----------------------------------------------------------------------------------------
# input        $Shop | current shop | DE_EPAGES::Shop::API::Object::Shop
# input        $Query | user input | string
# input        $Results | number of results | int
# input        $Fallback | fallback query, if used (optional) | string
#========================================================================================
sub UpdateSearchStatistics($$$;$) {
    my ( $Shop, $Query, $Results, $Fallback ) = @_;
    return unless (isValid($Query));
    # register original query
    RegisterTerm( $Shop, $Query, ( isValid($Fallback) ? 0 : $Results ) );

    # register fallback query.
    if ( isValid($Fallback) ) {
        RegisterTerm( $Shop, $Fallback, $Results );
    }

    return;
}

#========================================================================================
# function     SearchProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       SearchProducts( $Shop, $ParamBuilder );
#----------------------------------------------------------------------------------------
# description  Execute product search in $Shop. See <package DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder>
#               for configuration options.
#----------------------------------------------------------------------------------------
# input        $Shop           | shop which should be searched | DE_EPAGES::Shop::API::Object::Shop
# input        $ParamBuilder   | all setttings for search | DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder
# return       $ReturnVariable | result with object ids and facets data | ref.hash.
#========================================================================================
sub SearchProducts {
    my ( $Shop, $ParamBuilder ) = @_;
    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');
    TestObject('ParamBuilder', $ParamBuilder, 'DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder');

    my $SearchIndex = $Shop->get('RemoteProductSearchIndex');
    if ( !defined($SearchIndex) || !$SearchIndex->usableAtStorefront ) {
        Error( 'INVALID_SEARCH_INDEX', { 'Shop' => $Shop->pathString } );
    }

    LogDebug( 'Options for Search', $ParamBuilder->params );
    my $time1     = time;
    my $hResponse = $SearchIndex->remoteSearch->searchAsHash( $ParamBuilder->params );
    GetLog->debug( sprintf 'Results: %i Time: %.3f s', $hResponse->{results}, ( time - $time1 ) );

    return $hResponse;
}

#========================================================================================
# function     SearchProductsSqlFallback
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Pager = SearchProductsSqlFallback( $Shop, $ParamBuilder );
#----------------------------------------------------------------------------------------
# description  This method is deprecated and may be removed in near future!<br>
#
#               Construct an PagerSearch, which can execute a product search with SQL.
#               Use of this method should limited to fallback solution if <function SearchProducts>
#               is failing. Keep in mind the SQL fallback execute strict *PATTERN* search, which
#               will lead to heavy load on database and may take long time to yield a result.
#----------------------------------------------------------------------------------------
# input        $Servlet        | Servlet of current context | object
# input        $ParamBuilder   | Search options | DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder
# return       $Pager          | Pager which can execute an SQL search | DE_EPAGES::Search::API::Object::PagerSearch
#========================================================================================
sub SearchProductsSqlFallback {
  my ( $Servlet, $ParamBuilder ) = @_;
  TestObject('Servlet', $Servlet); # generic
  TestObject('ParamBuilder', $ParamBuilder, 'DE_EPAGES::RemoteSearch::API::Object::SearchParamBuilder');

  my $hParams =  $ParamBuilder->params();
  # Save original term for statistics
  my $SearchTerm = $hParams->{query};
  my $hOptions = _buildSQLPagerParams( $Servlet, $hParams );
  GetLog->debug('using fallback SQL product search');

  my $Pager = DE_EPAGES::Search::API::Object::PagerSearch->new($hOptions);
  UpdateSearchStatistics( $Servlet->object->getSite, $SearchTerm, $Pager->countItems() );

  return $Pager;
}

# map ParamBuilder to something digestable for SQL search
sub _buildSQLPagerParams {
    my ( $Servlet, $hParams ) = @_;
    my $ProductClass = LoadClassByAlias('Product');

    my $hOptions->{Parameters} = {
        'Shop.ID'         => $Servlet->object->id,
        'LanguageID'      => $Servlet->languageID,
        'SearchString'    => ReplaceWildcards($hParams->{query} // ''),
        'AttributeIDName' => $ProductClass->attribute('Name')->id,
        'CurrencyID'      => $Servlet->currencyID,
        'OrderBy'         => $Servlet->params('OrderBy'),
        'IsVisible'       => 1,
    };

    # reverse mapping from SearchParamBuilder._mapOrderByToSortBy()
    my $hOrderByMap = {
        PRICE_ASC  => { Attribute => 'ListPrice',   OrderDesc => 0 },
        PRICE_DESC => { Attribute => 'ListPrice',   OrderDesc => 1 },
        NAME_ASC   => { Attribute => 'NameOrAlias', OrderDesc => 0 },
        NAME_DESC  => { Attribute => 'NameOrAlias', OrderDesc => 1 },
    };

    my $Key = $hParams->{sortBy};

    if ($Key) {
        $Servlet->params( 'OrderBy',   $hOrderByMap->{$Key}{'Attribute'} );
        $Servlet->params( 'OrderDesc', $hOrderByMap->{$Key}{'OrderDesc'} );
    }
    else {

        # default
        $Servlet->params( 'OrderBy',   'NameOrAlias' );
        $Servlet->params( 'OrderDesc', 0 );
    }
    LogDebug( q{Options for SQL search:}, $hOptions );
    $hOptions->{SearchDefinition} = GetSearchDefinition( $ProductClass, 'SF-AllStrings' );
    $hOptions->{Servlet} = $Servlet;

    return $hOptions;
}

#========================================================================================
# function     GetDestinctSelections
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetDestinctSelections( $aFacets );
#----------------------------------------------------------------------------------------
# example      my $JSONDestinct = GetDestinctSelections( $hResponse->{facets} );
#----------------------------------------------------------------------------------------
# description  This function filters facet part of search result from function
#               <function SearchProducts>, and return only unique selections or an empty string.
#               For example: Customer is looking for shirt with color black or white, in size XL
#               and should have one pocket. This would result in {"size":"XL", "NumberPockets":"one"}.
#               Color is omitted because more then one value is selected for this facet.
#----------------------------------------------------------------------------------------
# input        $aFacets | $hResponse->{facets} from  <function SearchProducts> | array. ref.
# return       $JSONDestinct | key value string in JSON format | JSON string
#========================================================================================
sub GetDestinctSelections {
    my ($aFacets) = @_;
    my %hDestinct =
        map { ( $_->{attributeName} => _GetSelectedValue($_->{values}) ) }
        grep { $_->{attributeType} =~ m/^(Boolean|PreDefString|PreDefLocalizedString)$/ and _HasOneSelectedValue($_->{values}) }
        @$aFacets;

    return keys %hDestinct ? encode_json(\%hDestinct) : '';
}

sub _HasOneSelectedValue($) {
    my ($aFacetValues) = @_;
    my $Count = 0;
    foreach (@$aFacetValues){ $Count++ if exists $_->{selectedValue}}
    return $Count == 1 ? 1 : 0;
}

sub _GetSelectedValue {
    my ($aFacetValues) = @_;
    foreach (@$aFacetValues){
        return $_->{selectedValue} if exists $_->{selectedValue}
    }
    Error('Hash key "selectedValue" expected');
}

#========================================================================================
# function     UseSearchInCategoryContext
# state        public
#----------------------------------------------------------------------------------------
# syntax       UseSearchInCategoryContext( $Shop );
#----------------------------------------------------------------------------------------
# description  The display of search setting tab in MBO is controlled with shop feature 'DetailedSearch'.
#               It is desireable to execute advanced searches only if Merchant can configure them.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGES::Shop::API::Object::Shop
# return       $HasSettingTab | result of $Shop->featureMaxValue('DetailedSearch') | boolean
#========================================================================================
sub UseSearchInCategoryContext {
    my ($Shop) = @_;
    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    return $Shop->featureMaxValue('DetailedSearch');
}

#========================================================================================
# function     GetPageSizeForSearch
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PageSize = GetPageSizeForSearch( $Shop );
#----------------------------------------------------------------------------------------
# description  Return PagerSettings for product search
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGES::Shop::API::Object::Shop
# return       $PageSize | number of products per page | integer
#========================================================================================
sub GetPageSizeForSearch($) {
    my ($Shop) = @_;
    TestObject( 'Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );
    return GetPageSizeForObject( $Shop, 'SF-FacetedSearch', 'INC-SearchResults' );
}

1;
