#========================================================================================
# package      DE_EPAGES::RemoteSearch::t::RemoteIndexClient
# state        private
#========================================================================================
package DE_EPAGES::RemoteSearch::t::RemoteIndexClient;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::EPagesJ::API::Object::HTTPClient;
use DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use HTTP::Response;
use HTTP::Status qw ( :constants :is status_message );
use JSON;
use Test::MockObject;
use Test::More;
use URI::URL ();
use warnings;

use strict;

sub test {
        my $self = shift;

        $self->testIndexShop();
        $self->testDropIndex();
        $self->testIndexItems();
        $self->testDeleteItems();
        $self->testGetTask();
        $self->testIndexShops();
        $self->testCreateCategory();

        done_testing();
}

#----------------------------------------------------------------------------------------
sub testIndexShop {
    my $self   = shift;

    my $Client = $self->_createRemoteSearchWithMockedDatabase(
         $self->_fakeUrls(),
        'HttpClient' => $self->_mockHTTPClient( { 'post' => \&_assertCreateIndexContents } )
    );
    my $Task = $Client->indexShop(1, $self->_createShop());
    isa_ok($Task, 'DE_EPAGES::RemoteSearch::API::DTO::Task');
}

sub _createRemoteSearchWithMockedDatabase {
    my $self = shift;
    my ($aFakedUrls, $hFakedHttpClient) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient',
            [@$aFakedUrls, $hFakedHttpClient]);
    $Mockify->addMockWithReturnValue('_LoadRootObject', $self->_createSystem());
    return $Mockify->getMockObject();
}

sub testDropIndex {
    my $self   = shift;

    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->new(
        @{$self->_fakeUrls()},
        'HttpClient' => $self->_mockHTTPClient( { 'delete' => \&_assertDropIndexContents } )
    );
    my $Task = $Client->dropIndex(2);
    isa_ok($Task, 'DE_EPAGES::RemoteSearch::API::DTO::Task');
    is($Task->state, 'Pending', 'state is pending');
}

sub testIndexItems {
    my $self   = shift;

    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->new(
        @{$self->_fakeUrls()},
        'HttpClient' => $self->_mockHTTPClient( { 'post' => \&_assertIndexItemsContents } )
    );
    my $Task = $Client->indexItems([map {{'resourceId' => $_ }} qw (100 101 102)], 3);
    isa_ok($Task, 'DE_EPAGES::RemoteSearch::API::DTO::Task');
    is($Task->state, 'Pending', 'state is pending');
}

sub testDeleteItems {
    my $self   = shift;

    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->new(
        @{$self->_fakeUrls()},
        'HttpClient' => $self->_mockHTTPClient( { 'post' => \&_assertDeleteItemsContents } )
    );
    my $Task = $Client->deleteItems([map {{'resourceId' => $_ }} qw (100)], 4);
    isa_ok($Task, 'DE_EPAGES::RemoteSearch::API::DTO::Task');
    is($Task->state, 'Pending', 'state is pending');
}

sub testGetTask {
    my $self   = shift;

    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->new(
        @{$self->_fakeUrls()},
        'HttpClient' => $self->_mockHTTPClient( { 'get' => \&_assertgetTaskContents } )
    );
    my $Task = $Client->getTask('http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563');
    isa_ok($Task, 'DE_EPAGES::RemoteSearch::API::DTO::Task');
    is($Task->state, 'Pending', 'state is pending');
}

sub testIndexShops {
    my $self   = shift;

    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->new(
        @{$self->_fakeUrls()},
        'HttpClient' => $self->_mockHTTPClient( { 'post' => \&_assertIndexShopsContents } )
    );
    my $Task = $Client->indexShops([map {{'resourceId' => $_ }} qw (100 101 102)], 3);
    isa_ok($Task, 'DE_EPAGES::RemoteSearch::API::DTO::Task');
    is($Task->state, 'Pending', 'state is pending');
}

sub testCreateCategory {
    my $self = shift;

    my $Client = DE_EPAGES::RemoteSearch::API::Object::RemoteIndexClient->new(
        @{$self->_fakeUrls()},
        'HttpClient' => $self->_mockHTTPClient( { 'put' => \&_assertCreateCategoryContents } )
    );
    my $MockedCategory = $self->_createCategory();
    $Client->createCategory($MockedCategory);
}

#######################################################################
# Mockers
#######################################################################

sub _mockHTTPClient {
    my $self           = shift;
    my ($hMockMethods) = @_;
    my $mockRemote     = Test::MockObject->new();
    $mockRemote->set_isa('DE_EPAGES::EPagesJ::API::Object::HTTPClient');
    $mockRemote->fake_module( 'DE_EPAGES::EPagesJ::API::Object::HTTPClient', %$hMockMethods );
    while ( my ( $method, $handler ) = each %$hMockMethods ) {
        $mockRemote->mock( $method, $handler );
    }
    return $mockRemote;
}

sub _fakeUrls {
    my $self = shift;
    return [
        'IndexShopUrl'      => URI::URL->new('http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4'),
        'IndexShopsUrl'     => URI::URL->new('http://localhost:8088/rs-test/product/Store'),
        'IndexItemsUrl'     => URI::URL->new('http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4/update'),
        'DeleteItemsUrl'    => URI::URL->new('http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4/delete'),
        'CreateCategoryUrl' => URI::URL->new('http://localhost:8088/rs-test/Store/GUID-1-2-3-4/category')
    ];
}

sub _assertCreateIndexContents {
    shift;
    my $hShop = {
            'guid' => 'guid',
            'publicId' => 'alias',
            'objectId' => 'objectId',
            'storename' => 'TestStore',
            'domainName' => 'TestDomain',
            'systemDomainName' => 'SystemTestDomain',
            'domainSslSecured' => JSON::false,
            'systemDomainSslSecured' => JSON::false,
        };
    my ( $URL, $hHeader, $Content ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4', 'post path for CreateIndex' );
    is_deeply(
        decode_json($Content),{
                'shop' => $hShop,
                'replicationToken' => 1,
                'nosolr' => undef,
            },
        'post contents'
    );
    return HTTP::Response->new( HTTP_ACCEPTED, status_message(HTTP_ACCEPTED), undef,
        '{"link":{"href":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563","type":"application/xml","hrefURI":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563","rel":"self"},"createdOn":"2012-03-08T16:57:08.728Z","startedAt":"2012-03-08T16:57:08.894Z","completedAt":null,"state":"Pending"}'
    );
}

sub _assertDropIndexContents {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4', 'delete url for DropIndex' );
    return HTTP::Response->new( HTTP_ACCEPTED, status_message(HTTP_ACCEPTED), undef,
        '{"link":{"href":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563","type":"application/xml","hrefURI":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563","rel":"self"},"createdOn":"2012-03-08T16:57:08.728Z","startedAt":"2012-03-08T16:57:08.894Z","completedAt":null,"state":"Pending"}'
    );
}

sub _assertgetTaskContents {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563', 'url for task' );
    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef,
        '{"link":{"href":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563","type":"application/xml","hrefURI":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87c00563","rel":"self"},"createdOn":"2012-03-08T16:57:08.728Z","startedAt":"2012-03-08T16:57:08.894Z","completedAt":null,"state":"Pending"}'
    );
}
sub _assertIndexShopsContents {
    shift;
    my ($URL, $hHeader, $Content) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/product/Store', 'post url for indexShops');
    is($Content, encode_json({
    'replicationToken' => 3,
    'indexItems' => {
        items => [
            { 'resourceId' => '100' },
            { 'resourceId' => '101' },
            { 'resourceId' => '102' },
        ],
    }
    }), 'update items contents.');

    return HTTP::Response->new( HTTP_ACCEPTED, status_message(HTTP_ACCEPTED), undef,
        '{"link":{"href":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87caaaaa","type":"application/xml","hrefURI":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87caaaaa","rel":"self"},"createdOn":"2012-03-08T16:57:08.728Z","startedAt":"2012-03-08T16:57:08.894Z","completedAt":null,"state":"Pending"}'
    );
}

sub _assertIndexItemsContents {
    shift;
    my ($URL, $hHeader, $Content) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4/update', 'post url for InsertOrUpdateItems');
    is($Content, encode_json({
    'replicationToken' => 3,
    'indexItems' => {
        items => [
            { 'resourceId' => '100' },
            { 'resourceId' => '101' },
            { 'resourceId' => '102' },
        ],
    }
    }), 'update items contents.');

    return HTTP::Response->new( HTTP_ACCEPTED, status_message(HTTP_ACCEPTED), undef,
        '{"link":{"href":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87caaaaa","type":"application/xml","hrefURI":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87caaaaa","rel":"self"},"createdOn":"2012-03-08T16:57:08.728Z","startedAt":"2012-03-08T16:57:08.894Z","completedAt":null,"state":"Pending"}'
    );
}

sub _assertDeleteItemsContents {
    shift;
    my ($URL, $hHeader, $Content) = @_;
    is( $URL->abs, 'http://localhost:8088/rs-test/product/Store/GUID-1-2-3-4/delete', 'post url for DeleteItems');
    is($Content, encode_json({
    'replicationToken' => 4,
    'indexItems' => {
        'items' => [{ 'resourceId' => '100' }],
    }
    }), 'delete items contents.');

    return HTTP::Response->new( HTTP_ACCEPTED, status_message(HTTP_ACCEPTED), undef,
        '{"link":{"href":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87caaaaa","type":"application/xml","hrefURI":"http://localhost:8088/rs-test/task/4f58e4e451510d2b87caaaaa","rel":"self"},"createdOn":"2012-03-08T16:57:08.728Z","startedAt":"2012-03-08T16:57:08.894Z","completedAt":null,"state":"Pending"}'
    );

}


sub _assertCreateCategoryContents {
    shift;
    my ($URL, $hHeader, $Content) = @_;

    is( $URL->abs, 'http://localhost:8088/rs-test/Store/GUID-1-2-3-4/category', 'put url for CreateCategory');

    is_deeply(
        decode_json($Content),{
            'alias' => 'Pullover',
            'name' => { 'de' => 'Hose', 'en' => 'Pants' },
            'guid' => '591C1849-A2DA-C999-9593-AC15454617A3',
            'isSpecialOffer' => 'false',
            'displayName' => { 'de' => 'Schne Hose, Bruder', 'en' => 'Nice Pants Bro' },
            'objectId' => 32068,
            'parentId' => 20691,
            'position' => 150,
            'siteId' => 'objectId',
            'visible' => 'false',
            'visibleInNavigation' => 'true',
        },
        'put contents'
    );

    return HTTP::Response->new( HTTP_NO_CONTENT, status_message(HTTP_NO_CONTENT), undef, undef );
}

sub _createShop {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'alias',
        'DomainName' => 'TestDomain',
        'GUID' => 'guid',
        'LocaleID' => 'en_GB',
        'HasSSLCertificate' => 0,
        'ID' => 'objectId',
        'Languages' => [
            { 'Code2' => 'de', 'LanguageID' => 1 },
            { 'Code2' => 'en', 'LanguageID' => 2 }
        ],
    };

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $Mockify->addMockWithReturnValueAndParameterCheck('siteLocale', undef, ['string']);
    return $Mockify->getMockObject();
}

sub _createSystem {
    my $self = shift;
    my $hAttributes = {
        'DomainName' => 'SystemTestDomain',
        'HasSSLCertificate' => 0,
        'StoreName' => 'TestStore',
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Site', $hAttributes);
    return $Mockify->getMockObject();
}

sub _createCategory {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'Pullover',
        'GUID' => '591C1849-A2DA-C999-9593-AC15454617A3',
        'IsSpecialOffer' => 0,
        'IsVisible' => 0,
        'ID' => 32068,
        'Parent' => $self->_createRootCategory(),
        'Position' => 150,
        'Site' => $self->_createShop(),
        'VisibleInNavigation' => 1,
    };
    my $DefaultLanguageId = 1;

    my $MockifyCategory = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Content::API::Object::Category', $hAttributes, $DefaultLanguageId
    );
    $MockifyCategory->addLanguageDependentAttribute('NameOrAlias', {
        1 => 'Hose',
        2 => 'Pants',
    });
    $MockifyCategory->addLanguageDependentAttribute('DisplayTitle', {
        1 => 'Schne Hose, Bruder',
        2 => 'Nice Pants Bro',
    });

    return $MockifyCategory->getMockObject();

}

sub _createRootCategory {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'Categories',
        'ID' => 20691
    };

    my $MockifyRootCategory = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Content::API::Object::StartPage', $hAttributes
    );

    return $MockifyRootCategory->getMockObject();
}


####### Main ############

__PACKAGE__->runScript();
