#========================================================================================
# package      DE_EPAGES::SEO::API::Attributes::ContentObject_Ext
# state        private
# base         DE_EPAGES::Object::API::BaseAttribute
# description  Extends the content object attributes
#========================================================================================
package DE_EPAGES::SEO::API::Attributes::ContentObject_Ext;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use DE_EPAGES::SEO::API::SEO qw ( PerformSEOCheck );
use DE_EPAGES::SEO::API::Table::SEOQuality qw (
    ExistsSEOQualityByContentObjectID
    GetInfoSEOQualityByContentObjectID
    GetPKeySEOQualityByContentObjectID
    GetInfoSEOQualitiesByContentObjectID
    InsertSEOQuality
    UpdateSEOQuality
);

#========================================================================================
# function     getAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Package->( $Object, $aNames );
# example      # never call getAttributes directly!
#               $Value = $Object->get( $AttributeName );
#----------------------------------------------------------------------------------------
# description  Returns attribute values of an object.
#               Used by <function DE_EPAGES::Object::Object::Object::get>
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# input        $aNames | list of attribute names | ref.array
# return       $hValues | attribute name => value hash | ref.hash
#========================================================================================
sub getAttributes {
    shift;
    my ($Object, $aNames, $LanguageID) = @_;

    my %Info;
    foreach my $Name (@$aNames) {
        if ($Name eq 'SeoCheckRatings') {
            $Info{$Name} = {
                'Title'          => 0.35,
                'Url'            => 0.25,
                'Description'    => 0.2,
                'AdvDescription' => 0.2,
            };
        }
        elsif ($Name eq 'SEOQuality') {
            if (ExistsSEOQualityByContentObjectID($Object->id, $LanguageID)) {
                $Info{$Name} = GetInfoSEOQualityByContentObjectID($Object->id, $LanguageID)->{'Quality'};
            }
            else {
                $Info{$Name} = 0;
            }
        }
        elsif ($Name eq 'SEOQualityAverage') {
            my $hValues = $Object->get('CalculatedSEOCheckValues');
            $Info{$Name} = $hValues->{'NewSEOQualityAverage'};

        }
        elsif ($Name eq 'CalculatedSEOCheckValues') {
            $Info{$Name} = PerformSEOCheck($Object);
        }
    }
    return \%Info;
}

#========================================================================================
# function     setAttributes
# state        private
# syntax       $Value = $Object->set($AttributeName, $Value);
#========================================================================================
sub setAttributes {
    shift;
    my ($Object, $hAttributes, $LanguageID) = @_;

    foreach my $Name (keys %$hAttributes) {
        if ($Name eq 'SEOQuality') {
            my $SiteID = $Object->getSite()->id;
            my $Value = defined $hAttributes->{$Name} ? int($hAttributes->{$Name}) : 0;
            my $hSEOQualityValues = {
                'SiteID' => $SiteID,
                'LanguageID' => $LanguageID,
                'ContentObjectID' => $Object->id,
                'Quality' => $Value
            };

            if (ExistsSEOQualityByContentObjectID($Object->id, $LanguageID)) {
                UpdateSEOQuality(GetPKeySEOQualityByContentObjectID($Object->id, $LanguageID), $hSEOQualityValues);
            } else {
                InsertSEOQuality($hSEOQualityValues);
            }
        }
    }
}

#========================================================================================
# function     getAllAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Package->getAllAttributes( $Object, $aNames );
#----------------------------------------------------------------------------------------
# description  Returns multiple attribute values. Don't call this function directly.
#               This method is used by DALObject->exportableAttributes().
#----------------------------------------------------------------------------------------
# input        $Object   | object       | object
# input        $aNames     | list of attribute names (optional) | array ref
# return       $hValues    | attribute name => language => value hash | hash ref
#========================================================================================
sub getAllAttributes {
    my $self = shift;
    my ($Object, $aNames) = @_;

    my %Values;
    foreach my $Name (@$aNames) {
        if($Name eq 'SEOQuality') {
            foreach ( @{ GetInfoSEOQualitiesByContentObjectID($Object->id) } ) {
                $Values{$Name}{$_->{'LanguageID'}} = $_->{'Quality'};
            }
        } else {
            $Values{$Name} = $self->getAttributes($Object, [$Name])->{$Name};
        }
    }
    return \%Values;
}

1;
