use strict;
use Test::More tests => 31;
use DE_EPAGES::Core::API::File qw ( DeleteFile ExistsFile );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::String qw ( GetWordCount );
use DE_EPAGES::Database::API::Connection qw (
    ConnectTestSite
    DisconnectSite
    GetTestSite
    RunOnStore
);
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::SEO::API::SEO qw (
    CalculateSEOQuality
    GetSEOCheckValues
    GetShopSEOQualityValues
    PerformSEOCheck
    UpdateSEOQuality
);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::WebInterface::API::FakeRequest;
use Sys::Hostname;

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};

## no critic qw( ProhibitLongLines )
sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    my $Test = LoadObjectByPath('Test');
    my $Shop = $Test->insertChild('Shop', {
        Alias => 'TestShop',
        InitialShopWizardStartup => 0
    });
    my $ShopLangID = $Shop->get('LanguageID');

    $Shop->featureMaxValue( 'Pages', 10 );  ## no critic qw(ProhibitMagicNumbers)
    $Shop->featureMaxValue( 'Languages', 2);
    my $LanguageID = GetPKeyLanguageByCode( 'de' );
    $Shop->addLanguage($LanguageID);

    my $StartPage = $Shop->child('Categories');

    # test api
    my $Cat123 = $StartPage->insertChild('Category', { Alias => 'Cat123' });
    # direct
    $Cat123->set( { 'SEOQuality' => 33}, $ShopLangID );
    cmp_ok($Cat123->get('SEOQuality', $ShopLangID), '==', 33, 'set ok');  ## no critic qw(ProhibitMagicNumbers)

    # api
    my $hVals = {
        1 => { 'NewSEOQuality' => 55 },
        2 => { 'NewSEOQuality' => 66 },
    };
    UpdateSEOQuality($Cat123, $hVals);
    cmp_ok($Cat123->get('SEOQuality', 1), '==', 55, 'UpdateSEOQuality get by lang ok');  ## no critic qw(ProhibitMagicNumbers)
    cmp_ok($Cat123->get('SEOQuality', 2), '==', 66, 'UpdateSEOQuality get by lang ok');  ## no critic qw(ProhibitMagicNumbers)

    # start page seo test
    my $hValues = GetSEOCheckValues($Shop, $StartPage, $ShopLangID);
    # in clean shop SEO should be very bad:
    ok(!$hValues->{'Name'}, 'GetSEOCheckValues empty name');
    cmp_ok($hValues->{'Url'}->{'Value'}, 'eq', $StartPage->get('CanonicalURL', $ShopLangID), 'GetSEOCheckValues default URL');
    ok(!$hValues->{'Title'}->{'Value'}, 'GetSEOCheckValues empty title');
    ok(!$hValues->{'Title'}->{'Value'}, 'GetSEOCheckValues empty title');
    ok(!$hValues->{'Description'}->{'Value'}, 'GetSEOCheckValues empty desc');
    ok(!$hValues->{'AdvDescription'}->{'Value'}, 'GetSEOCheckValues empty long desc');
    foreach my $Token ( qw (Url Title Description AdvDescription )) {
        cmp_ok($hValues->{$Token}->{'RatingClass'}, 'eq', 'red', 'GetSEOCheckValues RatingClass red');
    }
    cmp_ok(CalculateSEOQuality($hValues, $StartPage), '==', 0, 'CalculateSEOQuality returns 0');

    # use article and set some SEO-nice values
    $Shop->set({ 'DomainName' => 'www.mytestshop.com' });
    $Shop->set({ 'Name' => 'MyTestShop' }, $ShopLangID);
    my $Art1 = $StartPage->insertChild('Article', { Alias => 'Article' });
    $Art1->set({
        'Name' => 'Nice test article - all u ever wanted to know!',
        'Description' => 'Curabitur pretium elementum purus, ac tincidunt tellus lobortis id. In quis blandit purus. Quisque quis elit varius est faucibus.',
        'Text' => 'Nunc arcu <em>metus</em>, dapibus in bibendum et, aliquet eu lectus. Donec ipsum tellus, tempor non dignissim at, vehicula lacinia enim. Morbi dignissim lacus eu ipsum placerat sit amet volutpat leo vestibulum.',
        'URI' => 'top-article-kat1',
    }, $ShopLangID);

    # general attributes
    my $hGeneralValues = {
        FormerSEOQuality => $Art1->get('SEOQualityAverage') || 0,
        SiteUrl => $Shop->get('WebUrl'),
        Domain => $Shop->get('DomainName'),
    };
    TriggerHook('SEOCheckTests', {
        'Object'        => $Art1,
        'Shop'          => $Shop,
        'Values'        => $hGeneralValues,
    });
    my $hLangValues = GetSEOCheckValues($Shop, $Art1, $ShopLangID);
    $hValues->{$ShopLangID} = {
        %$hGeneralValues,
        %$hLangValues,
    };

    foreach my $Token ( qw (Url Title Description AdvDescription )) {
        cmp_ok($hValues->{$ShopLangID}->{$Token}->{'Rating'}, '==', 1, 'GetSEOCheckValues Rating 1');
    }
    cmp_ok(CalculateSEOQuality($hValues->{$ShopLangID}, $Art1), '==', 90, 'CalculateSEOQuality returns 90 as image is missing');  ## no critic qw(ProhibitMagicNumbers)

    # check data
    my $hPerformValues = PerformSEOCheck($Art1);
    UpdateSEOQuality($Art1, $hPerformValues);
    cmp_ok($Art1->get('SEOQualityAverage'), '==', 50, 'SEOQualityAverage 2');  ## no critic qw(ProhibitMagicNumbers)

    #
    cmp_ok(GetWordCount('w1 - w2 | word3+word4 _ w5-w6'), '==', 7,  'GetWordCount counts only words');

    # abstracted from a customer's text
    cmp_ok(
        GetWordCount(
         'AB 1234 cccccc ddddd Ffff-Tttttt mmm Bbbb Dddddddd X-Xxx Ffff-Tttttt mmm Bbbb Dddddddd. 1 kkkkkkkkkkkkkk Aaaaaaaa fff Vvvvvvvvvv, eeeee mmm Eeeee Sssssssss. Mmmmmmmm: 100% Ppppppppp Mmmm: cc. 11 cc llll, 11 cc bbbb MM 1234 mmmmmm bbbbb Ffff-Tttttt mmm Bbbb Dddddddd ii 1 Fffffffffffffffff.'
        ),
        '==',
        44,     ## no critic qw(ProhibitMagicNumbers)
        'GetWordCount counts only words'
    );

    # title and name similarity (EPG-20601)
    $Art1->set({
        'Name' => 'Nice test article - all u ever wanted to know!',
        'Title' => 'Nice test article - all u ever wanted to know! now FREE shipping',
    }, $ShopLangID);
    $hValues = GetSEOCheckValues($Shop, $Art1, $ShopLangID);
    cmp_ok( $hValues->{'Title'}->{'Rating'}, '==', 1, 'EPG-20601: Be more tolarant, if name is part of the title. Title rating is ' . $hValues->{'Title'}->{'Rating'} . '.' );

    $Art1->set({
        'Name' => 'Wirkstoff: Aloe Vera, Swisa Beauty',
        'Title' => 'Swisa Beauty - Aloe Vera - hautfreundliche Produkte',
    }, $ShopLangID);
    $hValues = GetSEOCheckValues($Shop, $Art1, $ShopLangID);
    cmp_ok( $hValues->{'Title'}->{'Rating'}, '==', 1, 'EPG-20601: Be more tolarant, if name is part of the title. Title rating is ' . $hValues->{'Title'}->{'Rating'} . '.' );


    #===================================================================================#
    # EPG-27590: generate and submit a sitemap                                          #
    #===================================================================================#

    # (1) rating == 3 (red)
    $Shop->set({ 'ClosedByMerchant' => 1 });
    $hValues = GetShopSEOQualityValues($Shop);
    cmp_ok( $hValues->{'Sitemap'}->{'Rating'}, '==', 3, 'ShopSEO: rating for sitemap equals 3 if the shop is closed by merchant');

    # (2) rating == 2 (yellow)
    $Shop->set({ 'ClosedByMerchant' => 0 });
    DeleteFile($Shop->get('SitemapPathFileName'))       if ExistsFile($Shop->get('SitemapPathFileName'));
    DeleteFile($Shop->get('MobileSitemapPathFileName')) if ExistsFile($Shop->get('MobileSitemapPathFileName'));
    $hValues = GetShopSEOQualityValues($Shop);
    cmp_ok( $hValues->{'Sitemap'}->{'Rating'}, '==', 2, 'ShopSEO: rating for sitemap equals 2 if the shop is open, but nothing was done with respect to sitemaps');

    # (3) rating == 2 (yellow)
    $Shop->set({ 'DomainName' => hostname });
    my $URL = $Shop->get('WebUrlAdmin');
    my $URLParams = join('&', 'ViewAction=ViewSitemap', 'ChangeAction=BlockMonitor', 'ObjectID=' . $Shop->id);
    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    my $Response = $FakeRequest->get($URL . '/?' . $URLParams);
    ok( $Response->is_success, 'ShopSEO: created a new sitemap.xml' );
    $hValues = GetShopSEOQualityValues($Shop);
    cmp_ok( $hValues->{'Sitemap'}->{'Rating'}, '==', 2, 'ShopSEO: rating for sitemap equals 2 if the shop is open, a sitemap was generated, but the sitemap were not submitted to a search engine');

    # (4) rating == 1 (green)
    $Shop->set({ SubmittedSitemapDomain => $Shop->get('ProtocolAndServer') });  # mimic registration (ChangeAction = RegisterSitemap as admin)
    $hValues = GetShopSEOQualityValues($Shop);
    cmp_ok( $hValues->{'Sitemap'}->{'Rating'}, '==', 1, 'ShopSEO: rating for sitemap equals 1, since the sitemap url was registered on search-engines');


    ok( Cleanup(), 'cleanup');
    DisconnectSite();
}

sub Cleanup {
    LoadObjectByPath('/Test')->delete if ExistsObjectByPath('/Test');
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    return 1;
}

RunScript( Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
