#========================================================================================
# package      DE_EPAGES::Shop::API::Object::Feature
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  Object interface for Feature.
#========================================================================================
package DE_EPAGES::Shop::API::Object::Feature;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( GetInfoCartridge GetPKeyCartridgeByPackage );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject );
use DE_EPAGES::Shop::API::Table::ShopFeature qw ( GetInfoShopFeaturesByFeatureID );
use DE_EPAGES::Shop::DAL::Feature qw( DAL_DeleteShopFeaturesByFeatureID );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $Feature->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'Feature' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $Feature->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Feature = DE_EPAGES::Shop::API::Object::Feature->insert($hInfo);
# example      my $Class = LoadClassByAlias('Feature');
#               my $Feature = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertFeature' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $Feature | new object | object
# hook         OBJ_InsertFeature | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>FeatureID     - feature id           - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertFeature', {'Object' => $self, 'FeatureID' => $self->id });
    delete $hInfo->{$_} foreach qw ( CartridgeID MaxValue Unit VerifyFunctionName AllowToExceed ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Feature->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteFeature' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteFeature | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>FeatureID     - feature id           - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;
    TriggerHook('OBJ_DeleteFeature', {'Object' => $self, 'FeatureID' => $self->id });

    # remove the feature assignment from widget types
    $_->set({ 'Feature' => undef }) foreach @{LoadClassByAlias('WidgetType')->getObjects('ByFeatureID', [$self->id])};
    DAL_DeleteShopFeaturesByFeatureID( $self->id );
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Feature->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateFeature' and 'OBJ_AfterUpdateFeature'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateFeature | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>FeatureID     - feature id           - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateFeature | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>FeatureID     - feature id           - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateFeature', {'Object'=>$self, 'Values'=> $hValues, 'FeatureID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateFeature',  {'Object'=>$self, 'Values'=> $hValues, 'FeatureID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# syntax       $hValues = DE_EPAGES::Shop::API::Object::Feature->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'AllowToExceed' => 0,
    };
}

#========================================================================================
# function     findShops
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aShopIDs = $Feature->findShops;
#----------------------------------------------------------------------------------------
# description  Returns the ids of all shops that have the feature.
#----------------------------------------------------------------------------------------
# return       $aShopIDs | list of shop ids | array of int
#========================================================================================
sub findShops {
    my $self = shift;

    return [
        map { $_->{ShopID} } grep { $_->{'MaxValue'} > 0 }
        @{ GetInfoShopFeaturesByFeatureID( $self->id ) }
    ];
}

#========================================================================================
# function     shops
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aShopIDs = $Feature->shops;
#----------------------------------------------------------------------------------------
# description  Returns all shops that have the feature.
#----------------------------------------------------------------------------------------
# return       $aShops | list of shop objects | array of object
#========================================================================================
sub shops {
    my $self = shift;

    return [
        map { LoadObject( $_ ) }
        @{ $self->findShops }
    ];
}

#========================================================================================
# function     importXML
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = __PACKAGE__->importXML($Handler, $hElement, $Class, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Imports an object from an XML file. This function is called at the XML
#               start tag. The end tag triggers the <function importXMLEnd> function.
#----------------------------------------------------------------------------------------
# input        $Handler | xml import handler with support function addObject | object
# input        $hElement | XML element structure, see XML::SAX | ref.hash
# input        $Class | class object; the new object will be an instance of this class | object
# input        $hAttributes | attributes hash | ref.hash
# return       $Object | the imported object | object
#========================================================================================
sub importXML {
    my ($ObjectPackage, $Handler, $hElement, $Class, $hAttributes) = @_;

    # get the cartridge id by package name
    if( exists $hAttributes->{'Cartridge'} ) {
        my $Package = $hAttributes->{'Cartridge'};
        my $CartridgeID = GetPKeyCartridgeByPackage($Package);
        $hAttributes->{'CartridgeID'} = $CartridgeID;
        delete $hAttributes->{'Cartridge'};
    }

    return $ObjectPackage->SUPER::importXML($Handler, $hElement, $Class, $hAttributes );
}

#========================================================================================
# function     exportXMLAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAttributes = $Object->exportXMLAttributes( $Driver, $hAllAttributes );
#----------------------------------------------------------------------------------------
# description  Replaces the XML attribute CartridgeID by Cartridge, which contains the
#               package name.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function allAttributes> | ref.hash
# return       $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLAttributes {
    my $self = shift;
    my ($Driver, $hAllAttributes) = @_;

    # replace CartridgeID by Cartridge package
    my $hAttributes = $self->SUPER::exportXMLAttributes( $Driver, $hAllAttributes );
    if(defined $hAttributes->{'CartridgeID'}) {
        $hAttributes->{'Cartridge'} = GetInfoCartridge( $hAttributes->{'CartridgeID'} )->{'Package'};
        delete $hAttributes->{'CartridgeID'};
    }
    return $hAttributes;
}

1;
