#========================================================================================
# package      DE_EPAGES::Trigger::API::Object::BatchProcessHandler;
# state        public
#----------------------------------------------------------------------------------------
# description  Abstract class for processing queued objects.
#               <br><br>
#               This class collect items via method <function addItem>($object) and the
#               processing is startet by <function processItemList>(), which calls
#               implementation depending <function processItem>() method. If at construction
#               time parameter 'BatchSize' is specified, than processing is automatically
#               triggered the moment <function countItems>() >= <function batchSize>() is true
#               <br><br>
#               see t/BatchProcessHandler.t for subclassing, injecting options/objects example<br>
#               see <package DE_EPAGES::Trigger::API::BatchProcessor> - how to create event
#               based batch processing.<br>
#               see t/BatchProcessor2.t for an example
#----------------------------------------------------------------------------------------
# example
#
#package MyCartridge::Hooks::ProductBatchHandler;
#
#use parent qw( DE_EPAGES::Trigger::API::Object::BatchProcessHandler );
#
# #must be implemented
#sub processItem {
#    my $self = shift;
#    my ($hProduct) = @_;
#
#    # do something with each object
#
#    return 'process Product ' . $hProduct->{ProductID} . "\n";
#}
#
#sub processItemList {
#    my $self     = shift;
#    my $products = $self->items();
#
#    my $text;
#
#    foreach my $hProduct (@$products) {
#      $text .= $self->processItem($hProduct);
#    }
#
#    print $text;
#
#    # remove processed items from the list
#    $self->cleanupItems();
#
#}
#========================================================================================
package DE_EPAGES::Trigger::API::Object::BatchProcessHandler;

use strict;

use Scalar::Util qw( blessed );

use DE_EPAGES::Core::API::EpagesConfig qw ( EpagesConfig );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Handler = DE_EPAGES::Trigger::API::Object::BatchProcessHandler->new( {BatchSize => 50} );
#               my $Handler = DE_EPAGES::Trigger::API::Object::BatchProcessHandler->new();
#----------------------------------------------------------------------------------------
# description  You may pass optional parameter 'BatchSize' to define the
#               number of items queued before <function processItemList>() is called
#               automatically.
#----------------------------------------------------------------------------------------
# input        $hOptions |<ul><li>BatchSize - integer > 1 (optional)</li></ul> | ref.hash
# return       $object | new batch process handler | object
#========================================================================================
sub new {
    my $class    = shift;
    my ($hOptions) = @_;
    $hOptions //= {};

    my $self = bless({
        '_Items' => [],
    }, $class);
    $self->{'Options'} = $self->_initBatchProcessHandler($hOptions);

    return $self;
}
#----------------------------------------------------------------------------------------
sub _initBatchProcessHandler {
    my $self = shift;
    my ($hOptions) = @_;

    if (!defined $hOptions->{'BatchSize'}){
        my $DefaultValue = 1000;
        $hOptions->{'BatchSize'} = EpagesConfig()->get( 'DE_EPAGES::Trigger', 'BatchSize', $DefaultValue );
    }

    return $hOptions;
}
#========================================================================================
# function     addItem
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->addItem( $Item );
#----------------------------------------------------------------------------------------
# description  Add item to queue for later processing. If 'BatchSize' is set, this method
#               checks item count and calls processItemList() automatically if
#               <function countItems>() >= <function batchSize>() is true.
#----------------------------------------------------------------------------------------
# input        $Item |item for processing| type depends on impl of <function processItem>($item)
#========================================================================================
sub addItem {
    my $self = shift;
    my ($Item) = @_;

    push @{ $self->items() }, $Item;

    if ( $self->_isBatchSizeReached() ) {
        $self->processItemList();
    }
    return;
}
#----------------------------------------------------------------------------------------
sub _isBatchSizeReached {
    my $self = shift;

    return ($self->batchSize() > 0 and $self->batchSize() <= $self->countItems());
}

#========================================================================================
# function     processItemList
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->processItemList();
#----------------------------------------------------------------------------------------
# description  Passes each item to <function processItem>($hItem), after all items are processed
#               the items are flushed by calling method cleanupItems(). Override this
#               method to handle more than one object at once. You must delete processed
#               items by calling <function cleanupItems>() if you reimplement this method.
#========================================================================================
sub processItemList {
    my $self = shift;

    $self->processItem($_) foreach @{ $self->items };
    $self->cleanupItems();
    return;
}
#========================================================================================
# function     items
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aItemList = $Handler->items();
#----------------------------------------------------------------------------------------
# description  Returns list of items, accumulated by <function addItem>, for this handler.
#----------------------------------------------------------------------------------------
# return       $aItemList | list of items stored by this handler | ref.array
#========================================================================================
sub items {
    my $self = shift;

    return $self->{_Items};
}
#========================================================================================
# function     countItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       $count = $Handler->countItems();
#----------------------------------------------------------------------------------------
# description  Number of items stored for processing
#----------------------------------------------------------------------------------------
# return       $count | current count of unprocessed items | integer
#========================================================================================
sub countItems {
    my $self = shift;

    return scalar @{ $self->items() };
}
#========================================================================================
# function     processItem
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->processItem($Item);
#----------------------------------------------------------------------------------------
# description  Abstract, must be implemented!<br>
#               processItem is called by <function processItemList> to handle each object
#               separately. If you want control over the whole list prcoessing you must
#               override method processItemList.
#----------------------------------------------------------------------------------------
# input        $Item |item for processing, see <function addItem>| scalar
#========================================================================================
sub processItem {
    my $self = shift;
    my ($Item) = @_;
    die 'abstract method processItem called';
}
#========================================================================================
# function     cleanupItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->cleanupItems();
#----------------------------------------------------------------------------------------
# description  Empties the list of items. Baware this method don't distinguish bewteen
#               processed or unprocessed items.
#========================================================================================
sub cleanupItems {
    my $self = shift;

    $self->{_Items} = [];
}
#========================================================================================
# function     packageName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PackageName = $Handler->packageName();
#----------------------------------------------------------------------------------------
# description  Returns the package name.
#----------------------------------------------------------------------------------------
# return       $PackageName| name of the package | string
#========================================================================================
sub packageName {
    my $self = shift;

    return blessed($self);
}
#========================================================================================
# function     batchSize
# state        public
#----------------------------------------------------------------------------------------
# syntax       $BatchSize = $Handler->batchSize();
#----------------------------------------------------------------------------------------
# description  Number of items stored by the handler (n), which will trigger
#               <function processItems>() automatically.
#               The handler stores up to n-1 items whithout starting processing.
#               Returns 0 if 'BatchSize' is was not set in <function new>.
#----------------------------------------------------------------------------------------
# return       $BatchSize | number of items this handler can store before auto
#               processing, else 0 | integer
#========================================================================================
sub batchSize {
    my $self = shift;

    return $self->{'Options'}->{'BatchSize'};
}

1;
