############################################################################
#
#   BatchProcessHandler.t
#
#   BatchProcessHandler is abstract, hence we need to subclass it. For
#   testing of processItems(), processItemList() a simple Item class
#   with no real functionality is created
#
############################################################################
use strict;

## no critic (DE_EPAGES::Style::RequireCommentForExportedFunctions)
## no critic (Modules::ProhibitMultiplePackages)
package My::API::Object::ExampleBatchHandler;
use parent qw( DE_EPAGES::Trigger::API::Object::BatchProcessHandler );

use DE_EPAGES::Core::API::Error qw (Error);

sub new {
    my $class = shift;
    my ($hOptions) = @_;

    return $class->SUPER::new($hOptions);
}

sub customOption {
    my $self = shift;
    return $self->{Options}->{CustomOption} if ( defined $self->{Options}->{CustomOption} );
}

sub processItem {
    my $self = shift;
    my ($Item) = @_;

    # to keep it short we don't do anything usefull
    Error('No Valid ID for Item') if ( $Item->id() < 1 );
}

sub processItemList {
    my $self = shift;

    foreach my $Item ( @{ $self->{_Items} } ) {
        $self->processItem($Item);
    }

    $self->cleanupItems();
}

###############################################################################

package My::API::Object::Item;
use base qw ( DE_EPAGES::Core::API::Object::BaseObject );

sub id {
    shift->get('ID');
}

###############################################################################

package My::Test;

use Test::More ;
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Core::API::Error qw (Error ExistsError GetError );

use DE_EPAGES::Trigger::API::Object::BatchProcessHandler;


sub Main() {

    local $| = 1;
    Tests();
}

############################################
#                                          #
#             TESTS                        #
#                                          #
############################################

sub Tests {
    my $Handler = My::API::Object::ExampleBatchHandler->new({ 'BatchSize' => 3});
    isa_ok( $Handler, 'DE_EPAGES::Trigger::API::Object::BatchProcessHandler', 'Prove that the handler inherit from the correct package.' );
    isa_ok( $Handler, 'My::API::Object::ExampleBatchHandler', 'Prove implemetation package.' );

    like($Handler->packageName(), qr/My::API::Object::ExampleBatchHandler/, 'Prove that the method returns the package name.');

    is_deeply ($Handler->items(), [], 'Prove that items() returns initally an Empty array.');

    is( $Handler->batchSize(), 3, 'Prove that batchSize() returns the size of the batch.');

    is( $Handler->countItems(), 0, 'Prove that countItems() returns the initally item amount.');

    ok($Handler->processItemList(), 'Prove that processItemList() can be called, evenso noting was added.');


    eval{
        my $Item = My::API::Object::Item->new( 'ID' => 8 );
        $Handler->processItem($Item); #usually called by processItemList()
    };
    ok(!ExistsError, 'Prove that processItem was called without error when processing an item.') or diag (GetError->message);

    my $Count = _addItemsToHandler($Handler);    #add some items via addItem() method, this will trigger processItemList automaticly when there more items than batch size
    my $RestItems = $Count % $Handler->batchSize();

    isa_ok( $Handler, 'My::API::Object::ExampleBatchHandler', 'Prove that handler still exists.' );

    is( $Handler->countItems(), $RestItems, 'Prove that the internal item count, counts correctly. Before' );

    $Handler->processItemList();

    is( $Handler->countItems(), 0, 'Prove that the processItemList clean up afterwarts.' );

    # Tests when the BatchSize is defined and smaller that the Item amount
    $Handler = My::API::Object::ExampleBatchHandler->new( { 'BatchSize' => 3, 'CustomOption' => 'HaHaHa' } );
    isa_ok( $Handler, 'My::API::Object::ExampleBatchHandler', 'Prove that handler still exists.' );

    like( $Handler->customOption(), qr/HaHaHa/, 'getter for custom option' );

    is( $Handler->batchSize(), 3, 'set/read BatchSize' );

    $Count = _addItemsToHandler($Handler); #add some items via addItem() method
    $RestItems = $Count % $Handler->batchSize();

    is( $Handler->countItems(), $RestItems, 'Prove that the internal item count, counts correctly' );

    # Tests when the BatchSize is defined and bigger that the Item amount
    $Handler = My::API::Object::ExampleBatchHandler->new( { 'BatchSize' => 9, 'CustomOption' => 'HeHeHe' } );
    isa_ok( $Handler, 'My::API::Object::ExampleBatchHandler', 'Prove that handler still exists.' );

    like( $Handler->customOption(), qr/HeHeHe/, 'getter for custom option' );

    is( $Handler->batchSize(), 9, 'set/read BatchSize' );

    $Count = _addItemsToHandler($Handler); #add some items via addItem() method

    is( $Handler->countItems(), $Count, 'Prove that the internal item count, counts correctly' );
    done_testing();
}

sub _addItemsToHandler($) {
    my ($Handler) = @_;

    my $count;
    foreach (qw(1337 42 23 13 7 16 52 2)) {
        my $Item = My::API::Object::Item->new( ID => $_ );
        $Handler->addItem($Item);
        $count++;
    }
    return $count;
}

RunScript( Sub => sub { Tests() } );
